@@ -16,15 +16,15 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 from twisted.spread import pb
-from twisted.internet import main
-def gotObject(object):
-    print "got object:",object
-    object.callRemote("echo", "hello network").addCallback(gotEcho)
-def gotEcho(echo):
-    print 'server echoed:',echo
-    main.shutDown()
-def gotNoObject(reason):
-    print "no object:",reason
-    main.shutDown()
-pb.getObjectAt("localhost", 8789, 30).addCallbacks(gotObject, gotNoObject)
-main.run()
+from twisted.internet import reactor
+from twisted.python import util
+
+factory = pb.PBClientFactory()
+reactor.connectTCP("localhost", 8789, factory)
+d = factory.getRootObject()
+d.addCallback(lambda object: object.callRemote("echo", "hello network"))
+d.addCallback(lambda echo: 'server echoed: '+echo)
+d.addErrback(lambda reason: 'error: '+str(reason.value))
+d.addCallback(util.println)
+d.addCallback(lambda _: reactor.stop())
+reactor.run()
