@@ -1,6 +1,6 @@
 #! /usr/bin/python
 
-from twisted.internet.app import Application
+from twisted.application import service, internet
 from twisted.internet import reactor
 from twisted.spread import pb
 from copy_sender import LilyPond, CopyPond
@@ -21,6 +21,6 @@ class Receiver(pb.Root):
     def remote_shutdown(self):
         reactor.stop()
 
-app = Application("copy_receiver")
-app.listenTCP(8800, pb.BrokerFactory(Receiver()))
-app.run(save=0)
+application = service.Application("copy_receiver")
+internet.TCPServer(8800, pb.PBServerFactory(Receiver())).setServiceParent(
+    service.IServiceCollection(application))
