@@ -1,47 +1,91 @@
 #! /usr/bin/python
 
-import weakref
+import weakref, types
 
 from twisted.python import components, failure
 from twisted.internet import defer
 
 import slicer, schema, tokens, banana
 from tokens import BananaError, Violation
-from slicer import UnbananaFailure
+from slicer import UnbananaFailure, BaseUnslicer
 
 class PendingRequest(object):
-    def __init__(self, constraint=None):
+    def __init__(self):
         self.deferred = defer.Deferred()
-        self.constraint = constraint # this constrains the results
+        self.constraint = None # this constrains the results
+    def setConstraint(self, constraint):
+        self.constraint = constraint
 
 class RemoteReference(object):
-    def __init__(self, broker, refID, interfaces=None):
+    def __init__(self, broker, refID, interfaces=None, schema=None):
         self.broker = broker
         self.refID = refID
         self.interfaces = interfaces
+        self.schema = schema
 
     def __del__(self):
         self.broker.freeRemoteReference(self.refID)
 
     def callRemote(self, _name, *args, **kwargs):
-        reqID = self.broker.newRequestID()
+        # for consistency, *all* failures are reported asynchronously.
+        try:
+            # this implements an argument _resultConstraint=None, but keeps
+            # it from interfering with any positional parameters
+            _resultConstraint = kwargs.get("_resultConstraint")
+            if kwargs.has_key("_resultConstraint"):
+                del kwargs["_resultConstraint"]
 
-        # TODO: check args against arg constraint
-        assert not args # TODO: turn positional arguments into kwargs
+            # newRequestID() could fail with a StaleBrokerError
+            reqID = self.broker.newRequestID()
+            req = PendingRequest()
+            self.broker.waitingForAnswers[reqID] = req
 
-        resultConstraint = None # TODO: get return value constraint
-        req = PendingRequest(resultConstraint)
-        self.broker.waitingForAnswers[reqID] = req
+            methodSchema = None
+            if self.schema:
+                # getMethodSchema() could raise KeyError for bad methodnames
+                methodSchema = self.schema.getMethodSchema(_name)
 
-        child = CallSlicer(self.broker)
-        self.broker.slice2(child, (reqID, self.refID, _name, kwargs))
+            if methodSchema:
+                # turn positional arguments into kwargs
+
+                # mapArguments() could fail for bad argument names or
+                # missing required parameters
+                argsdict = methodSchema.mapArguments(args, kwargs)
+                c = methodSchema.getArgsConstraint()
+                if c:
+                    # check args against arg constraint. This could fail if
+                    # any arguments are of the wrong type
+                    c.checkArgs(kwargs)
+                # get return value constraint
+                req.setConstraint(methodSchema.getResponseConstraint())
+            else:
+                assert not args
+                argsdict = kwargs
+
+            if _resultConstraint:
+                req.setConstraint(_resultConstraint) # overrides schema
+
+            child = CallSlicer(self.broker)
+            # this could fail if any of the arguments (or their children)
+            # are unsliceable
+            self.broker.slice2(child, (reqID, self.refID, _name, argsdict))
+        except:
+            req.deferred.errback(failure.Failure())
+
+        # the remote end could send back an error response for many reasons:
+        #  bad method name
+        #  bad argument types (violated their schema)
+        #  exception during method execution
+        #  method result violated the results schema
+        # something else could occur to cause an errback:
+        #  connection lost before response completely received
+        #  exception during deserialization of the response
+        #   [but only if it occurs after the reqID is received]
+        #  method result violated our results schema
+        # if none of those occurred, the callback will be run
 
         return req.deferred
 
-class BaseUnslicer(slicer.BaseUnslicer):
-    def __init__(self, broker):
-        self.broker = broker
-
 class ReferenceUnslicer(BaseUnslicer):
     refID = None
     interfaces = None
@@ -61,9 +105,9 @@ class ReferenceUnslicer(BaseUnslicer):
     def doOpen(self, opentype):
         # only for the interface list
         self.ilistConstraint.checkOpentype(opentype)
-        unslicer = self.opener(opentype)
-        unslicer.opener = self.opener
-        unslicer.setConstraint(self.ilistConstraint)
+        unslicer = self.open(opentype)
+        if unslicer:
+            unslicer.setConstraint(self.ilistConstraint)
         return unslicer
 
     def receiveChild(self, token):
@@ -146,10 +190,10 @@ class CallUnslicer(BaseUnslicer):
         # we don't have to bother checking self.stage or self.argname
         if self.argConstraint:
             self.argConstraint.checkOpentype(opentype)
-        unslicer = self.opener(opentype)
-        unslicer.opener = self.opener
-        if self.argConstraint:
-            unslicer.setConstraint(self.argConstraint)
+        unslicer = self.open(opentype)
+        if unslicer:
+            if self.argConstraint:
+                unslicer.setConstraint(self.argConstraint)
         return unslicer
 
     def receiveChild(self, token):
@@ -236,10 +280,10 @@ class AnswerUnslicer(BaseUnslicer):
     def doOpen(self, opentype):
         if self.resultConstraint:
             self.resultConstraint.checkOpentype(opentype)
-        unslicer = self.opener(opentype)
-        unslicer.opener = self.opener
-        if self.resultConstraint:
-            unslicer.setConstraint(self.resultConstraint)
+        unslicer = self.open(opentype)
+        if unslicer:
+            if self.resultConstraint:
+                unslicer.setConstraint(self.resultConstraint)
         return unslicer
 
     def receiveChild(self, token):
@@ -276,9 +320,9 @@ class ErrorUnslicer(BaseUnslicer):
 
     def doOpen(self, opentype):
         self.fConstraint.checkOpentype(opentype)
-        unslicer = self.opener(opentype)
-        unslicer.opener = self.opener
-        unslicer.setConstraint(self.fConstraint)
+        unslicer = self.open(opentype)
+        if unslicer:
+            unslicer.setConstraint(self.fConstraint)
         return unslicer
 
     def receiveChild(self, token):
@@ -303,13 +347,14 @@ class ErrorUnslicer(BaseUnslicer):
 
 
 class PBRootUnslicer(slicer.RootUnslicer):
-    # topRegistry defines what objects are allowed at the top-level
+    # topRegistry defines what objects are allowed at the top-level. All of
+    # these accept a Broker in their __init__ call
     topRegistry = {
-        "remote": ReferenceUnslicer,
-        "decref" : DecRefUnslicer,
-        "call": CallUnslicer,
-        "answer": AnswerUnslicer,
-        "error": ErrorUnslicer,
+        ("remote",): ReferenceUnslicer,
+        ("decref",): DecRefUnslicer,
+        ("call",): CallUnslicer,
+        ("answer",): AnswerUnslicer,
+        ("error",): ErrorUnslicer,
         }
     # openRegistry defines what objects are allowed at the second level and
     # below
@@ -319,21 +364,10 @@ class PBRootUnslicer(slicer.RootUnslicer):
         if typebyte != tokens.OPEN:
             raise BananaError("top-level must be OPEN")
 
-    def doOpen(self, opentype):
-        # TODO: refactor this with RootUnslicer.doOpen
-        if len(self.protocol.receiveStack) == 1 and opentype == "vocab":
-            # only legal at top-level
-            child = VocabUnslicer()
-        else:
-            try:
-                child = self.topRegistry[opentype](self.broker)
-            except KeyError:
-                raise BananaError("unknown OPEN type '%s'" % opentype,
-                                  self.where() + ".<OPEN(%s)>" % opentype)
-        if not child:
-            # TODO: Violation? or should we just drop the connection?
-            raise KeyError, "no such open type '%s'" % opentype
-        child.opener = self.open
+    def openTop(self, opentype):
+        child = self.open(opentype, self.topRegistry)
+        if child:
+            child.broker = self.broker
         return child
 
     def receiveChild(self, obj):
@@ -404,8 +438,19 @@ class CallSlicer(BaseSlicer):
             self.send(argname)
             self.send(args[argname])
 
+PBSlicerRegistry = {}
+PBSlicerRegistry.update(slicer.BaseSlicerRegistry)
+del PBSlicerRegistry[types.InstanceType]
+
 class PBRootSlicer(slicer.RootSlicer):
-    pass
+    SlicerRegistry = PBSlicerRegistry
+
+    def slicerFactoryForObject(self, obj):
+        if isinstance(obj, Referenceable):
+            return ReferenceableSlicer
+        if isinstance(obj, Copyable):
+            return CopyableSlicer
+        return slicer.RootSlicer.slicerFactoryForObject(self, obj)
 
 
 class Broker(banana.Banana):
@@ -536,6 +581,7 @@ class Broker(banana.Banana):
             child = DecRefSlicer(self)
             self.slice2(child, refID)
         except:
+            print "failure during freeRemoteReference"
             f = failure.Failure()
-            f.printTraceback()
+            print f.getTraceback()
             raise
