@@ -101,13 +101,15 @@ class Constraint:
                        FLOAT: None,
                        }
     def checkOpentype(self, opentype):
-        """Check the OPEN token type. Raise an exception if it is not
-        accepted.
+        """Check the OPEN type (the tuple of Index Tokens). Raise an
+        exception if it is not accepted.
         """
         if self.opentypes == None:
             return
         if opentype not in self.opentypes:
-            raise Violation, "unacceptable OPEN type"
+            print "opentype %s, self.opentypes %s" % (opentype,
+                                                      self.opentypes)
+            raise Violation, "unacceptable OPEN type '%s'" % (opentype,)
 
     def checkObject(self, obj):
         """Validate an existing object. Usually objects are validated as
@@ -258,7 +260,7 @@ def OPENBYTES(dummy):
 
 class BooleanConstraint(Constraint):
     taster = openTaster
-    opentypes = ["boolean"]
+    opentypes = [("boolean",)]
     _myint = IntegerConstraint()
 
     def __init__(self, value=None):
@@ -282,8 +284,16 @@ class BooleanConstraint(Constraint):
         return 1+self._myint.maxDepth(seen)
 
 class InterfaceConstraint(Constraint):
+    """This constraint accepts any instance which implements the given
+    Interface. The object may be a RemoteCopy if the classname they provide
+    maps to a local class which implements the given interface, or it may be
+    a RemoteReference if they claim the backing object implements the
+    interface.
+    """
+    # TODO: do we need an string-to-Interface map just like we have a
+    # classname-to-class/factory map?
     taster = openTaster
-    opentypes = ["instance"]
+    opentypes = [("instance",)]
 
     def __init__(self, interface):
         self.interface = interface
@@ -294,7 +304,7 @@ class InterfaceConstraint(Constraint):
 
 class ClassConstraint(Constraint):
     taster = openTaster
-    opentypes = ["instance"]
+    opentypes = [("instance",)]
 
     def __init__(self, klass):
         self.klass = klass
@@ -338,7 +348,7 @@ ChoiceOf = PolyConstraint
 
 class TupleConstraint(Constraint):
     taster = openTaster
-    opentypes = ["tuple"]
+    opentypes = [("tuple",)]
 
     def __init__(self, *elemConstraints):
         self.constraints = [makeConstraint(e) for e in elemConstraints]
@@ -373,7 +383,7 @@ class ListConstraint(Constraint):
     constraint."""
 
     taster = openTaster
-    opentypes = ["list"]
+    opentypes = [("list",)]
 
     def __init__(self, constraint, maxLength=30):
         self.constraint = makeConstraint(constraint)
@@ -405,7 +415,7 @@ ListOf = ListConstraint
 
 class DictConstraint(Constraint):
     taster = openTaster
-    opentypes = ["dict"]
+    opentypes = [("dict",)]
 
     def __init__(self, keyConstraint, valueConstraint, maxKeys=30):
         self.keyConstraint = makeConstraint(keyConstraint)
@@ -451,7 +461,7 @@ class AttributeDictConstraint(Constraint):
     Some special constraints are legal here: Optional.
     """
     taster = openTaster
-    opentypes = ["attrdict"]
+    opentypes = [("attrdict",)]
 
     def __init__(self, *attrTuples, **kwargs):
         self.ignoreUnknown = kwargs.get('ignoreUnknown', False)
@@ -537,7 +547,7 @@ class MethodArgumentsConstraint(Constraint):
         # build up a list of required arguments for later checking
         self.args = {}
         self.required = []
-        for argname, constraint in self.args:
+        for argname, constraint in kwargs.items():
             if not isinstance(constraint, Optional):
                 self.required.append(argname)
             self.args[argname] = makeConstraint(constraint)
@@ -587,6 +597,51 @@ class MethodArgumentsConstraint(Constraint):
         # TODO: implement the rest of maxSize, just like a dictionary
         raise NotImplementedError
 
+class RemoteMethodSchema:
+    # under development
+    def __init__(self):
+        self.argumentNames = []
+        self.argsConstraint = None
+        self.responseConstraint = None
+        self.options = {} # return, wait, reliable, etc
+
+    def mapArguments(self, args, kwargs):
+        """Create a dictionary of arguments. All positional arguments must
+        be turned into keyword ones. All default arguments should be filled
+        in (?).
+        """
+        # python probably provides a utility function for this
+
+        # TODO: this does not really work. Fix it.
+
+        # TODO: this would also be a good place to implement the
+        # schema-driven Copyable vs Referenceable decisions
+        for i in range(len(args)):
+            name = self.argumentNames[i]
+            if kwargs.has_key(name):
+                raise TypeError(
+                    "got multiple values for keyword argument '%s'" % name)
+            kwargs[name] = args[i]
+        return kwargs
+
+    def getArgsConstraint(self):
+        # return a MethodArgumentsConstraint
+        return self.argsConstraint
+
+    def getResponseConstraint(self):
+        return self.responseConstraint
+
+class RemoteReferenceSchema:
+    # under development
+    def __init__(self):
+        self.methods = {} # values are RemoteMethodSchema instances
+    def getMethods(self):
+        return self.methods.keys()
+    def getMethodSchema(self, methodname):
+        return self.methods[methodname]
+    
+
+
 #TODO
 class Shared(Constraint):
     def __init__(self, constraint, refLimit=None):
