@@ -14,13 +14,14 @@
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-import struct, array, pwd, os, stat, time, errno
+
+import struct, errno
 
 from twisted.internet import defer, protocol
-from twisted.python import log
+from twisted.python import failure, log
 
 from common import NS, getNS
-
+from twisted.conch.interfaces import ISFTPServer, ISFTPFile
 
 class FileTransferBase(protocol.Protocol):
 
@@ -30,7 +31,7 @@ class FileTransferBase(protocol.Protocol):
 
     def __init__(self):
         self.buf = ''
-        otherVersion = None # this gets set
+        self.otherVersion = None # this gets set
 
     def sendPacket(self, kind, data):
         self.transport.write(struct.pack('!LB', len(data)+1, kind) + data)
@@ -58,6 +59,8 @@ class FileTransferBase(protocol.Protocol):
             try:
                 f(data)
             except Exception, e:
+                log.err(e)
+                continue
                 reqId = struct.unpack('!L', data[:4])[0]
                 self._ebStatus(failure.Failure(e), reqId)
 
@@ -121,9 +124,9 @@ class FileTransferBase(protocol.Protocol):
 
 class FileTransferServer(FileTransferBase):
 
-    def __init__(self, avatar):
+    def __init__(self, data=None, avatar=None):
         FileTransferBase.__init__(self)
-        self.avatar = avatar
+        self.client = ISFTPServer(avatar) # yay interfaces
         self.openFiles = {}
         self.openDirs = {}
 
@@ -136,7 +139,7 @@ class FileTransferServer(FileTransferBase):
             ext_name, data = getNS(data)
             ext_data, data = getNS(data)
             ext[ext_name] = ext_data
-        our_ext = self.gotVersion(version, ext)
+        our_ext = self.client.gotVersion(version, ext)
         our_ext_data = ""
         for (k,v) in our_ext.items():
             our_ext_data += NS(k) + NS(v)
@@ -149,12 +152,12 @@ class FileTransferServer(FileTransferBase):
         flags, data = struct.unpack('!L', data[:4])[0], data[4:]
         attrs, data = self._parseAttributes(data)
         assert data == '', 'still have data in OPEN: %s' % repr(data)
-        d = defer.maybeDeferred(self.openFile, filename, flags, attrs)
+        d = defer.maybeDeferred(self.client.openFile, filename, flags, attrs)
         d.addCallback(self._cbOpenFile, requestId)
         d.addErrback(self._ebStatus, requestId, "open failed")
 
     def _cbOpenFile(self, fileObj, requestId):
-        fileId = str(id(fileObj))
+        fileId = str(hash(fileObj))
         if fileId in self.openFiles:
             raise KeyError, 'id already open'
         self.openFiles[fileId] = fileObj
@@ -166,12 +169,12 @@ class FileTransferServer(FileTransferBase):
         assert data == '', 'still have data in CLOSE: %s' % repr(data)
         if handle in self.openFiles:
             fileObj = self.openFiles[handle]
-            d = defer.maybeDeferred(self.closeFile, fileObj)
+            d = defer.maybeDeferred(fileObj.close)
             d.addCallback(self._cbClose, handle, requestId)
             d.addErrback(self._ebStatus, requestId, "close failed")
         elif handle in self.openDirs:
-            dirObj = self.openDirs[handle]
-            d = defer.maybeDeferred(self.closeDirectory, dirObj)
+            dirObj = self.openDirs[handle][0]
+            d = defer.maybeDeferred(dirObj.close)
             d.addCallback(self._cbClose, handle, requestId, 1)
             d.addErrback(self._ebStatus, requestId, "close failed")
         else:
@@ -193,7 +196,7 @@ class FileTransferServer(FileTransferBase):
             self._ebRead(failure.Failure(KeyError()), requestId)
         else:
             fileObj = self.openFiles[handle]
-            d = defer.maybeDeferred(self.readFile, fileObj, offset, length)
+            d = defer.maybeDeferred(fileObj.readChunk, offset, length)
             d.addCallback(self._cbRead, requestId)
             d.addErrback(self._ebStatus, requestId, "read failed")
 
@@ -212,7 +215,7 @@ class FileTransferServer(FileTransferBase):
             self._ebWrite(failure.Failure(KeyError()), requestId)
         else:
             fileObj = self.openFiles[handle]
-            d = defer.maybeDeferred(self.writeFile, fileObj, offset, writeData)
+            d = defer.maybeDeferred(fileObj.writeChunk, offset, writeData)
             d.addCallback(self._cbStatus, requestId, "write succeeded")
             d.addErrback(self._ebStatus, requestId, "write failed")
 
@@ -220,7 +223,7 @@ class FileTransferServer(FileTransferBase):
         requestId, data = struct.unpack('!L', data[:4])[0], data[4:]
         filename, data = getNS(data)
         assert data == '', 'still have data in REMOVE: %s' % repr(data)
-        d = defer.maybeDeferred(self.removeFile, filename)
+        d = defer.maybeDeferred(self.client.removeFile, filename)
         d.addCallback(self._cbStatus, requestId, "remove succeeded")
         d.addErrback(self._ebStatus, requestId, "remove failed")
 
@@ -229,7 +232,7 @@ class FileTransferServer(FileTransferBase):
         oldPath, data = getNS(data)
         newPath, data = getNS(data)
         assert data == '', 'still have data in RENAME: %s' % repr(data)
-        d = defer.maybeDeferred(self.renameFile, oldPath, newPath)
+        d = defer.maybeDeferred(self.client.renameFile, oldPath, newPath)
         d.addCallback(self._cbStatus, requestId, "rename succeeded")
         d.addErrback(self._ebStatus, requestId, "rename failed")
 
@@ -238,7 +241,7 @@ class FileTransferServer(FileTransferBase):
         path, data = getNS(data)
         attrs, data = self._parseAttributes(data)
         assert data == '', 'still have data in MKDIR: %s' % repr(data)
-        d = defer.maybeDeferred(self.makeDirectory, path, attrs)
+        d = defer.maybeDeferred(self.client.makeDirectory, path, attrs)
         d.addCallback(self._cbStatus, requestId, "mkdir succeeded")
         d.addErrback(self._ebStatus, requestId, "mkdir failed")
 
@@ -246,7 +249,7 @@ class FileTransferServer(FileTransferBase):
         requestId, data = struct.unpack('!L', data[:4])[0], data[4:]
         path, data = getNS(data)
         assert data == '', 'still have data in RMDIR: %s' % repr(data)
-        d = defer.maybeDeferred(self.removeDirectory, path)
+        d = defer.maybeDeferred(self.client.removeDirectory, path)
         d.addCallback(self._cbStatus, requestId, "rmdir succeeded")
         d.addErrback(self._ebStatus, requestId, "rmdir failed")
 
@@ -254,15 +257,15 @@ class FileTransferServer(FileTransferBase):
         requestId, data = struct.unpack('!L', data[:4])[0], data[4:]
         path, data = getNS(data)
         assert data == '', 'still have data in OPENDIR: %s' % repr(data)
-        d = defer.maybeDeferred(self.openDirectory, path)
+        d = defer.maybeDeferred(self.client.openDirectory, path)
         d.addCallback(self._cbOpenDirectory, requestId)
         d.addErrback(self._ebStatus, requestId, "opendir failed")
 
     def _cbOpenDirectory(self, dirObj, requestId):
-        handle = str(id(dirObj))
+        handle = str(hash(dirObj))
         if handle in self.openDirs:
             raise KeyError, "already opened this directory"
-        self.openDirs[handle] = dirObj
+        self.openDirs[handle] = [dirObj, iter(dirObj)]
         self.sendPacket(FXP_HANDLE, struct.pack('!L', requestId) + NS(handle))
 
     def packet_READDIR(self, data):
@@ -272,12 +275,31 @@ class FileTransferServer(FileTransferBase):
         if handle not in self.openDirs:
             self._ebStatus(failure.Failure(KeyError()), requestId)
         else:
-            dirObj = self.openDirs[handle]
-            d = defer.maybeDeferred(self.scanDirectory, dirObj)
-            d.addCallback(self._cbScanDirectory, requestId)
+            dirObj, dirIter = self.openDirs[handle]
+            d = defer.maybeDeferred(self._scanDirectory, dirIter, [])
+            d.addCallback(self._cbSendDirectory, requestId)
             d.addErrback(self._ebStatus, requestId, "scan directory failed")
 
-    def _cbScanDirectory(self, result, requestId):
+    def _scanDirectory(self, dirIter, f):
+        while len(f) < 250:
+            try:
+                info = dirIter.next()
+            except StopIteration:
+                if not f:
+                    raise EOFError
+                return f
+            if isinstance(info, defer.Deferred):
+                d.addCallback(self._cbScanDirectory, dirIter, f)
+                return
+            else:
+                f.append(info)
+        return f
+
+    def _cbScanDirectory(self, result, dirIter, f):
+        f.append(result)
+        return self._scanDirectory(dirIter, f)
+
+    def _cbSendDirectory(self, result, requestId):
         data = ''
         for (filename, longname, attrs) in result:
             data += NS(filename)
@@ -290,7 +312,7 @@ class FileTransferServer(FileTransferBase):
         requestId, data = struct.unpack('!L', data[:4])[0], data[4:]
         path, data = getNS(data)
         assert data == '', 'still have data in STAT/LSTAT: %s' % repr(data)
-        d = defer.maybeDeferred(self.getAttrs, path, followLinks)
+        d = defer.maybeDeferred(self.client.getAttrs, path, followLinks)
         d.addCallback(self._cbStat, requestId)
         d.addErrback(self._ebStatus, requestId, 'stat/lstat failed')
 
@@ -304,8 +326,8 @@ class FileTransferServer(FileTransferBase):
         if handle not in self.openDirs:
             self._ebStatus(failure.Failure(KeyError()), requestId)
         else:
-            dirObj = self.openDirs[handle]
-            d = defer.maybeDeferred(self.getAttrsOpaque, dirObj)
+            fileObj = self.openFiles[handle]
+            d = defer.maybeDeferred(fileObj.getAttrs)
             d.addCallback(self._cbStat, requestId)
             d.addErrback(self._ebStatus, requestId, 'fstat failed')
 
@@ -318,7 +340,7 @@ class FileTransferServer(FileTransferBase):
         path, data = getNS(data)
         attrs, data = self._parseAttributes(data)
         assert data == '', 'still have data in SETSTAT: %s' % repr(data)
-        d = defer.maybeDeferred(self.setAttrs, path, attrs)
+        d = defer.maybeDeferred(self.client.setAttrs, path, attrs)
         d.addCallback(self._cbStatus, requestId, 'setstat succeeded')
         d.addErrback(self._ebStatus, requestId, 'setstat failed')
 
@@ -331,7 +353,7 @@ class FileTransferServer(FileTransferBase):
             self._ebStatus(failure.Failure(KeyError()), requestId)
         else:
             fileObj = self.openFiles[handle]
-            d = defer.maybeDeferred(self.setAttrsOpaque, fileObj, attrs)
+            d = defer.maybeDeferred(fileObj.setAttrs, attrs)
             d.addCallback(self._cbStatus, requestId, 'fsetstat succeeded')
             d.addErrback(self._ebStatus, requestId, 'fsetstat failed')
 
@@ -339,18 +361,18 @@ class FileTransferServer(FileTransferBase):
         requestId, data = struct.unpack('!L', data[:4])[0], data[4:]
         path, data = getNS(data)
         assert data == '', 'still have data in READLINK: %s' % repr(data)
-        d = defer.maybeDeferred(self.readLink, path)
+        d = defer.maybeDeferred(self.client.readLink, path)
         d.addCallback(self._cbReadLink, requestId)
         d.addErrback(self._ebStatus, requestId, 'readlink failed')
 
     def _cbReadLink(self, result, requestId):
-        self._cbScanDirectory([(result, '', {})], requestId)
+        self._cbSendDirectory([(result, '', {})], requestId)
 
     def packet_SYMLINK(self, data):
         requestId, data = struct.unpack('!L', data[:4])[0], data[4:]
         linkPath, data = getNS(data)
         targetPath, data = getNS(data)
-        d = defer.maybeDeferred(self.makeLink, linkPath, targetPath)
+        d = defer.maybeDeferred(self.client.makeLink, linkPath, targetPath)
         d.addCallback(self._cbStatus, requestId, 'symlink succeeded')
         d.addErrback(self._ebStatus, requestId, 'symlink failed')
 
@@ -358,7 +380,7 @@ class FileTransferServer(FileTransferBase):
         requestId, data = struct.unpack('!L', data[:4])[0], data[4:]
         path, data = getNS(data)
         assert data == '', 'still have data in REALPATH: %s' % repr(data)
-        d = defer.maybeDeferred(self.realPath, path)
+        d = defer.maybeDeferred(self.client.realPath, path)
         d.addCallback(self._cbReadLink, requestId) # same return format
         d.addErrback(self._ebStatus, requestId, 'realpath failed')
 
@@ -397,77 +419,42 @@ class FileTransferServer(FileTransferBase):
         data += NS(lang)
         self.sendPacket(FXP_STATUS, data)
 
-    def _runAsUser(self, f, *args):
-        euid = os.geteuid()
-        egid = os.getegid()
-        uid, gid = self.avatar.getUserGroupId()
-        os.setegid(0)
-        os.seteuid(0)
-        os.setegid(gid)
-        os.seteuid(uid)
-        try:
-            if not hasattr(f,'__iter__'):
-                f = [(f, ) + args]
-            for i in f:
-                r = i[0](*i[1:])
-        except:
-            os.setegid(0)
-            os.seteuid(0)
-            os.setegid(egid)
-            os.seteuid(euid)
-            raise
-        else:
-            os.setegid(0)
-            os.seteuid(0)
-            os.setegid(egid)
-            os.seteuid(euid)
-            return r
+class FileTransferClient(FileTransferBase):
 
-    def _setAttrs(self, path, attrs):
+    def __init__(self, extData = {}):
         """
-        NOTE: this function assumes it is runner as the logged-in user:
-        i.e. under _runAsUser()
+        extData is a dict of extended_name : extended_data items
+        to be sent to the server.
         """
-        if attrs.has_key("uid") and attrs.has_key("gid"):
-            os.lchown(path, attrs["uid"], attrs["gid"])
-        if attrs.has_key("permissions"):
-            os.chmod(path, attrs["permissions"])
-        if attrs.has_key("atime") and attrs.has_key("mtime"):
-            os.utime(path, (attrs["atime"]. attrs["mtime"]))
+        FileTransferBase.__init__(self)
+        self.extData = {}
+        self.counter = 0
+        self.openRequests = {} # id -> Deferred
+        self.wasAFile = {} # Deferred -> 1 TERRIBLE HACK
 
-    def _getAttrs(self, s):
-        return {
-            "size" : s.st_size,
-            "uid" : s.st_uid,
-            "gid" : s.st_gid,
-            "permissions" : s.st_mode,
-            "atime" : s.st_atime,
-            "mtime" : s.st_mtime
-        }
+    def connectionMade(self):
+        data = struct.pack('!L', max(self.versions))
+        for k,v in self.extData.itervalues():
+            data += NS(k) + NS(v)
+        self.sendPacket(FXP_INIT, data)
 
-    def _absPath(self, path):
-        uid, gid = self.avatar.getUserGroupId()
-        home = pwd.getpwuid(uid)[5]
-        return os.path.abspath(os.path.join(home, path))
+    def _sendRequest(self, msg, data):
+        data = struct.pack('!L', self.counter) + data
+        d = defer.Deferred()
+        self.openRequests[self.counter] = d
+        self.counter += 1
+        self.sendPacket(msg, data)
+        return d
 
-    def gotVersion(self, otherVersion, extData):
-        """
-        Called when the client sends their version info.
-
-        otherVersion is an integer representing the version of the SFTP
-        protocol they are claiming.
-        extData is a dictionary of extended_name : extended_data items.
-        These items are sent by the client to indicate additional features.
-
-        This method should return a dictionary of extended_name : extended_data
-        items.  These items are the additional features (if any) supported
-        by the server.
-        """
-        return {}
+    def _parseRequest(self, data):
+        (id,) = struct.unpack('!L', data[:4])
+        d = self.openRequests[id]
+        del self.openRequests[id]
+        return d, data[4:]
 
     def openFile(self, filename, flags, attrs):
         """
-        Called when the clients asks to open a file.
+        Open a file.
 
         filename is a string representing the file to open.
 
@@ -482,87 +469,20 @@ class FileTransferServer(FileTransferBase):
             permissions: the permissions of the file with as an integer.
             the bit representation of this field is defined by POSIX.
             atime: the access time of the file as seconds since the epoch.
-            mtime: modification time of the file as seconds since the epoch.
+            mtime: the modification time of the file as seconds since the epoch.
             ext_*: extended attributes.  The server is not required to
             understand this, but it may.
 
         NOTE: there is no way to indicate text or binary files.  it is up
         to the SFTP client to deal with this.
 
-        This method returns an opaque identifier for this file.  This
-        identifier is passed as the first argument to the other file
-        access methods.  Alternatively, it can return a Deferred that will
-        be called back with the identifier.
-
-        The only requirement on the opaque identifier is that it is usable
-        as the value in a directory.  Opening the same file twice should not
-        return the same opaque identifer, and the identifier should last
-        as long as it is open, or until the end of the client connection.
+        This method returns an Deferred that is called back with an object
+        that meets the ISFTPFile interface.
         """
-        openFlags = 0
-        if flags & FXF_READ == FXF_READ and flags & FXF_WRITE == 0:
-            openFlags = os.O_RDONLY
-        if flags & FXF_WRITE == FXF_WRITE and flags & FXF_READ == 0:
-            openFlags = os.O_WRONLY
-        if flags & FXF_WRITE == FXF_WRITE and flags & FXF_READ == FXF_READ:
-            openFlags = os.O_RDWR
-        if flags & FXF_APPEND == FXF_APPEND:
-            openFlags |= os.O_APPEND
-        if flags & FXF_CREAT == FXF_CREAT:
-            openFlags |= os.O_CREAT
-        if flags & FXF_TRUNC == FXF_TRUNC:
-            openFlags |= os.O_TRUNC
-        if flags & FXF_EXCL == FXF_EXCL:
-            openFlags |= os.O_EXCL
-        if attrs.has_key("permissions"):
-            mode = attrs["permissions"]
-            del attrs["permissions"]
-        else:
-            mode = 0777
-        filename = self._absPath(filename)
-        fd = self._runAsUser(os.open, filename, openFlags, mode)
-        if attrs:
-            self._runAsUser(self._setAttrs, filename, attrs)
-        return fd
-
-    def closeFile(self, opaqueId):
-        """
-        Close the file represented by the opaqueId.
-
-        This method returns nothing if the close succeeds immediatly, or a
-        Deferred that is called back when the close succeeds.
-        """
-        return self._runAsUser(os.close, opaqueId)
-
-    def readFile(self, opaqueId, offset, length):
-        """
-        Read from the file represented by the opaqueId.
-
-        offset is an integer that is the index to start from in the file.
-        length is the maximum length of data to return.  The actual amount
-        returned may less than this.  For normal disk files, however,
-        this should read the requested number (up to the end of the file).
-
-        If EOF is reached before any data is read, raise EOFError.
-
-        This method returns the data as a string, or a Deferred that is
-        called back with same.
-        """
-        return self._runAsUser([(os.lseek, opaqueId, offset, 0),
-                                (os.read, opaqueId, length)])
-
-    def writeFile(self, opaqueId, offset, data):
-        """
-        Write to the file represented by the opaqueId
-
-        offset is an integer that is the index to start from in the file.
-        data is a string that is the data to write.
-
-        This method returns when the write completes, or a Deferred that is
-        called when it completes.
-        """
-        return self._runAsUser([(os.lseek, opaqueId, offset, 0),
-                                (os.write, opaqueId, data)])
+        data = NS(filename) + struct.pack('!L', flags) + self._packAttrs(attrs)
+        d = self._sendRequest(FXP_OPEN, data)
+        self.wasAFile[d] = 1 # HACK
+        return d
 
     def removeFile(self, filename):
         """
@@ -570,11 +490,9 @@ class FileTransferServer(FileTransferBase):
 
         filename is the name of the file as a string.
 
-        This method returns when the remove succeeds, or a Deferred that is
-        called back when it succeeds.
+        This method returns a Deferred that is called back when it succeeds.
         """
-        filename = self._absPath(filename)
-        return self._runAsUser(os.remove, filename)
+        return self._sendRequest(FXP_REMOVE, NS(filename))
 
     def renameFile(self, oldpath, newpath):
         """
@@ -583,12 +501,9 @@ class FileTransferServer(FileTransferBase):
         oldpath is the current location of the file.
         newpath is the new file name.
 
-        This method returns when the rename succeeds, or a Deferred that is
-        called back when it succeeds.
+        This method returns a Deferred that is called back when it succeeds.
         """
-        oldpath = self._absPath(oldpath)
-        newpath = self._absPath(newpath)
-        return self._runAsUser(os.rename, oldpath, newpath)
+        return self._sendRequest(FXP_RENAME, NS(oldpath)+NS(newpath))
 
     def makeDirectory(self, path, attrs):
         """
@@ -598,12 +513,10 @@ class FileTransferServer(FileTransferBase):
         attrs is a dictionary of attributes to create the directory with.
         It's meaning is the same as the attrs in the openFile method.
 
-        This method returns when the directory is created, or a Deferred that
-        is called back when it is created.
+        This method returns a Deferred that is called back when it is 
+        created.
         """
-        path = self._absPath(path)
-        return self._runAsUser([(os.mkdir, path),
-                                (self._setAttrs, path, attrs)])
+        return self._sendRequest(FXP_MKDIR, NS(path)+self._packAttrs(attrs))
 
     def removeDirectory(self, path):
         """
@@ -614,11 +527,9 @@ class FileTransferServer(FileTransferBase):
         It is an error to remove a directory that has files or directories in
         it.
 
-        This method returns when the directory is removed, or a Deferred that
-        is called back when it is removed.
+        This method returns a Deferred that is called back when it is removed.
         """
-        path = self._absPath(path)
-        self._runAsUser(os.rmdir, path)
+        return self._sendRequest(FXP_RMDIR, NS(path))
 
     def openDirectory(self, path):
         """
@@ -626,22 +537,16 @@ class FileTransferServer(FileTransferBase):
 
         path is the directory to open.
 
-        This method returns an opaqueId that is passed as the first argument
-        to scanDirectory, or a Deferred that is called back with same.
-        """
-        return [self._absPath(path)]
+        This method returns a Deferred that is called back with an iterable 
+        object that has a close() method,
 
-    def scanDirectory(self, opaqueId):
-        """
-        Scan the directory represented by the opaqueId.
+        The close() method is called when the client is finished reading
+        from the directory.  At this point, the iterable will no longer
+        be used.
 
-        This method returns a sequence of sequences of (filename, longname,
-        attrs) or a Deferred of same.  This does not need to be all the files
-        in the directory, but calls to this method should not return files
-        returned by previous calls.  The container sequence must support
-        __len__ and be iterable, but otherwise may be any other "sequence-like"
-        object.  The interior sequences must support __getitem__, but
-        otherwise may be any "sequence-like" object.
+        The iterable returns triples of the form (filename, longname, attrs)
+        or a Deferred that returns the same.  The sequence must support 
+        __getitem__, but otherwise may be any 'sequence-like' object.
 
         filename is the name of the file relative to the directory.
         logname is an expanded format of the filename.  The recommended format
@@ -654,50 +559,8 @@ class FileTransferServer(FileTransferBase):
         size in bytes, modification time.
 
         attrs is a dictionary in the format of the attrs argument to openFile.
-
-        NOTE:  the way this works is the client sends a request,
-        SSH_FXP_READDIR.  This method is called once for each request, and
-        returns the files that this method returns.  The client keeps
-        sending requests, and this method is called, until this method
-        sends an exception (typically EOFError) to indicate that there are no
-        more files in that directory.  Then, the client knows to stop sending
-        requests.
-
-        No more than 250 files should be returned at once, as some clients
-        cannot handle packets that large.
         """
-        if len(opaqueId) > 2: raise EOFError
-        path = opaqueId[0]
-        if len(opaqueId) == 2:
-            start = opaqueId[1]
-        else:
-            start = 0
-        files = []
-        d = self._runAsUser(os.listdir, path)[start:]
-        for f in d[:250]:
-            s = self._runAsUser(os.lstat, os.path.join(path, f))
-            longname = _lsLine(f, s)
-            attrs = self._getAttrs(s)
-            files.append((f, longname, attrs))
-        if len(d) > 250:
-            start += 250
-            if len(opaqueId) == 2:
-                opaqueId[1] = start
-            else:
-                opaqueId.append(start)
-        else:
-            opaqueId.append(1)
-            opaqueId.append(1)
-        return files
-
-    def closeDirectory(self, opaqueId):
-        """
-        Close the directory referenced by the opaqueId.
-
-        This method returns when the directory is closed, or a Deferred that
-        is called back when it is closed.
-        """
-        return
+        return self._sendRequest(FXP_OPENDIR, NS(path))
 
     def getAttrs(self, path, followLinks):
         """
@@ -711,48 +574,23 @@ class FileTransferServer(FileTransferBase):
         This method returns a dictionary in the same format as the attrs
         argument to openFile or a Deferred that is called back with same.
         """
-        path = self._absPath(path)
-        if followLinks:
-            s = self._runAsUser(os.stat, path)
-        else:
-            s = self._runAsUser(os.lstat, path)
-        return self._getAttrs(s)
-
-    def getAttrsOpaque(self, opaqueId):
-        """
-        Return the attributes for the given opaqueId.
-
-        This method returns a dictionary in the same format as the attrs
-        argument to openFile or a Deferred that is called back with same.
-        """
-        s = self._runAsUser(os.fstat, opaqueId)
-        return self._getAttrs(s)
+        if followLinks: m = FXP_STAT
+        else: m = FXP_LSTAT
+        return self._sendRequest(m, NS(path))
 
     def setAttrs(self, path, attrs):
         """
         Set the attributes for the path.
 
         path is the path to set attributes for as a string.
-        attrs is a dictionary in the same format as the attrs argument to
+        attrs is a idctionary in the same format as the attrs argument to
         openFile.
 
         This method returns when the attributes are set or a Deferred that is
         called back when they are.
         """
-        path = self._absPath(path)
-        self._runAsUser(self._setAttrs, path, attrs)
-
-    def setAttrsOpaque(self, opaqueId, attrs):
-        """
-        Set the attributes for the opaqueId.
-
-        attrs is a dictionary in the same format as the attrs argument to
-        openFile.
-
-        This method returns when the attributes are set or a Deferred that is
-        called back when they are.
-        """
-        raise NotImplementedError
+        data = NS(path) + self._packAttributes(attrs)
+        return self._sendRequest(FXP_SETSTAT, data)
 
     def readLink(self, path):
         """
@@ -763,8 +601,7 @@ class FileTransferServer(FileTransferBase):
         This method returns the target of the link, or a Deferred that
         returns the same.
         """
-        path = self._absPath(path)
-        return self._runAsUser(os.readlink, path)
+        return self._sendRequest(FXP_READLINK, NS(path))
 
     def makeLink(self, linkPath, targetPath):
         """
@@ -776,9 +613,7 @@ class FileTransferServer(FileTransferBase):
         This method returns when the link is made, or a Deferred that
         returns the same.
         """
-        linkPath = self._absPath(linkPath)
-        targetPath = self._absPath(targetPath)
-        return self._runAsUser(os.symlink, targetPath, linkPath)
+        return self._sendRequest(FXP_SYMLINK, NS(linkpath)+NS(targetpath))
 
     def realPath(self, path):
         """
@@ -789,11 +624,127 @@ class FileTransferServer(FileTransferBase):
         This method returns the absolute path as a string, or a Deferred
         that returns the same.
         """
-        return self._absPath(path)
+        return self._sendRequest(FXP_REALPATH, NS(path))
+
+    def extendedRequest(self, request, data):
+        return self._sendRequest(FXP_EXTENDED, data)
+
+    def packet_VERSION(self, data):
+        version, data = struct.unpack('!L', data[:4])[0], data[4:]
+        d = {}
+        while data:
+            k, data = getNS(data)
+            v, data = getNS(data)
+            d[k]=v
+        self.version = version
+        self.gotServerVersion(version, d)
+
+    def packet_STATUS(self, data):
+        d, data = self._parseRequest(data)
+        code, data = struct.unpack('!L', data[:4])[0], data[4:]
+        msg, data = getNS(data)
+        lang = getNS(data)
+        if code == FX_OK:
+            d.callback((msg, lang))
+        else:
+            d.errback(SFTPError(code, msg, lang))
+
+    def packet_HANDLE(self, data):
+        d, data = self._parseRequest(data)
+        print 'got a handle'
+        if self.wasAFile.has_key(d):
+            d.callback(ClientFile(self, getNS(data)[0]))
+            del self.wasAFile[d]
+        else:
+            print 'was a directory'
+            d.callback(ClientDirectory(self, getNS(data)[0]))
+
+    def packet_DATA(self, data):
+        d, data = self._parseRequest(data)
+        d.callback(getNS(data)[0])
+
+    def packet_NAME(self, data):
+        print 'got a name request'
+        d, data = self._parseRequest(data)
+        count, data = struct.unpack('!L', data[:4])[0], data[4:]
+        files = []
+        for i in range(count):
+            filename, data = getNS(data)
+            longname, data = getNS(data)
+            attrs, data = self._parseAttributes(data)
+            files.append((filename, longname, attrs))
+        d.callback(files)
+
+    def packet_ATTRS(self, data):
+        d, data = self._parseRequest(data)
+        d.callback(self._parseAttributes(data)[0])
+
+    def packet_EXTENDED_REPLY(self, data):
+        d, data = self._parseRequest(data)
+        d.callback(data)
+
+    def gotServerVersion(self, serverVersion, extData):
+        """
+        Called when the client sends their version info.
+
+        otherVersion is an integer representing the version of the SFTP
+        protocol they are claiming.
+        extData is a dictionary of extended_name : extended_data items.
+        These items are sent by the client to indicate additional features.
+        """
+
+class ClientFile:
+
+    def __init__(self, parent, handle):
+        self.parent = parent
+        self.handle = NS(handle)
+
+    def close(self):
+        return self.parent._sendRequest(FXP_CLOSE, self.handle)
+
+    def readChunk(self, offset, length):
+        data = self.handle + struct.pack("!QL", offset, length)
+        return self.parent._sendRequest(FXP_READ, data)
+
+    def writeChunk(self, offset, length):
+        data = self.handle + struct.pack("!QL", offset, length)
+        return self.parent._sendRequest(FXP_WRITE, data)
+
+    def getAttrs(self):
+        return self.parent._sendRequest(FXP_FSTAT, self.handle)
+
+    def setAttrs(self, attrs):
+        data = self.handle + self.parent._packAttributes(attrs)
+        return self.parent._sendRequest(FXP_FSTAT, data)
+
+class ClientDirectory:
+
+    def __init__(self, parent, handle):
+        self.parent = parent
+        self.handle = NS(handle)
+        self.filesCache = []
+
+    def close(self):
+        return self.parent._sendRequest(FXP_CLOSE, self.handle)
+
+    def __iter__(self):
+        return self
+
+    def next(self):
+        if self.filesCache:
+            return self.filesCache.pop(0)
+        d = self.parent._sendRequest(FXP_READDIR, self.handle)
+        d.addCallback(self._cbReadDir)
+        return d
+
+    def _cbReadDir(self, names):
+        self.filesCache = names[1:]
+        return names[0]
+
 
 class SFTPError(Exception):
 
-    def __init__(self, errorCode, errorMessage, lang):
+    def __init__(self, errorCode, errorMessage, lang = ''):
         Exception.__init__(self)
         self.code = errorCode
         self.message = errorMessage
@@ -802,55 +753,6 @@ class SFTPError(Exception):
     def __str__(self):
         return 'SFTPError %s: %s' % (self.code, self.message)
 
-
-def _lsLine(name, s):
-    mode = s.st_mode
-    perms = array.array('c', '-'*10)
-    ft = stat.S_IFMT(mode)
-    if stat.S_ISDIR(ft): perms[0] = 'd'
-    elif stat.S_ISCHR(ft): perms[0] = 'c'
-    elif stat.S_ISBLK(ft): perms[0] = 'b'
-    elif stat.S_ISREG(ft): perms[0] = '-'
-    elif stat.S_ISFIFO(ft): perms[0] = 'f'
-    elif stat.S_ISLNK(ft): perms[0] = 'l'
-    elif stat.S_ISSOCK(ft): perms[0] = 's'
-    else: perms[0] = '!'
-    # user
-    if mode&stat.S_IRUSR:perms[1] = 'r'
-    if mode&stat.S_IWUSR:perms[2] = 'w'
-    if mode&stat.S_IXUSR:perms[3] = 'x'
-    # group
-    if mode&stat.S_IRGRP:perms[4] = 'r'
-    if mode&stat.S_IWGRP:perms[5] = 'w'
-    if mode&stat.S_IXGRP:perms[6] = 'x'
-    # other
-    if mode&stat.S_IROTH:perms[7] = 'r'
-    if mode&stat.S_IWOTH:perms[8] = 'w'
-    if mode&stat.S_IXOTH:perms[9] = 'w'
-    # suid/sgid
-    if mode&stat.S_ISUID:
-        if perms[3] == 'x': perms[3] = 's'
-        else: perms[3] = 'S'
-    if mode&stat.S_ISGID:
-        if perms[6] == 'x': perms[6] = 's'
-        else: perms[6] = 'S'
-    l = perms.tostring()
-    l += str(s.st_nlink).rjust(5) + ' '
-    un = str(s.st_uid)
-    l += un.ljust(9)
-    gr = str(s.st_gid)
-    l += gr.ljust(9)
-    sz = str(s.st_size)
-    l += sz.rjust(8)
-    l += ' '
-    sixmo = 60 * 60 * 24 * 7 * 26
-    if s.st_mtime + sixmo < time.time(): # last edited more than 6mo ago
-        l += time.strftime("%b %2d  %Y ", time.localtime(s.st_mtime))
-    else:
-        l += time.strftime("%b %2d %H:%S ", time.localtime(s.st_mtime))
-    l += name
-    return l
-
 FXP_INIT            =   1
 FXP_VERSION         =   2
 FXP_OPEN            =   3
