@@ -24,7 +24,8 @@ Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
 
 import socket # needed only for sync-dns
 
-import warnings, sys
+import sys
+import warnings
 from bisect import insort
 
 try:
@@ -60,19 +61,19 @@ class DelayedCall(styles.Ephemeral):
 
     def getTime(self):
         """Return the time at which this call will fire
-        
+
         @rtype: C{float}
         @return: The number of seconds after the epoch at which this call is
         scheduled to be made.
         """
         return self.time
-    
+
     def cancel(self):
         """Unschedule this call
-        
+
         @raise AlreadyCancelled: Raised if this call has already been
         unscheduled.
-        
+
         @raise AlreadyCalled: Raised if this call has already been made.
         """
         if self.cancelled:
@@ -85,11 +86,11 @@ class DelayedCall(styles.Ephemeral):
 
     def reset(self, secondsFromNow):
         """Reschedule this call for a different time
-        
+
         @type secondsFromNow: C{float}
         @param secondsFromNow: The number of seconds from the time of the
         C{reset} call at which this call will be scheduled.
-        
+
         @raise AlreadyCancelled: Raised if this call has been cancelled.
         @raise AlreadyCalled: Raised if this call has already been made.
         """
@@ -103,7 +104,7 @@ class DelayedCall(styles.Ephemeral):
 
     def delay(self, secondsLater):
         """Reschedule this call for a later time
-        
+
         @type secondsLater: C{float}
         @param secondsLater: The number of seconds after the originally
         scheduled time for which to reschedule this call.
@@ -121,7 +122,7 @@ class DelayedCall(styles.Ephemeral):
 
     def active(self):
         """Determine whether this call is still pending
-        
+
         @rtype: C{bool}
         @return: True if this call has not yet been made or cancelled,
         False otherwise.
@@ -263,16 +264,19 @@ class ReactorBase:
     def sigInt(self, *args):
         """Handle a SIGINT interrupt.
         """
+        log.msg("Received SIGINT, shutting down.")
         self.callLater(0, self.stop)
 
     def sigBreak(self, *args):
         """Handle a SIGBREAK interrupt.
         """
+        log.msg("Received SIGBREAK, shutting down.")
         self.callLater(0, self.stop)
 
     def sigTerm(self, *args):
         """Handle a SIGTERM interrupt.
         """
+        log.msg("Received SIGTERM, shutting down.")
         self.callLater(0, self.stop)
 
     def disconnectAll(self):
@@ -462,7 +466,6 @@ class ReactorBase:
     def clientUNIX(self, address, protocol, timeout=30):
         """Deprecated - use connectUNIX instead.
         """
-        import warnings
         warnings.warn("clientUNIX is deprecated - use connectUNIX instead.",
                       category=DeprecationWarning, stacklevel=2)
         f = BCFactory(protocol)
@@ -472,7 +475,6 @@ class ReactorBase:
     def clientTCP(self, host, port, protocol, timeout=30):
         """Deprecated - use connectTCP instead.
         """
-        import warnings
         warnings.warn("clientTCP is deprecated - use connectTCP instead.",
                       category=DeprecationWarning, stacklevel=2)
         f = BCFactory(protocol)
@@ -482,7 +484,6 @@ class ReactorBase:
     def clientSSL(self, host, port, protocol, contextFactory, timeout=30):
         """Deprecated - use connectSSL instead.
         """
-        import warnings
         warnings.warn("clientSSL is deprecated - use connectSSL instead.",
                       category=DeprecationWarning, stacklevel=2)
         f = BCFactory(protocol)
@@ -529,7 +530,7 @@ class BaseConnector(styles.Ephemeral):
 
     timeoutID = None
     factoryStarted = 0
-    
+
     def __init__(self, factory, timeout, reactor):
         self.state = "disconnected"
         self.reactor = reactor
@@ -587,7 +588,7 @@ class BaseConnector(styles.Ephemeral):
             # factory hasn't called our connect() method
             self.factory.doStop()
             self.factoryStarted = 0
-    
+
     def connectionLost(self, reason):
         self.state = "disconnected"
         self.factory.clientConnectionLost(self, reason)
@@ -599,10 +600,10 @@ class BaseConnector(styles.Ephemeral):
 
 class BasePort(abstract.FileDescriptor):
     """Basic implementation of a ListeningPort."""
-    
+
     addressFamily = None
     socketType = None
-    
+
     def createInternetSocket(self):
         s = socket.socket(self.addressFamily, self.socketType)
         s.setblocking(0)
