@@ -44,6 +44,22 @@ def logError(err):
     return err
 
 def succeed(result):
+    """
+    Return a Deferred that has already had '.callback(result)' called.
+
+    This is useful when you're writing synchronous code to an
+    asynchronous interface: i.e., some code is calling you expecting a
+    Deferred result, but you don't actually need to do anything
+    asynchronous. Just return defer.succeed(theResult).
+
+    See L{fail} for a version of this function that uses a failing
+    Deferred rather than a successful one.
+
+    @param result: The result to give to the Deferred's 'callback'
+           method.
+
+    @rtype: L{Deferred}
+    """
     d = Deferred()
     d.callback(result)
     return d
@@ -51,6 +67,15 @@ def succeed(result):
 class _nothing: pass
 
 def fail(result=_nothing):
+    """
+    Return a Deferred that has already had '.errback(result)' called.
+
+    See L{succeed}'s docstring for rationale.
+
+    @param result: The same argument that L{Deferred.errback<twisted.internet.defer.Deferred.errback>} takes.
+
+    @rtype: L{Deferred}
+    """
     if result is _nothing:
         result = failure.Failure()
     d = Deferred()
@@ -73,7 +98,7 @@ def execute(callable, *args, **kw):
 
 def maybeDeferred(f, *args, **kw):
     """Invoke a function that may or may not return a deferred.
-    
+
     Call the given function with the given arguments.  If the returned
     object is a C{Deferred}, return it.  If the returned object is a C{Failure},
     wrap it with C{fail} and return it.  Otherwise, wrap it in C{succeed} and
@@ -82,10 +107,10 @@ def maybeDeferred(f, *args, **kw):
 
     @type f: Any callable
     @param f: The callable to invoke
-    
+
     @param args: The arguments to pass to C{f}
     @param kw: The keyword arguments to pass to C{f}
-    
+
     @rtype: C{Deferred}
     @return: The result of the function call, wrapped in a C{Deferred} if
     necessary.
@@ -167,6 +192,8 @@ class Deferred:
 
         These will be executed when the 'master' callback is run.
         """
+        assert callable(callback)
+        assert errback == None or callable(errback)
         cbs = ((callback, callbackArgs, callbackKeywords),
                (errback or (passthru), errbackArgs, errbackKeywords))
         if self.default:
@@ -270,15 +297,7 @@ class Deferred:
         if self.called:
             if not self.debug:
                 raise AlreadyCalledError
-            extra = "\n"
-            if hasattr(self, "creator"):
-                extra += " C: Deferred was created:\n C:"
-                extra += "".join(self.creator).rstrip().replace("\n","\n C:")
-                extra += "\n"
-            if hasattr(self, "invoker"):
-                extra += " I: First Invoker was:\n I:"
-                extra += "".join(self.invoker).rstrip().replace("\n","\n I:")
-                extra += "\n"
+            extra = "\n" + self._debugInfo()
             raise AlreadyCalledError(extra)
         if self.debug:
             self.invoker = traceback.format_stack()[:-2]
@@ -293,6 +312,17 @@ class Deferred:
             del self.timeoutCall
         self._runCallbacks()
 
+    def _debugInfo(self):
+        info = ''
+        if hasattr(self, "creator"):
+            info += " C: Deferred was created:\n C:"
+            info += "".join(self.creator).rstrip().replace("\n","\n C:")
+            info += "\n"
+        if hasattr(self, "invoker"):
+            info += " I: First Invoker was:\n I:"
+            info += "".join(self.invoker).rstrip().replace("\n","\n I:")
+            info += "\n"
+        return info
 
     def _runCallbacks(self):
         if not self.paused:
@@ -338,8 +368,15 @@ class Deferred:
         @param timeoutFunc: will receive the Deferred and *args, **kw as its
         arguments.  The default timeoutFunc will call the errback with a
         L{TimeoutError}.
+
+        DON'T USE THIS! It's a bad idea! Use a function called by reactor.callLater instead
+        to accomplish the same thing!
+
+        YOU HAVE BEEN WARNED!
         """
 
+        if self.called:
+            return
         assert not self.timeoutCall, "Don't call setTimeout twice on the same Deferred."
 
         from twisted.internet import reactor
@@ -354,10 +391,11 @@ class Deferred:
 
 
     def __str__(self):
+        cname = self.__class__.__name__
         if hasattr(self, 'result'):
-            return "<Deferred at %s  current result: %r>" % (hex(id(self)),
-                                                            self.result)
-        return "<Deferred at %s>" % hex(id(self))
+            return "<%s at %s  current result: %r>" % (cname, hex(id(self)),
+                                                       self.result)
+        return "<%s at %s>" % (cname, hex(id(self)))
     __repr__ = __str__
 
 
@@ -369,7 +407,9 @@ class Deferred:
         """
         if (self.called and
             isinstance(self.result, failure.Failure)):
-            log.msg("Unhandled error in Deferred:")
+            log.msg("Unhandled error in Deferred:", isError=True)
+            if self.debug:
+                log.msg("(debug: " + self._debugInfo() + ")", isError=True)
             log.err(self.result)
 
 
@@ -381,32 +421,45 @@ class DeferredList(Deferred):
     tuples, 'success' being a boolean.
 
     Note that you can still use a L{Deferred} after putting it in a
-    DeferredList.  In particular, if you want to suppress 'Unhandled error in
-    Deferred' messages, you will still need to add errbacks to the Deferreds
-    *after* putting them in the DeferredList, as a DeferredList won't swallow
-    the errors.
+    DeferredList.  For example, you can suppress 'Unhandled error in Deferred'
+    messages by adding errbacks to the Deferreds *after* putting them in the
+    DeferredList, as a DeferredList won't swallow the errors.  (Although a more
+    convenient way to do this is simply to set the consumeErrors flag)
     """
 
     fireOnOneCallback = 0
     fireOnOneErrback = 0
 
-    def __init__(self, deferredList, fireOnOneCallback=0, fireOnOneErrback=0):
+    def __init__(self, deferredList, fireOnOneCallback=0, fireOnOneErrback=0,
+                 consumeErrors=0):
         """Initialize a DeferredList.
 
         @type deferredList:  C{list} of L{Deferred}s
         @param deferredList: The list of deferreds to track.
         @param fireOnOneCallback: (keyword param) a flag indicating that
-                             only one callback needs to
-                             be fired for me to call my callback
+                             only one callback needs to be fired for me to call
+                             my callback
         @param fireOnOneErrback: (keyword param) a flag indicating that
-                            only one errback needs to be fired for me to
-                            call my errback
+                            only one errback needs to be fired for me to call
+                            my errback
+        @param consumeErrors: (keyword param) a flag indicating that any errors
+                            raised in the original deferreds should be
+                            consumed by this DeferredList.  This is useful to
+                            prevent spurious warnings being logged.
         """
         self.resultList = [None] * len(deferredList)
         Deferred.__init__(self)
         if len(deferredList) == 0 and not fireOnOneCallback:
             self.callback(self.resultList)
 
+        # These flags need to be set *before* attaching callbacks to the
+        # deferreds, because the callbacks use these flags, and will run
+        # synchronously if any of the deferreds are already fired.
+        self.fireOnOneCallback = fireOnOneCallback
+        self.fireOnOneErrback = fireOnOneErrback
+        self.consumeErrors = consumeErrors
+        self.finishedCount = 0
+
         index = 0
         for deferred in deferredList:
             deferred.addCallbacks(self._cbDeferred, self._cbDeferred,
@@ -414,10 +467,11 @@ class DeferredList(Deferred):
                                   errbackArgs=(index,FAILURE))
             index = index + 1
 
-        self.fireOnOneCallback = fireOnOneCallback
-        self.fireOnOneErrback = fireOnOneErrback
-
     def addDeferred(self, deferred):
+        """DEPRECATED"""
+        import warnings
+        warnings.warn('DeferredList.addDeferred is deprecated.',
+                      DeprecationWarning, stacklevel=2)
         self.resultList.append(None)
         index = len(self.resultList) - 1
         deferred.addCallbacks(self._cbDeferred, self._cbDeferred,
@@ -429,13 +483,18 @@ class DeferredList(Deferred):
         """
         self.resultList[index] = (succeeded, result)
 
+        self.finishedCount += 1
         if not self.called:
             if succeeded == SUCCESS and self.fireOnOneCallback:
                 self.callback((result, index))
             elif succeeded == FAILURE and self.fireOnOneErrback:
                 self.errback(failure.Failure((result, index)))
-            elif None not in self.resultList:
+            elif self.finishedCount == len(self.resultList):
                 self.callback(self.resultList)
+
+        if succeeded == FAILURE and self.consumeErrors:
+            result = None
+
         return result
 
 
@@ -461,8 +520,8 @@ def gatherResults(deferredList, fireOnOneErrback=0):
 
 # Constants for use with DeferredList
 
-SUCCESS = 1
-FAILURE = 0
+SUCCESS = True
+FAILURE = False
 
 __all__ = ["Deferred", "DeferredList", "succeed", "fail", "FAILURE", "SUCCESS",
            "AlreadyCalledError", "TimeoutError", "gatherResults",
