@@ -25,6 +25,13 @@ Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
 from twisted.python.components import Interface
 
 
+class IAddress(Interface):
+    """An address, e.g. a TCP (host, port).
+
+    Default implementations are in twisted.internet.address.
+    """
+
+
 ### Reactor Interfaces
 
 class IConnector(Interface):
@@ -49,10 +56,7 @@ class IConnector(Interface):
     def getDestination(self):
         """Return destination this will try to connect to.
 
-        This can be one of:
-          1. TCP -- ('INET', host, port)
-          2. UNIX -- ('UNIX', address)
-          3. SSL -- ('SSL', host, port)
+        This will be an IAddress implementing object.
         """
 
 
@@ -154,7 +158,7 @@ class IReactorArbitrary(Interface):
     def listenWith(self, portType, *args, **kw):
         """Start an instance of the given C{portType} listening.
 
-        @type portType: type which implements C{IListeningPort}
+        @type portType: type which implements L{IListeningPort}
         @param portType: The object given by C{portType(*args, **kw)}
         will be started listening.
         """
@@ -163,12 +167,11 @@ class IReactorArbitrary(Interface):
         """
         Start an instance of the given C{connectorType} connecting.
 
-        @type connectorType: type which implements C{IConnector}
+        @type connectorType: type which implements L{IConnector}
         @param connectorType: The object given by C{connectorType(*args, **kw)}
         will be started connecting.
         """
 
-
 class IReactorTCP(Interface):
 
     def listenTCP(self, port, factory, backlog=5, interface=''):
@@ -176,7 +179,7 @@ class IReactorTCP(Interface):
 
         @param port: a port number on which to listen
 
-        @param factory: a twisted.internet.protocol.ServerFactory instance
+        @param factory: a L{twisted.internet.protocol.ServerFactory} instance
 
         @param backlog: size of the listen queue
 
@@ -184,7 +187,7 @@ class IReactorTCP(Interface):
 
         @returns: an object that satisfies the IListeningPort interface
 
-        @raise CannotListenError: as defined in twisted.internet.error, if it
+        @raise CannotListenError: as defined here L{twisted.internet.error.CannotListenError}, if it
            cannot listen on this port (e.g., it cannot bind to the required port
            number)
         """
@@ -196,7 +199,7 @@ class IReactorTCP(Interface):
 
         @param port: a port number
 
-        @param factory: a twisted.internet.protocol.ClientFactory instance
+        @param factory: a L{twisted.internet.protocol.ClientFactory} instance
 
         @param timeout: number of seconds to wait before assuming the
                         connection has failed.
@@ -204,12 +207,11 @@ class IReactorTCP(Interface):
         @param bindAddress: a (host, port) tuple of local address to bind
                             to, or None.
 
-        @returns:  An object implementing IConnector. This connector will call
+        @returns:  An object implementing L{IConnector}. This connector will call
            various callbacks on the factory when a connection is made,
-           failed, or lost - see ClientFactory docs for details.
+           failed, or lost - see L{ClientFactory<twisted.internet.protocol.ServerFactory>} docs for details.
         """
 
-
 class IReactorSSL(Interface):
 
     def connectSSL(self, host, port, factory, contextFactory, timeout=30, bindAddress=None):
@@ -254,7 +256,7 @@ class IReactorSSL(Interface):
 class IReactorUNIX(Interface):
     """UNIX socket methods."""
 
-    def connectUNIX(self, address, factory, timeout=30):
+    def connectUNIX(self, address, factory, timeout=30, checkPID=0):
         """Connect a client protocol to a UNIX socket.
 
         @param address: a path to a unix socket on the filesystem.
@@ -264,10 +266,13 @@ class IReactorUNIX(Interface):
         @param timeout: number of seconds to wait before assuming the connection
             has failed.
 
+        @param checkPID: if True, check for a pid file to verify that a server
+            is listening.
+
         @returns: an L{IConnector}.
         """
 
-    def listenUNIX(self, address, factory, backlog=5, mode=0666):
+    def listenUNIX(self, address, factory, backlog=5, mode=0666, wantPID=0):
         """Listen on a UNIX socket.
 
         @param address: a path to a unix socket on the filesystem.
@@ -277,6 +282,8 @@ class IReactorUNIX(Interface):
         @param backlog: number of connections to allow in backlog.
 
         @param mode: mode to set on the unix socket.
+
+        @param wantPID: if True, create a pidfile for the socket.
         """
 
 
@@ -291,7 +298,7 @@ class IReactorUNIXDatagram(Interface):
         @param protocol: a L{twisted.internet.protocol.ConnectedDatagramProtocol} instance
 
         @param maxPacketSize: maximum packet size to accept
-        
+
         @param mode: mode to set on the unix socket.
 
         @param bindAddress: address to bind to
@@ -327,7 +334,9 @@ class IReactorUDP(Interface):
 
     def connectUDP(self, remotehost, remoteport, protocol, localport=0,
                   interface='', maxPacketSize=8192):
-        """Connects a L{twisted.internet.protocol.ConnectedDatagramProtocol} instance to a UDP port.
+        """DEPRECATED.
+
+        Connects a L{twisted.internet.protocol.ConnectedDatagramProtocol} instance to a UDP port.
         """
 
 
@@ -339,14 +348,9 @@ class IReactorMulticast(Interface):
     """
 
     def listenMulticast(self, port, protocol, interface='', maxPacketSize=8192):
-        """Connects a given DatagramProtocol to the given numeric UDP port.
+        """Connects a given L{DatagramProtocol<twisted.internet.protocol.DatagramProtocol>} to the given numeric UDP port.
 
-        @returns: object conforming to IListeningPort.
-        """
-
-    def connectMulticast(self, remotehost, remoteport, protocol, localport=0,
-                         interface='', maxPacketSize=8192):
-        """Connects a ConnectedDatagramProtocol instance to a UDP port.
+        @returns: object conforming to L{IListeningPort}.
         """
 
 
@@ -381,7 +385,7 @@ class IReactorProcess(Interface):
                        in which case use those file descriptors.
                        (Not available on all systems.)
 
-        @see: C{twisted.internet.protocol.ProcessProtocol}
+        @see: L{twisted.internet.protocol.ProcessProtocol}
         """
 
 class IReactorTime(Interface):
@@ -488,15 +492,16 @@ class IReactorThreads(Interface):
         """
 
     def callFromThread(self, callable, *args, **kw):
-        """Call a function from within another thread.
+        """Call a function from within another (i.e. non-reactor) thread.
 
-        This should wake up the main thread (where run() is executing) and run
-        the given function.
+        Use this method when you want to run a function in the reactor's thread
+        from another thread.  Calling callFromThread should wake up the main
+        thread (where reactor.run() is executing) and run the given callable in
+        that thread.
 
-        I hope it is obvious from this description that this method must be
-        thread safe.  (If you want to call a method in the next mainloop
-        iteration, but you're in the same thread, use callLater with a delay of
-        0.)
+        Obviously, the callable must be thread safe.  (If you want to call a
+        function in the next mainloop iteration, but you're in the same thread,
+        use callLater with a delay of 0.)
         """
 
     def suggestThreadPoolSize(self, size):
@@ -611,13 +616,13 @@ class IReactorPluggableResolver(Interface):
     def installResolver(self, resolver):
         """Set the internal resolver to use to for name lookups.
 
-        @type resolver: An object implementing the C{IResolverSimple} interface
+        @type resolver: An object implementing the L{IResolverSimple} interface
         @param resolver: The new resolver to use.
         """
 
 
 class IReactorFDSet(Interface):
-    """Implement me to be able to use FileDescriptor type resources.
+    """Implement me to be able to use L{FileDescriptor<twisted.internet.abstract.FileDescriptor>} type resources.
 
     This assumes that your main-loop uses UNIX-style numeric file descriptors
     (or at least similarly opaque IDs returned from a .fileno() method)
@@ -659,7 +664,7 @@ class IListeningPort(Interface):
     def startListening(self):
         """Start listening on this port.
 
-        @raise CannotListenError: as defined in C{twisted.internet.error},
+        @raise CannotListenError: as defined here L{twisted.internet.error.CannotListenError},
                                   if it cannot listen on this port (e.g.,
                                   it is a TCP port and it cannot bind to
                                   the required port number)
@@ -675,9 +680,7 @@ class IListeningPort(Interface):
     def getHost(self):
         """Get the host that this port is listening for.
 
-        @returns: a tuple of C{(proto_type, ...)}, where proto_type will be
-                  a string such as 'INET', 'SSL', 'UNIX'.  The rest of the
-                  tuple will be identifying information about the port.
+        @returns: a IAddress.
         """
 
 
@@ -707,10 +710,25 @@ class IWriteDescriptor(IFileDescriptor):
 
 
 class IReadWriteDescriptor(IReadDescriptor, IWriteDescriptor):
-    """I am a FileDescriptor that can both read and write.
+    """I am a L{FileDescriptor<twisted.internet.abstract.FileDescriptor>} that can both read and write.
     """
 
 
+class ISystemHandle(Interface):
+    """An object that wraps a networking OS-specific handle."""
+
+    def getHandle(self):
+        """Return a system- and reactor-specific handle.
+
+        This might be a socket.socket() object, or some other type of
+        object, depending on which reactor is being used. Use and
+        manipulate at your own risk.
+
+        This might be used in cases where you want to set specific
+        options not exposed by the Twisted APIs.
+        """
+
+
 class IConsumer(Interface):
     """A consumer consumes data from a producer."""
 
@@ -719,7 +737,7 @@ class IConsumer(Interface):
 
         This sets self to be a consumer for a producer.  When this object
         runs out of data on a write() call, it will ask the producer
-        to resumeProducing(). A producer should implement the IProducer
+        to resumeProducing(). A producer should implement the L{IProducer}
         interface.   A push producer which is unable to pause or stop
         need not register or unregister.
         """
@@ -764,7 +782,7 @@ class IPushProducer(IProducer):
 
     This interface is semi-stable.
     """
-    
+
     def pauseProducing(self):
         """Pause producing data.
 
@@ -785,7 +803,7 @@ class IPullProducer(IProducer):
 
     This interface is semi-stable.
     """
-        
+
     def resumeProducing(self):
         """Produce data for the consumer a single time.
 
@@ -794,9 +812,9 @@ class IPullProducer(IProducer):
         by calling the consumer's write() method a single time with
         produced data.
         """
-    
+
 class IProtocol(Interface):
-    
+
     def dataReceived(self, data):
         """Called whenever data is received.
 
@@ -844,7 +862,7 @@ class IProtocolFactory(Interface):
     """
 
     def buildProtocol(self, addr):
-        """Return an object implementing IProtocol, or None.
+        """Return an object implementing L{IProtocol}, or None.
 
         This method will be called when a connection has been established
         to addr.
@@ -898,11 +916,7 @@ class ITransport(Interface):
         """
 
     def getPeer(self):
-        '''Return a tuple of (TYPE, ...).
-
-        This indicates the other end of the connection.  TYPE indicates
-        what sort of connection this is: "INET", "UNIX", or something
-        else.
+        '''Return an L{IAddress}.
 
         Treat this method with caution.  It is the unfortunate
         result of the CGI and Jabber standards, but should not
@@ -913,7 +927,7 @@ class ITransport(Interface):
 
     def getHost(self):
         """
-        Similar to getPeer, but returns a tuple describing this side of the
+        Similar to getPeer, but returns an address describing this side of the
         connection.
         """
 
@@ -941,21 +955,22 @@ class ITCPTransport(ITransport):
         to allow detection of lost peers in a non-infinite amount of time."""
 
     def getHost(self):
-        """Returns tuple ('INET', host, port)."""
+        """Returns IPv4Address."""
 
     def getPeer(self):
-        """Returns tuple ('INET', host, port)."""
-    
+        """Returns IPv4Address."""
+
+
 class ITLSTransport(ITCPTransport):
     """A TCP transport that supports switching to TLS midstream.
 
-    Once TLS mode is started the transport will implement ISSLTransport.
+    Once TLS mode is started the transport will implement L{ISSLTransport}.
     """
-    
+
     def startTLS(self, contextFactory):
         """Initiate TLS negotiation.
-        
-        @param contextFactory: A context factory (see ssl.py)
+
+        @param contextFactory: A context factory (see L{ssl.py<twisted.internet.ssl>})
         """
 
 class ISSLTransport(ITCPTransport):
@@ -1011,58 +1026,64 @@ class IServiceCollection(Interface):
 
 
 class IUDPTransport(Interface):
-    """Transport for UDP PacketProtocols."""
+    """Transport for UDP DatagramProtocols."""
 
-    def write(self, packet, (host, port)):
+    def write(self, packet, addr=None):
         """Write packet to given address.
 
-        Might raise error.ConnectionRefusedError.
+        @param addr: a tuple of (ip, port). For connected transports must
+                     be the address the transport is connected to, or None.
+                     In non-connected mode this is mandatory.
+        @raise L{MessageLengthError<twisted.internet.error.MessageLengthError>}
         """
 
+    def connect(self, host, port):
+        """Connect the transport to an address.
+
+        This changes it to connected mode. Datagrams can only be sent to
+        this address, and will only be received from this address. In addition
+        the protocol's connectionRefused method might get called if destination
+        is not receiving datagrams.
+
+        @param host: an IP address, not a domain name ('127.0.0.1', not 'localhost')
+        @param port: port to connect to.
+        """
+    
     def getHost(self):
-        """Return ('INET_UDP', interface, port) we are listening on."""
+        """Returns IPv4Address."""
 
 
 class IUDPConnectedTransport(Interface):
-    """Transport for UDP ConnectedPacketProtocols."""
+    """DEPRECATED. Transport for UDP ConnectedPacketProtocols."""
 
     def write(self, packet):
-        """Write packet to address we are connected to.
-
-        Might raise error.ConnectionRefusedError.
-        """
+        """Write packet to address we are connected to."""
 
     def getHost(self):
-        """Return ('INET_UDP', interface, port) we are listening on."""
+        """Returns UNIXAddress."""
 
 
 class IUNIXDatagramTransport(Interface):
     """Transport for UDP PacketProtocols."""
 
     def write(self, packet, address):
-        """Write packet to given address.
-
-        Might raise error.ConnectionRefusedError.
-        """
+        """Write packet to given address."""
 
     def getHost(self):
-        """Return ('UNIX_DGRAM', address) we are listening on."""
+        """Returns UNIXAddress."""
 
 
 class IUNIXDatagramConnectedTransport(Interface):
     """Transport for UDP ConnectedPacketProtocols."""
 
     def write(self, packet):
-        """Write packet to address we are connected to.
-
-        Might raise error.ConnectionRefusedError.
-        """
+        """Write packet to address we are connected to."""
 
     def getHost(self):
-        """Return ('UNIX_DGRAM', address) we are listening on."""
+        """Returns UNIXAddress."""
 
     def getPeer(self):
-        """Return ('UNIX_DGRAM', address) we are connected to."""
+        """Returns UNIXAddress."""
 
 
 class IMulticastTransport(Interface):
