@@ -28,38 +28,23 @@ import os, sys, traceback, select, errno, struct, cStringIO, types, signal
 
 try:
     import pty
-    import fcntl, termios
-except:
+except ImportError:
     pty = None
 
 try:
-    import pwd, grp
-    from os import setgroups
-    def initgroups(uid, primaryGid):
-        username = pwd.getpwuid(uid)[0]
-        l=[primaryGid]
-        for groupname, password, gid, userlist in grp.getgrall():
-            if username in userlist:
-                l.append(gid)
-        setgroups(l)
-except:
-    def initgroups(uid, primaryGid):
-        pass
-
-def switch_uid(uid, gid):
-    os.setgid(gid)
-    initgroups(uid, gid)
-    os.setuid(uid)
+    import fcntl, termios
+except ImportError:
+    fcntl = None
 
 from twisted.persisted import styles
 from twisted.python import log, failure
+from twisted.python.util import switchUID
 from twisted.internet import protocol
 
 # Sibling Imports
 import abstract, main, fdesc, error
 from main import CONNECTION_LOST, CONNECTION_DONE
 
-
 reapProcessHandlers = {}
 
 def reapAllProcesses():
@@ -68,13 +53,15 @@ def reapAllProcesses():
     for process in reapProcessHandlers.values():
         process.reapProcess()
 
+
 def registerReapProcessHandler(pid, process):
     if reapProcessHandlers.has_key(pid):
         raise RuntimeError
     try:
         aux_pid, status = os.waitpid(pid, os.WNOHANG)
     except:
-        log.deferr()
+        log.msg('Failed to reap %d:' % pid)
+        log.err()
         aux_pid = None
     if aux_pid:
         process.processEnded(status)
@@ -89,27 +76,34 @@ def unregisterReapProcessHandler(pid, process):
     del reapProcessHandlers[pid]
 
 
-class ProcessWriter(abstract.FileDescriptor, styles.Ephemeral):
-    """(Internal) Helper class to write to Process's stdin.
+class ProcessWriter(abstract.FileDescriptor):
+    """(Internal) Helper class to write into a Process's input pipe.
 
     I am a helper which describes a selectable asynchronous writer to a
-    process's stdin.
+    process's input pipe, including stdin.
     """
     connected = 1
     ic = 0
 
-    def __init__(self, proc):
+    def __init__(self, reactor, proc, name, fileno):
         """Initialize, specifying a Process instance to connect to.
         """
-        abstract.FileDescriptor.__init__(self)
+        abstract.FileDescriptor.__init__(self, reactor)
         self.proc = proc
+        self.name = name
+        self.fd = fileno
+
+    def fileno(self):
+        """Return the fileno() of my process's stdin.
+        """
+        return self.fd
 
     # Copy relevant parts of the protocol
     def writeSomeData(self, data):
         """Write some data to the open process.
         """
         try:
-            rv = os.write(self.proc.stdin, data)
+            rv = os.write(self.fd, data)
             if rv == len(data):
                 self.startReading()
             return rv
@@ -129,9 +123,10 @@ class ProcessWriter(abstract.FileDescriptor, styles.Ephemeral):
         abstract.FileDescriptor.write(self, data)
 
     def doRead(self):
-        """This does nothing.
+        """The only way this pipe can become readable is at EOF, because the
+        child has closed it.
         """
-        fd = self.fileno()
+        fd = self.fd
         r, w, x = select.select([fd], [fd], [], 0)
         if r and w:
             return CONNECTION_LOST
@@ -140,51 +135,64 @@ class ProcessWriter(abstract.FileDescriptor, styles.Ephemeral):
         """See abstract.FileDescriptor.connectionLost.
         """
         abstract.FileDescriptor.connectionLost(self, reason)
-        os.close(self.proc.stdin)
-        self.proc.inConnectionLost()
+        os.close(self.fd)
+        self.proc.childConnectionLost(self.name)
 
-    def fileno(self):
-        """Return the fileno() of my process's stdin.
-        """
-        return self.proc.stdin
 
-class ProcessError(abstract.FileDescriptor):
-    """ProcessError
+class ProcessReader(abstract.FileDescriptor):
+    """ProcessReader
 
-    I am a selectable representation of a process's stderr.
+    I am a selectable representation of a process's output pipe, such as
+    stdout and stderr.
     """
-    def __init__(self, proc):
+    def __init__(self, reactor, proc, name, fileno):
         """Initialize, specifying a process to connect to.
         """
-        abstract.FileDescriptor.__init__(self)
+        abstract.FileDescriptor.__init__(self, reactor)
         self.proc = proc
+        self.name = name
+        self.fd = fileno
 
     def fileno(self):
         """Return the fileno() of my process's stderr.
         """
-        return self.proc.stderr
+        return self.fd
+
+    def writeSomeData(self, data):
+        # the only time this is actually called is after .loseConnection Any
+        # actual write attempt would fail, so we must avoid that. This hack
+        # allows us to use .loseConnection on both readers and writers.
+        assert data == ""
+        return CONNECTION_LOST
 
     def doRead(self):
-        """Call back to my process's doError.
+        """This is called when the pipe becomes readable.
         """
-        return self.proc.doError()
+        return fdesc.readFromFD(self.fd, self.dataReceived)
+
+    def dataReceived(self, data):
+        self.proto.childDataReceived(self.name, data)
 
     def connectionLost(self, reason):
-        """I close my process's stderr.
+        """Close my end of the pipe, signal the Process (which signals the
+        ProcessProtocol).
         """
         abstract.FileDescriptor.connectionLost(self, reason)
-        os.close(self.proc.stderr)
-        self.proc.errConnectionLost()
+        os.close(self.fd)
+        self.proc.childConnectionLost(self.name)
+
 
 class ProcessExitedAlready(Exception):
     """The process has already excited, and the operation requested can no longer be performed."""
     pass
 
-class Process(abstract.FileDescriptor, styles.Ephemeral):
+
+class Process(styles.Ephemeral):
     """An operating-system Process.
 
-    This represents an operating-system process with standard input,
-    standard output, and standard error streams connected to it.
+    This represents an operating-system process with arbitrary input/output
+    pipes connected to it. Those pipes may represent standard input,
+    standard output, and standard error, or any other file descriptor.
 
     On UNIX, this is implemented using fork(), exec(), pipe()
     and fcntl(). These calls may not exist elsewhere so this
@@ -192,8 +200,11 @@ class Process(abstract.FileDescriptor, styles.Ephemeral):
     on sockets...)
     """
 
+    debug = False
+    debug_child = False
+
     def __init__(self, reactor, command, args, environment, path, proto,
-                 uid=None, gid=None):
+                 uid=None, gid=None, childFDs=None):
         """Spawn an operating-system process.
 
         This is where the hard work of disconnecting all currently open
@@ -204,8 +215,32 @@ class Process(abstract.FileDescriptor, styles.Ephemeral):
         specified.  (Implementation Note: this doesn't support all the arcane
         nuances of setXXuid on UNIX: it will assume that either your effective
         or real UID is 0.)
+
+        @param childFDs: a dictionary mapping
+            fd_in_child -> current_fd_in_parent/'r'/'w'
+
+             If the value is a number, it specifies one of the parent's fds
+             that will be remapped to the child's fd. This is useful for
+             things like inetd and shell-like file redirection.
+
+             If it is the string 'r', a pipe will be created and attached to
+             the child at that fd number, and the parent will be able to
+             read from the pipe. This is useful for the child's stdout and
+             stderr.
+
+             If it is the string 'w', a pipe will be created and attached,
+             and the parent will be able to write into that pipe. This is
+             useful for the child's stdin.
+
+            If childFDs is not passed, the default behaviour is to use a
+            mapping that opens the usual stdin/stdout/stderr pipes.
         """
-        abstract.FileDescriptor.__init__(self, reactor)
+
+        if not proto:
+            assert 'r' not in childFDs.values() and 'w' not in childFDs.values()
+
+        self.lostProcess = False
+
         settingUID = (uid is not None) or (gid is not None)
         if settingUID:
             curegid = os.getegid()
@@ -219,36 +254,75 @@ class Process(abstract.FileDescriptor, styles.Ephemeral):
             # prepare to change UID in subprocess
             os.setuid(0)
             os.setgid(0)
-        stdout_read, stdout_write = os.pipe()
-        stderr_read, stderr_write = os.pipe()
-        stdin_read,  stdin_write  = os.pipe()
+
+        self.pipes = {}
+        # keys are childFDs, we can sense them closing
+        # values are ProcessReader/ProcessWriters
+
+        helpers = {}
+        # keys are childFDs
+        # values are parentFDs
+
+        if childFDs is None:
+            childFDs = {0: "w", # we write to the child's stdin
+                        1: "r", # we read from their stdout
+                        2: "r", # and we read from their stderr
+                        }
+
+        debug = self.debug
+        if debug: print "childFDs", childFDs
+
+        # fdmap.keys() are filenos of pipes that are used by the child.
+        fdmap = {} # maps childFD to parentFD
+        for childFD, target in childFDs.items():
+            if debug: print "[%d]" % childFD, target
+            if target == "r":
+                # we need a pipe that the parent can read from
+                readFD, writeFD = os.pipe()
+                if debug: print "readFD=%d, writeFD%d" % (readFD, writeFD)
+                fdmap[childFD] = writeFD     # child writes to this
+                helpers[childFD] = readFD    # parent reads from this
+                fdesc.setNonBlocking(readFD)
+            elif target == "w":
+                # we need a pipe that the parent can write to
+                readFD, writeFD = os.pipe()
+                if debug: print "readFD=%d, writeFD=%d" % (readFD, writeFD)
+                fdmap[childFD] = readFD      # child reads from this
+                helpers[childFD] = writeFD   # parent writes to this
+                fdesc.setNonBlocking(writeFD)
+            else:
+                assert type(target) == int, '%r should be an int' % (target,)
+                fdmap[childFD] = target      # parent ignores this
+        if debug: print "fdmap", fdmap
+        if debug: print "helpers", helpers
+        # the child only cares about fdmap.values()
+
         self.pid = os.fork()
         if self.pid == 0: # pid is 0 in the child process
-            # stop debugging, if I am!  I don't care anymore!
-            sys.settrace(None)
-            # Destroy my stdin / stdout / stderr (in that order)
+
+            # do not put *ANY* code outside the try block. The child process
+            # must either exec or _exit. If it gets outside this block (due
+            # to an exception that is not handled here, but which might be
+            # handled higher up), there will be two copies of the parent
+            # running in parallel, doing all kinds of damage.
+
+            # After each change to this code, review it to make sure there
+            # are no exit paths.
+
             try:
-                os.dup2(stdin_read, 0)
-                os.dup2(stdout_write, 1)
-                os.dup2(stderr_write, 2)
-                # XXX TODO FIXME: 256 is a magic number here; really we need a
-                # way of saying "close all open FDs except 0, 1, 2".  This will
-                # fail in a surprising and subtle way if the current process
-                # has more than 256 FDs open.  On linux this would be
-                # "[os.close(int(fd)) for fd in os.listdir('/proc/self/fd')]"
-                # but I seriously doubt that's portable.
-                for fd in range(3, 256):
-                    try:    os.close(fd)
-                    except: pass
-                if path:
-                    os.chdir(path)
-                # set the UID before I actually exec the process
-                if settingUID:
-                    switch_uid(uid, gid)
-                os.execvpe(command, args, environment)
+                # stop debugging, if I am!  I don't care anymore!
+                sys.settrace(None)
+                # close all parent-side pipes
+                self._setupChild(fdmap)
+                self._execChild(path, settingUID, uid, gid,
+                                command, args, environment)
             except:
                 # If there are errors, bail and try to write something
                 # descriptive to stderr.
+                # XXX: The parent's stderr isn't necessarily fd 2 anymore, or
+                #      even still available
+                # XXXX: however even libc assumes write(2,err) is a useful
+                #       thing to attempt
                 try:
                     stderr = os.fdopen(2,'w')
                     stderr.write("Upon execvpe %s %s in environment %s\n:" %
@@ -260,32 +334,154 @@ class Process(abstract.FileDescriptor, styles.Ephemeral):
                         os.close(fd)
                 except:
                     pass # make *sure* the child terminates
+            # Did you read the comment about not adding code here?
             os._exit(1)
+
+        # we are the parent
+
         if settingUID:
             os.setregid(currgid, curegid)
             os.setreuid(curruid, cureuid)
-        self.status = -1
-        for fd in stdout_write, stderr_write, stdin_read:
-            os.close(fd)
-        for fd in (stdout_read, stderr_read, stdin_write):
-            fdesc.setNonBlocking(fd)
-        self.stdout = stdout_read # os.fdopen(stdout_read, 'r')
-        self.stderr = stderr_read # os.fdopen(stderr_read, 'r')
-        self.stdin = stdin_write
-        # ok now I really have a fileno()
-        self.writer = ProcessWriter(self)
-        self.writer.startReading()
-        self.err = ProcessError(self)
-        self.err.startReading()
-        self.startReading()
-        self.connected = 1
+        self.status = -1 # this records the exit status of the child
+
+        # arrange for the parent-side pipes to be read and written
+        for childFD, parentFD in helpers.items():
+            os.close(fdmap[childFD])
+
+            if childFDs[childFD] == "r":
+                reader = ProcessReader(reactor, self, childFD, parentFD)
+                reader.proto = proto
+                self.pipes[childFD] = reader
+                reader.startReading()
+
+            if childFDs[childFD] == "w":
+                writer = ProcessWriter(reactor, self, childFD, parentFD)
+                writer.proto = proto
+                self.pipes[childFD] = writer
+                # we do startReading here to watch for EOF. We won't do an
+                # actual .startWriting until some data has been written to
+                # the transmit buffer.
+                writer.startReading()
+
         self.proto = proto
         try:
-            self.proto.makeConnection(self)
+            # the 'transport' is used for some compatibility methods
+            if self.proto is not None:
+                self.proto.makeConnection(self)
         except:
-            log.deferr()
+            log.err()
         registerReapProcessHandler(self.pid, self)
 
+    def _setupChild(self, fdmap):
+        """
+        fdmap[childFD] = parentFD
+
+        The child wants to end up with 'childFD' attached to what used to be
+        the parent's parentFD. As an example, a bash command run like
+        'command 2>&1' would correspond to an fdmap of {0:0, 1:1, 2:1}.
+        'command >foo.txt' would be {0:0, 1:os.open('foo.txt'), 2:2}.
+
+        Step 1: close all file descriptors that aren't values of fdmap.
+        This means 0 .. maxfds.
+
+        Step 2: for each childFD:
+         if fdmap[childFD] == childFD, the descriptor is already in place.
+         Make sure the CLOEXEC flag is not set, then delete the entry from
+         fdmap.
+
+         if childFD is in fdmap.values(), then the target descriptor is
+         busy. Use os.dup() to move it elsewhere, update all fdmap[childFD]
+         items that point to it, then close the original. Then fall through
+         to the next case.
+
+         now fdmap[childFD] is not in fdmap.values(), and is free. Use
+         os.dup2() to move it to the right place, then close the original.
+
+        """
+
+        debug = self.debug_child
+        if debug:
+            #errfd = open("/tmp/p.err", "a", 0)
+            errfd = sys.stderr
+            print >>errfd, "starting _setupChild"
+
+        destList = fdmap.values()
+        try:
+            import resource
+            maxfds = resource.getrlimit(resource.RLIMIT_NOFILE)[1] + 1
+            # OS-X reports 9223372036854775808. That's a lot of fds to close
+            if maxfds > 1024:
+                maxfds = 1024
+        except:
+            maxfds = 256
+
+        for fd in range(maxfds):
+            if fd in destList:
+                continue
+            if debug and fd == errfd.fileno():
+                continue
+            try:    os.close(fd)
+            except: pass
+
+        # at this point, the only fds still open are the ones that need to
+        # be moved to their appropriate positions in the child (the targets
+        # of fdmap, i.e. fdmap.values() )
+
+        if debug: print >>errfd, "fdmap", fdmap
+        childlist = fdmap.keys()
+        childlist.sort()
+
+        for child in childlist:
+            target = fdmap[child]
+            if target == child:
+                # fd is already in place
+                if debug: print >>errfd, "%d already in place" % target
+                if fcntl and hasattr(fcntl, 'FD_CLOEXEC'):
+                    old = fcntl.fcntl(child, fcntl.F_GETFD)
+                    fcntl.fcntl(child,
+                                fcntl.F_SETFD, old & ~fcntl.FD_CLOEXEC)
+            else:
+                if child in fdmap.values():
+                    # we can't replace child-fd yet, as some other mapping
+                    # still needs the fd it wants to target. We must preserve
+                    # that old fd by duping it to a new home.
+                    newtarget = os.dup(child) # give it a safe home
+                    if debug: print >>errfd, "os.dup(%d) -> %d" % (child,
+                                                                   newtarget)
+                    os.close(child) # close the original
+                    for c,p in fdmap.items():
+                        if p == child:
+                            fdmap[c] = newtarget # update all pointers
+                # now it should be available
+                if debug: print >>errfd, "os.dup2(%d,%d)" % (target, child)
+                os.dup2(target, child)
+
+        # At this point, the child has everything it needs. We want to close
+        # everything that isn't going to be used by the child, i.e.
+        # everything not in fdmap.keys(). The only remaining fds open are
+        # those in fdmap.values().
+
+        # Any given fd may appear in fdmap.values() multiple times, so we
+        # need to remove duplicates first.
+
+        old = []
+        for fd in fdmap.values():
+            if not fd in old:
+                if not fd in fdmap.keys():
+                    old.append(fd)
+        if debug: print >>errfd, "old", old
+        for fd in old:
+            os.close(fd)
+
+    def _execChild(self, path, settingUID, uid, gid,
+                   command, args, environment):
+        if path:
+            os.chdir(path)
+        # set the UID before I actually exec the process
+        if settingUID:
+            switchUID(uid, gid)
+        os.execvpe(command, args, environment)
+
     def reapProcess(self):
         """Try to reap a process (without blocking) via waitpid.
 
@@ -301,135 +497,111 @@ class Process(abstract.FileDescriptor, styles.Ephemeral):
         try:
             pid, status = os.waitpid(self.pid, os.WNOHANG)
         except:
-            log.deferr()
+            log.msg('Failed to reap %d:' % self.pid)
+            log.err()
             pid = None
         if pid:
             self.processEnded(status)
-            del reapProcessHandlers[pid]
+            unregisterReapProcessHandler(pid, self)
 
+    def writeToChild(self, childFD, data):
+        self.pipes[childFD].write(data)
+    
+    def closeChildFD(self, childFD):
+        # for writer pipes, loseConnection tries to write the remaining data
+        # out to the pipe before closing it
+        # if childFD is not in the list of pipes, assume that it is already
+        # closed
+        if self.pipes.has_key(childFD):
+            self.pipes[childFD].loseConnection()
+
+    def pauseProducing(self):
+        for p in self.pipes.itervalues():
+            if isinstance(p, ProcessReader):
+                p.stopReading()
+ 
+    def resumeProducing(self):
+        for p in self.pipes.itervalues():
+            if isinstance(p, ProcessReader):
+                p.startReading()
+
+    # compatibility
     def closeStdin(self):
         """Call this to close standard input on this process.
         """
-        if hasattr(self, "writer"):
-            self.writer.loseConnection()
-
-    def closeStderr(self):
-        """Close stderr."""
-        if hasattr(self, "err"):
-             self.err.stopReading()
-             self.err.connectionLost(None)
-
+        self.closeChildFD(0)
     def closeStdout(self):
-        """Close stdout."""
-        if not self.lostOutConnection:
-            self.stopReading()
-            self.connectionLost(None)
-
+        self.closeChildFD(1)
+    def closeStderr(self):
+        self.closeChildFD(2)
     def loseConnection(self):
         self.closeStdin()
         self.closeStderr()
         self.closeStdout()
+    def write(self,data):
+        """Call this to write to standard input on this process.
+        """
+        self.pipes[0].write(data)
 
     def signalProcess(self, signalID):
-        if signalID in ('HUP', 'STOP', 'INT', 'KILL'):
+        if signalID in ('HUP', 'STOP', 'INT', 'KILL', 'TERM'):
             signalID = getattr(signal, 'SIG'+signalID)
         if self.pid is None:
             raise ProcessExitedAlready
         os.kill(self.pid, signalID)
 
-    def doError(self):
-        """Called when my standard error stream is ready for reading.
-        """
-        return fdesc.readFromFD(self.stderr, self.proto.errReceived)
+    def processEnded(self, status):
+        # this is called when the child terminates (SIGCHLD)
+        self.status = status
+        self.lostProcess = True
+        self.pid = None
+        #for fd, helper in self.pipes.items():
+        #    helper.connectionLost(None)
+        ##self.closeStdin()
+        self.maybeCallProcessEnded()
 
-    def doRead(self):
-        """Called when my standard output stream is ready for reading.
-        """
-        return fdesc.readFromFD(self.stdout, self.proto.outReceived)
-
-    def doWrite(self):
-        """Called when my standard output stream is ready for writing.
-
-        This will only happen in the case where the pipe to write to is
-        broken.
-        """
-        return CONNECTION_DONE
-
-    def write(self,data):
-        """Call this to write to standard input on this process.
-        """
-        self.writer.write(data)
-
-    def fileno(self):
-        """This returns the file number of standard output on this process.
-        """
-        return self.stdout
-
-    lostErrorConnection = 0
-    lostOutConnection = 0
-    lostInConnection = 0
-    lostProcess = 0
+    def childConnectionLost(self, childFD):
+        # this is called when one of the helpers (ProcessReader or
+        # ProcessWriter) notices their pipe has been closed
+        del self.pipes[childFD]
+        try:
+            self.proto.childConnectionLost(childFD)
+        except:
+            log.err()
+        self.maybeCallProcessEnded()
 
     def maybeCallProcessEnded(self):
-        if (self.lostErrorConnection and
-            self.lostOutConnection and
-            self.lostInConnection):
-            if self.lostProcess:
-                try:
-                    exitCode = sig = None
-                    if self.status != -1:
-                        if os.WIFEXITED(self.status):
-                            exitCode = os.WEXITSTATUS(self.status)
-                        else:
-                            sig = os.WTERMSIG(self.status)
-                    else:
-                        pass # wonder when this happens
-                    if exitCode or sig:
-                        e = error.ProcessTerminated(exitCode, sig, self.status)
-                    else:
-                        e = error.ProcessDone(self.status)
-                    self.proto.processEnded(failure.Failure(e))
-                except:
-                    log.deferr()
+        # we don't call ProcessProtocol.processEnded until:
+        #  the child has terminated, AND
+        #  all writers have indicated an error status, AND
+        #  all readers have indicated EOF
+        # This insures that we've gathered all output from the process.
+
+        if self.pipes:
+            #print "maybe, but pipes still", self.pipes.keys()
+            return
+        if not self.lostProcess:
+            #print "maybe, but haven't .lostProcess yet"
+            self.reapProcess()
+            return
+        try:
+            exitCode = sig = None
+            if self.status != -1:
+                if os.WIFEXITED(self.status):
+                    exitCode = os.WEXITSTATUS(self.status)
+                else:
+                    sig = os.WTERMSIG(self.status)
             else:
-                self.reapProcess()
-
-    def processEnded(self, status):
-        self.status = status
-        self.lostProcess = 1
-        self.pid = None
-        self.closeStdin()
-        self.maybeCallProcessEnded()
-
-    def inConnectionLost(self):
-        try:
-            self.proto.inConnectionLost()
+                pass # don't think this can happen
+            if exitCode or sig:
+                e = error.ProcessTerminated(exitCode, sig, self.status)
+            else:
+                e = error.ProcessDone(self.status)
+            if self.proto is not None:
+                self.proto.processEnded(failure.Failure(e))
+                self.proto = None
         except:
-            log.deferr()
-        del self.writer
-        self.lostInConnection = 1
-        self.maybeCallProcessEnded()
-
-    def errConnectionLost(self):
-        self.lostErrorConnection = 1
-        del self.err
-        try:
-            self.proto.errConnectionLost()
-        except:
-            log.deferr()
-        self.maybeCallProcessEnded()
-
-    def connectionLost(self, reason):
-        """stdout closed.
-        """
-        self.lostOutConnection = 1
-        abstract.FileDescriptor.connectionLost(self, reason)
-        os.close(self.stdout)
-        try:
-            self.proto.outConnectionLost()
-        except:
-            log.deferr()
-        self.maybeCallProcessEnded()
+            log.err()
 
 
 class PTYProcess(abstract.FileDescriptor, styles.Ephemeral):
@@ -498,7 +670,7 @@ class PTYProcess(abstract.FileDescriptor, styles.Ephemeral):
 
                 # set the UID before I actually exec the process
                 if settingUID:
-                    switch_uid(uid, gid)
+                    switchUID(uid, gid)
                 os.execvpe(command, args, environment)
             except:
                 stderr = os.fdopen(1, 'w')
@@ -520,7 +692,7 @@ class PTYProcess(abstract.FileDescriptor, styles.Ephemeral):
         try:
             self.proto.makeConnection(self)
         except:
-            log.deferr()
+            log.err()
         registerReapProcessHandler(self.pid, self)
 
     def reapProcess(self):
@@ -543,11 +715,11 @@ class PTYProcess(abstract.FileDescriptor, styles.Ephemeral):
             else:
                 raise
         except:
-            log.deferr()
+            log.err()
             pid = None
         if pid:
             self.processEnded(status)
-            del reapProcessHandlers[pid]
+            unregisterReapProcessHandler(self.pid, self)
 
     # PTYs do not have stdin/stdout/stderr. They only have in and out, just
     # like sockets. You cannot close one without closing off the entire PTY.
@@ -605,8 +777,9 @@ class PTYProcess(abstract.FileDescriptor, styles.Ephemeral):
                 else:
                     e = error.ProcessDone(self.status)
                 self.proto.processEnded(failure.Failure(e))
+                self.proto = None
             except:
-                log.deferr()
+                log.err()
 
     def connectionLost(self, reason):
         """I call this to clean up when one or all of my connections has died.
