@@ -1,4 +1,5 @@
-
+# -*- test-case-name: twisted.test.test_factories -*-
+#
 # Twisted, the Framework of Your Internet
 # Copyright (C) 2001 Matthew W. Lefkowitz
 #
@@ -93,6 +94,8 @@ class Factory:
         factory.
 
         Override this method to alter how Protocol instances get created.
+
+        @param addr: an object implementing L{twisted.internet.interfaces.IAddress}
         """
         p = self.protocol()
         p.factory = self
@@ -202,7 +205,7 @@ class ReconnectingClientFactory(ClientFactory):
     maxRetries = None
     _callID = None
     connector = None
-    
+
     continueTrying = 1
 
     def clientConnectionFailed(self, connector, reason):
@@ -242,7 +245,7 @@ class ReconnectingClientFactory(ClientFactory):
 
         log.msg("%s will retry in %d seconds" % (connector, self.delay,))
         from twisted.internet import reactor
-        
+
         def reconnector():
             self._callID = None
             connector.connect()
@@ -405,22 +408,31 @@ class ProcessProtocol(BaseProtocol):
     """Processes have some additional methods besides receiving data.
     """
 
+    def childDataReceived(self, childFD, data):
+        if childFD == 1:
+            self.outReceived(data)
+        elif childFD == 2:
+            self.errReceived(data)
+
     def outReceived(self, data):
         """Some data was received from stdout."""
-
     def errReceived(self, data):
-        """Some data was received from stderr.
-        """
+        """Some data was received from stderr."""
 
-    def errConnectionLost(self):
-        """This will be called when stderr is closed.
-        """
-
-    def outConnectionLost(self):
-        """Called when stdout is shut down."""
+    def childConnectionLost(self, childFD):
+        if childFD == 0:
+            self.inConnectionLost()
+        elif childFD == 1:
+            self.outConnectionLost()
+        elif childFD == 2:
+            self.errConnectionLost()
 
     def inConnectionLost(self):
-        """Called when stdin is shut down."""
+        """This will be called when stdin is closed."""
+    def outConnectionLost(self):
+        """This will be called when stdout is closed."""
+    def errConnectionLost(self):
+        """This will be called when stderr is closed."""
 
     def processEnded(self, reason):
         """This will be called when the subprocess is finished.
@@ -434,7 +446,7 @@ class AbstractDatagramProtocol:
 
     transport = None
     numPorts = 0
-    noisy = "sure, why not"
+    noisy = True
 
     def __getstate__(self):
         d = self.__dict__.copy()
@@ -442,7 +454,10 @@ class AbstractDatagramProtocol:
         return d
 
     def doStart(self):
-        """Make sure startProtocol is called."""
+        """Make sure startProtocol is called.
+
+        This will be called by makeConnection(), users should not call it.
+        """
         if not self.numPorts:
             if self.noisy:
                 log.msg("Starting protocol %s" % self)
@@ -450,9 +465,13 @@ class AbstractDatagramProtocol:
         self.numPorts = self.numPorts + 1
 
     def doStop(self):
-        """Make sure stopProtocol is called."""
+        """Make sure stopProtocol is called.
+
+        This will be called by the port, users should not call it.
+        """
         assert self.numPorts > 0
         self.numPorts = self.numPorts - 1
+        self.transport = None
         if not self.numPorts:
             if self.noisy:
                 log.msg("Stopping protocol %s" % self)
@@ -476,6 +495,7 @@ class AbstractDatagramProtocol:
         This sets the 'transport' attribute of this DatagramProtocol, and calls the
         doStart() callback.
         """
+        assert self.transport == None
         self.transport = transport
         self.doStart()
 
@@ -486,12 +506,23 @@ class AbstractDatagramProtocol:
         @param addr: tuple of source of datagram.
         """
 
+
 class DatagramProtocol(AbstractDatagramProtocol):
     """Protocol for datagram-oriented transport, e.g. UDP."""
 
+    def connectionRefused(self):
+        """Called due to error from write in connected mode.
+
+        Note this is a result of ICMP message generated by *previous*
+        write.
+        """
+
 
 class ConnectedDatagramProtocol(DatagramProtocol):
-    """Protocol for connected datagram-oriented transport, e.g. UDP."""
+    """Protocol for connected datagram-oriented transport.
+
+    No longer necessary for UDP.
+    """
 
     def datagramReceived(self, datagram):
         """Called when a datagram is received.
@@ -505,12 +536,6 @@ class ConnectedDatagramProtocol(DatagramProtocol):
         Usually this will be due to a DNS lookup failure.
         """
 
-    def connectionRefused(self):
-        """Called for connection refused error.
-
-        Note this is a result of ICMP message generated by *previous*
-        write.
-        """
 
 
 class FileWrapper:
