@@ -1,16 +1,16 @@
 # -*- test-case-name: twisted.test.test_ssl -*-
 # Twisted, the Framework of Your Internet
 # Copyright (C) 2001 Matthew W. Lefkowitz
-# 
+#
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of version 2.1 of the GNU Lesser General Public
 # License as published by the Free Software Foundation.
-# 
+#
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -28,8 +28,8 @@ moment this is not enforced, but in the future it might be.
 
 API Stability: stable
 
-Future Plans: 
-    - split module so reactor-specific classes are in a separate module 
+Future Plans:
+    - split module so reactor-specific classes are in a separate module
     - support for switching TCP into SSL
     - more options
 
@@ -52,13 +52,14 @@ import tcp, main, interfaces
 
 # Twisted imports
 from twisted.python import log
-from twisted.internet import base
+from twisted.internet import base, address
+
 
 class ContextFactory:
     """A factory for SSL context objects, for server SSL connections."""
 
     isClient = 0
-    
+
     def getContext(self):
         """Return a SSL.Context object. override in subclasses."""
         raise NotImplementedError
@@ -106,24 +107,22 @@ class ClientContextFactory:
 
 class Client(tcp.Client):
     """I am an SSL client."""
+
+    __implements__ = tuple([i for i in tcp.Client.__implements__ if i is not interfaces.ITLSTransport]), interfaces.ISSLTransport
+    
     def __init__(self, host, port, bindAddress, ctxFactory, connector, reactor=None):
         # tcp.Client.__init__ depends on self.ctxFactory being set
         self.ctxFactory = ctxFactory
         tcp.Client.__init__(self, host, port, bindAddress, connector, reactor)
 
     def getHost(self):
-        """Returns a tuple of ('SSL', hostname, port).
-
-        This indicates the address from which I am connecting.
-        """
-        return ('SSL',)+self.socket.getsockname()
+        """Returns the address from which I am connecting."""
+        h, p = self.socket.getsockname()
+        return address.IPv4Address('TCP', h, p, 'SSL')
 
     def getPeer(self):
-        """Returns a tuple of ('SSL', hostname, port).
-
-        This indicates the address that I am connected to.
-        """
-        return ('SSL',)+self.addr
+        """Returns the address that I am connected."""
+        return address.IPv4Address('TCP', self.addr[0], self.addr[1], 'SSL')
 
     def _connectDone(self):
         self.startTLS(self.ctxFactory)
@@ -134,29 +133,28 @@ class Client(tcp.Client):
 class Server(tcp.Server):
     """I am an SSL server.
     """
-    def getHost(self):
-        """Returns a tuple of ('SSL', hostname, port).
 
-        This indicates the servers address.
-        """
-        return ('SSL',)+self.socket.getsockname()
+    __implements__ = tcp.Server.__implements__, interfaces.ISSLTransport
+    
+    def getHost(self):
+        """Return server's address."""
+        h, p = self.socket.getsockname()
+        return address.IPv4Address('TCP', h, p, 'SSL')
 
     def getPeer(self):
-        """
-        Returns a tuple of ('SSL', hostname, port), indicating the connected
-        client's address.
-        """
-        return ('SSL',)+self.client
+        """Return address of peer."""
+        h, p = self.client
+        return address.IPv4Address('TCP', h, p, 'SSL')
 
 
 class Port(tcp.Port):
     """I am an SSL port."""
     transport = Server
-    
+
     def __init__(self, port, factory, ctxFactory, backlog=5, interface='', reactor=None):
         tcp.Port.__init__(self, port, factory, backlog, interface, reactor)
         self.ctxFactory = ctxFactory
-    
+
     def createInternetSocket(self):
         """(internal) create an SSL socket
         """
@@ -182,7 +180,8 @@ class Connector(base.BaseConnector):
         return Client(self.host, self.port, self.bindAddress, self.contextFactory, self, self.reactor)
 
     def getDestination(self):
-        return ('SSL', self.host, self.port)
+        return address.IPv4Address('TCP', self.host, self.port, 'SSL')
+ 
 
 __all__ = ["ContextFactory", "DefaultOpenSSLContextFactory", "ClientContextFactory"]
 
