@@ -16,9 +16,13 @@
 
 """Standard input/out/err support.
 
-API Stability: semi-stable
+API Stability: unstable (pending deprecation in favor of a reactor-based API)
 
-Future Plans: support for stderr, perhaps
+Future Plans:
+
+    support for stderr, perhaps
+    Rewrite to use the reactor instead of an ad-hoc mechanism for connecting
+        protocols to transport.
 
 Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
 """
@@ -106,4 +110,9 @@ class StandardIO(abstract.FileDescriptor):
     def connectionLost(self, reason):
         """The connection was lost.
         """
-        self.protocol.connectionLost()
+        try:
+            self.protocol.connectionLost(reason)
+        except TypeError:
+            import warnings
+            warnings.warn("Protocol.connectionLost() should take a 'reason' argument")
+            self.protocol.connectionLost()
