@@ -70,7 +70,7 @@ elif os.name != 'java':
 from errno import EAGAIN
 
 # Twisted Imports
-from twisted.internet import protocol, defer, base
+from twisted.internet import protocol, defer, base, address
 from twisted.persisted import styles
 from twisted.python import log, failure, reflect
 from twisted.python.runtime import platform, platformType
@@ -89,7 +89,7 @@ class _TLSMixin:
 
     def getPeerCertificate(self):
         return self.socket.get_peer_certificate()
-    
+
     def doRead(self):
         if self.writeBlockedOnRead:
             self.writeBlockedOnRead = 0
@@ -119,7 +119,7 @@ class _TLSMixin:
         Connection.loseConnection(self)
         if self.connected:
             self.startReading()
-    
+
     def doWrite(self):
         if self.writeBlockedOnRead:
             self.stopWriting()
@@ -182,7 +182,7 @@ class _TLSMixin:
             # from the socket
             self.stopWriting()
             self.startReading()
-            
+
             # don't close socket just yet
             return None
 
@@ -193,7 +193,7 @@ class Connection(abstract.FileDescriptor):
     connection based socket.
     """
 
-    __implements__ = abstract.FileDescriptor.__implements__, interfaces.ITCPTransport
+    __implements__ = abstract.FileDescriptor.__implements__, interfaces.ITCPTransport, interfaces.ISystemHandle
 
     TLS = 0
 
@@ -218,10 +218,15 @@ class Connection(abstract.FileDescriptor):
 
         def _startTLS(self):
             self.TLS = 1
-            class TLSConnection(_TLSMixin, self.__class__):
-                pass
+            klass = self.__class__
+            class TLSConnection(_TLSMixin, klass):
+                __implements__ = interfaces.ISSLTransport, klass.__implements__
             self.__class__ = TLSConnection
 
+    def getHandle(self):
+        """Return the socket for this connection."""
+        return self.socket
+    
     def doRead(self):
         """Calls self.protocol.dataReceived with all available data.
 
@@ -418,7 +423,7 @@ class BaseClient(Connection):
             else:
                 self.failIfNotConnected(error.getConnectError((connectResult, os.strerror(connectResult))))
                 return
-        
+
         # If I have reached this point without raising or returning, that means
         # that the socket is connected.
         del self.doWrite
@@ -450,7 +455,7 @@ class Client(BaseClient):
         # BaseClient.__init__ is invoked later
         self.connector = connector
         self.addr = (host, port)
-        
+
         whenDone = self.resolveAddress
         err = None
         skt = None
@@ -469,18 +474,18 @@ class Client(BaseClient):
         self._finishInit(whenDone, skt, err, reactor)
 
     def getHost(self):
-        """Returns a tuple of ('INET', hostname, port).
+        """Returns an IPv4Address.
 
         This indicates the address from which I am connecting.
         """
-        return ('INET',)+self.socket.getsockname()
+        return address.IPv4Address('TCP', *(self.socket.getsockname() + ('INET',)))
 
     def getPeer(self):
-        """Returns a tuple of ('INET', hostname, port).
+        """Returns an IPv4Address.
 
         This indicates the address that I am connected to.
         """
-        return ('INET',)+self.addr
+        return address.IPv4Address('TCP', *(self.addr + ('INET',)))
 
     def __repr__(self):
         s = '<%s to %s at %x>' % (self.__class__, self.addr, id(self))
@@ -525,19 +530,18 @@ class Server(Connection):
         return holder
 
     def getHost(self):
-        """Returns a tuple of ('INET', hostname, port).
+        """Returns an IPv4Address.
 
-        This indicates the servers address.
+        This indicates the server's address.
         """
-        return ('INET',)+self.socket.getsockname()
+        return address.IPv4Address('TCP', *(self.socket.getsockname() + ('INET',)))
 
     def getPeer(self):
-        """
-        Returns a tuple of ('INET', hostname, port), indicating the connected
-        client's address.
-        """
-        return ('INET',)+self.client
+        """Returns an IPv4Address.
 
+        This indicates the client's address.
+        """
+        return address.IPv4Address('TCP', *(self.client + ('INET',)))
 
 class Port(base.BasePort):
     """I am a TCP server port, listening for connections.
@@ -596,6 +600,9 @@ class Port(base.BasePort):
         self.numberAccepts = 100
         self.startReading()
 
+    def _buildAddr(self, (host, port)):
+        return address._ServerFactoryIPv4Address('TCP', host, port)
+    
     def doRead(self):
         """Called when my socket is ready for reading.
 
@@ -623,8 +630,8 @@ class Port(base.BasePort):
                     elif e.args[0] == EPERM:
                         continue
                     raise
-                
-                protocol = self.factory.buildProtocol(addr)
+
+                protocol = self.factory.buildProtocol(self._buildAddr(addr))
                 if protocol is None:
                     skt.close()
                     continue
@@ -684,11 +691,11 @@ class Port(base.BasePort):
         return reflect.qual(self.factory.__class__)
 
     def getHost(self):
-        """Returns a tuple of ('INET', hostname, port).
+        """Returns an IPv4Address.
 
         This indicates the server's address.
         """
-        return ('INET',)+self.socket.getsockname()
+        return address.IPv4Address('TCP', *(self.socket.getsockname() + ('INET',)))
 
 
 class Connector(base.BaseConnector):
@@ -707,4 +714,4 @@ class Connector(base.BaseConnector):
         return Client(self.host, self.port, self.bindAddress, self, self.reactor)
 
     def getDestination(self):
-        return ('INET', self.host, self.port)
+        return address.IPv4Address('TCP', self.host, self.port, 'INET')
