@@ -20,7 +20,7 @@
 
 Please do not use this module directly.
 
-API Stability: unstable
+API Stability: semi-stable
 
 Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
 """
@@ -30,6 +30,7 @@ import os
 import socket
 import operator
 import struct
+import warnings
 
 from twisted.python.runtime import platformType
 if platformType == 'win32':
@@ -43,7 +44,7 @@ elif platformType != 'java':
     from errno import EWOULDBLOCK, EINTR, EMSGSIZE, ECONNREFUSED, EAGAIN
 
 # Twisted Imports
-from twisted.internet import protocol, base, defer
+from twisted.internet import protocol, base, defer, address
 from twisted.persisted import styles
 from twisted.python import log, reflect
 
@@ -54,12 +55,12 @@ import abstract, main, error, interfaces
 class Port(base.BasePort):
     """UDP port, listening for packets."""
 
-    __implements__ = base.BasePort.__implements__, interfaces.IUDPTransport
+    __implements__ = base.BasePort.__implements__, interfaces.IUDPTransport, interfaces.ISystemHandle
 
     addressFamily = socket.AF_INET
     socketType = socket.SOCK_DGRAM
     maxThroughput = 256 * 1024 # max bytes we read in one eventloop iteration
-    
+
     def __init__(self, port, proto, interface='', maxPacketSize=8192, reactor=None):
         """Initialize with a numeric port to listen on.
         """
@@ -70,10 +71,16 @@ class Port(base.BasePort):
         self.maxPacketSize = maxPacketSize
         self.interface = interface
         self.setLogStr()
-
+        self._connected = False
+        self._connectedAddr = None
+    
     def __repr__(self):
         return "<%s on %s>" % (self.protocol.__class__, self.port)
 
+    def getHandle(self):
+        """Return a socket object."""
+        return self.socket
+    
     def startListening(self):
         """Create and bind my socket, and begin listening on it.
 
@@ -82,7 +89,7 @@ class Port(base.BasePort):
         """
         self._bindSocket()
         self._connectToProtocol()
-    
+
     def _bindSocket(self):
         log.msg("%s starting on %s"%(self.protocol.__class__, self.port))
         try:
@@ -93,7 +100,7 @@ class Port(base.BasePort):
         self.connected = 1
         self.socket = skt
         self.fileno = self.socket.fileno
-    
+
     def _connectToProtocol(self):
         self.protocol.makeConnection(self)
         self.startReading()
@@ -119,25 +126,53 @@ class Port(base.BasePort):
             except:
                 log.deferr()
 
-    def write(self, datagram, addr):
+    def write(self, datagram, addr=None):
         """Write a datagram.
 
-        @param addr: should be a tuple (host, port)
+        @param addr: should be a tuple (ip, port), can be None in connected mode.
         """
-        try:
-            return self.socket.sendto(datagram, addr)
-        except socket.error, se:
-            no = se.args[0]
-            if no == EINTR:
-                return self.write(datagram, addr)
-            elif no == EMSGSIZE:
-                raise error.MessageLengthError, "message too long"
-            else:
-                raise
+        if self._connected:
+            assert addr in (None, self._connectedAddr)
+            try:
+                return self.socket.send(datagram)
+            except socket.error, se:
+                no = se.args[0]
+                if no == EINTR:
+                    return self.write(datagram)
+                elif no == EMSGSIZE:
+                    raise error.MessageLengthError, "message too long"
+                elif no == ECONNREFUSED:
+                    self.protocol.connectionRefused()
+                else:
+                    raise
+        else:
+            assert addr != None
+            if not addr[0].replace(".", "").isdigit():
+                warnings.warn("Please only pass IPs to write(), not hostnames", DeprecationWarning, stacklevel=2)
+            try:
+                return self.socket.sendto(datagram, addr)
+            except socket.error, se:
+                no = se.args[0]
+                if no == EINTR:
+                    return self.write(datagram, addr)
+                elif no == EMSGSIZE:
+                    raise error.MessageLengthError, "message too long"
+                else:
+                    raise
 
     def writeSequence(self, seq, addr):
         self.write("".join(seq), addr)
 
+    def connect(self, host, port):
+        """'Connect' to remote server."""
+        if self._connected:
+            raise RuntimeError, "already connected, reconnecting is not currently supported"
+        if not abstract.isIPAddress(host):
+            raise ValueError, "please pass only IP addresses, not domain names"
+        self._connected = True
+        self._connectedAddr = (host, port)
+        self.socket.connect((host, port))
+    
     def loseConnection(self):
         """Stop accepting connections on this port.
 
@@ -155,7 +190,7 @@ class Port(base.BasePort):
             result = None
         self.loseConnection()
         return result
-    
+
     def connectionLost(self, reason=None):
         """Cleans up my socket.
         """
@@ -183,23 +218,26 @@ class Port(base.BasePort):
 
     def getHost(self):
         """
-        Returns a tuple of ('INET_UDP', hostname, port), indicating
-        the servers address
+        Returns an IPv4Address.
+
+        This indicates the address from which I am connecting.
         """
-        return ('INET_UDP',)+self.socket.getsockname()
+        return address.IPv4Address('UDP', *(self.socket.getsockname() + ('INET_UDP',)))
 
 
 class ConnectedPort(Port):
-    """A connected UDP socket."""
+    """DEPRECATED.
+
+    A connected UDP socket."""
 
     __implements__ = Port.__implements__, interfaces.IUDPConnectedTransport
-        
+
     def __init__(self, (remotehost, remoteport), port, proto, interface='', maxPacketSize=8192, reactor=None):
         assert isinstance(proto, protocol.ConnectedDatagramProtocol)
         Port.__init__(self, port, proto, interface, maxPacketSize, reactor)
         self.remotehost = remotehost
         self.remoteport = remoteport
-    
+
     def startListening(self):
         self._bindSocket()
         if abstract.isIPAddress(self.remotehost):
@@ -213,7 +251,7 @@ class ConnectedPort(Port):
         self.realAddress = addr
         self.socket.connect((addr, self.remoteport))
         self._connectToProtocol()
-    
+
     def connectionFailed(self, reason):
         self.loseConnection()
         self.protocol.connectionFailed(reason)
@@ -258,19 +296,16 @@ class ConnectedPort(Port):
         Returns a tuple of ('INET_UDP', hostname, port), indicating
         the remote address.
         """
-        return ('INET_UDP', self.remotehost, self.remoteport)
+        return address.IPv4Address('UDP', self.remotehost, self.remoteport, 'INET_UDP')
 
 
 class MulticastMixin:
-    """Implement multicast functionality.
-
-    Initial implementation, probably needs some changes for Windows support.
-    """
+    """Implement multicast functionality."""
 
     def getOutgoingInterface(self):
         i = self.socket.getsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_IF)
         return socket.inet_ntoa(struct.pack("@i", i))
-    
+
     def setOutgoingInterface(self, addr):
         """Returns Deferred of success."""
         return self.reactor.resolve(addr).addCallback(self._setInterface)
@@ -279,17 +314,17 @@ class MulticastMixin:
         i = socket.inet_aton(addr)
         self.socket.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_IF, i)
         return 1
-    
+
     def getLoopbackMode(self):
         return self.socket.getsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_LOOP)
-    
+
     def setLoopbackMode(self, mode):
         mode = struct.pack("b", operator.truth(mode))
         self.socket.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_LOOP, mode)
 
     def getTTL(self):
         return self.socket.getsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL)
-    
+
     def setTTL(self, ttl):
         ttl = struct.pack("b", ttl)
         self.socket.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, ttl)
@@ -310,7 +345,7 @@ class MulticastMixin:
             cmd = socket.IP_DROP_MEMBERSHIP
         self.socket.setsockopt(socket.IPPROTO_IP, cmd, addr + interface)
         return 1
-    
+
     def leaveGroup(self, addr, interface=""):
         """Leave multicast group, return Deferred of success."""
         return self.reactor.resolve(addr).addCallback(self._joinAddr1, interface, 0)
@@ -323,6 +358,8 @@ class MulticastPort(MulticastMixin, Port):
 
 
 class ConnectedMulticastPort(MulticastMixin, ConnectedPort):
-    """Connected UDP Port that supports multicasting."""
+    """DEPRECATED.
+
+    Connected UDP Port that supports multicasting."""
 
     __implements__ = ConnectedPort.__implements__, interfaces.IMulticastTransport
