@@ -16,9 +16,12 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 from twisted.protocols import dns
+from twisted.persisted import styles
 from twisted.python import failure
 from twisted.internet import interfaces, defer
 
+from twisted.names import common
+
 def searchFileFor(file, name):
     try:
         fp = open(file)
@@ -38,35 +41,34 @@ def searchFileFor(file, name):
     return None
 
 
-class Resolver:
-    """A resolver that services hosts(5) format files."""
 
-    __implements__ = (interfaces.IResolverSimple,)
+class Resolver(common.ResolverBase, styles.Versioned):
+    """A resolver that services hosts(5) format files."""
+    #TODO: IPv6 support
+
+    persistenceVersion = 1
+
+    def upgradeToVersion1(self):
+        # <3 exarkun
+        self.typeToMethod = {}
+        for (k, v) in common.typeToMethod.items():
+            self.typeToMethod[k] = getattr(self, v)
+    
 
     def __init__(self, file='/etc/hosts', ttl = 60 * 60):
+        common.ResolverBase.__init__(self)
         self.file = file
         self.ttl = ttl
 
 
-    def query(self, query, timeout = None):
-        return self._lookup(str(query.name), query.type, query.type, timeout)
-
-
-    def _lookup(self, name, cls, type, timeout):
-        if cls != dns.IN or type != dns.A:
-            return defer.fail(failure.Failure(NotImplementedError(type, cls)))
-        return self.lookupAddress(
-            name, timeout
-        )
-
-
     def lookupAddress(self, name, timeout = None):
         res = searchFileFor(self.file, name)
         if res:
             return defer.succeed([
-                [dns.RRHeader(name, dns.A, dns.IN, self.ttl)], (), ()
+                (dns.RRHeader(name, dns.A, dns.IN, self.ttl, dns.Record_A(res, self.ttl)),), (), ()
             ])
         return defer.fail(failure.Failure(dns.DomainError(name)))
 
 
-    lookupAllRecords = lookupAddress
+    # When we put IPv6 support in, this'll need a real impl
+    lookupAllRecords = lookupAddress 
