@@ -28,6 +28,10 @@ a time.
 
 No files should be placed in the working directory of a DirDBM save those
 created by the DirDBM itself!
+
+API Stability: stable
+
+Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
 """
 
 
