@@ -41,7 +41,7 @@ from twisted.internet.protocol import ClientFactory, ServerFactory, Protocol, \
 from twisted import application, internet, python
 from twisted.python import failure, log, components
 
-from twisted.cred import error, portal, checkers, credentials
+from twisted.cred import error as cred_error, portal, checkers, credentials
 
 # constants
 
@@ -823,7 +823,7 @@ class FTP(object, basic.LineReceiver, policies.TimeoutMixin):
         self.reply(USR_LOGGED_IN_PROCEED)
 
     def _ebLogin(self, failure):
-        r = failure.trap(error.UnauthorizedLogin, TLDNotSetInRealmError)
+        r = failure.trap(cred_error.UnauthorizedLogin, TLDNotSetInRealmError)
         if r == TLDNotSetInRealmError:
             log.debug(failure.getErrorMessage())
             self.reply(REQ_ACTN_NOT_TAKEN, 'internal server error')
@@ -1173,8 +1173,11 @@ class IFTPShell(components.Interface):
         pass
 
 
-
-import pwd, grp
+try:
+    import pwd, grp
+except ImportError:
+    # not a POSIX platform
+    pass
 
 def _callWithDefault(default, _f, *_a, **_kw):
     try:
@@ -1227,8 +1230,9 @@ def _testPermissions(uid, gid, spath, mode='r'):
             return True
     return False   
 
+
 class FTPAnonymousShell(object):
-    """"""
+    """Only works on POSIX platforms at the moment."""
     __implements__ = (IFTPShell,)
 
     uid      = None        # uid of anonymous user for shell
@@ -1537,8 +1541,6 @@ class ProtocolWrapper(Protocol):
         self.original.connectionLost(reason)
         # Signal that transfer has completed
         self.deferred.callback(None)
-    def connectionFailed(self):
-        self.deferred.errback(Failure(FTPError('Connection failed')))
 
 
 class SenderProtocol(Protocol):
@@ -1640,12 +1642,20 @@ class FTPClient(basic.LineReceiver):
         self.response = []
 
         self.passive = passive
+        self._failed = 0
 
     def fail(self, error):
         """Disconnect, and also give an error to any queued deferreds."""
         self.transport.loseConnection()
-        while self.actionQueue:
-            ftpCommand = self.popCommandQueue()
+        self._fail(error)
+
+    def _fail(self, error):
+        """Errback all queued deferreds."""
+        if self._failed:
+            # We're recursing; bail out here for simplicity
+            return error
+        self._failed = 1
+        for ftpCommand in self.actionQueue:
             ftpCommand.fail(Failure(ConnectionLost('FTP connection lost', error)))
         return error
 
@@ -1708,13 +1718,13 @@ class FTPClient(basic.LineReceiver):
         else:
             return None
 
-    def receiveFromConnection(self, command, protocol):
+    def receiveFromConnection(self, commands, protocol):
         """
         Retrieves a file or listing generated by the given command,
         feeding it to the given protocol.
 
-        @param command: string of an FTP command to execute then receive the
-            results of (e.g. LIST, RETR)
+        @param command: list of strings of FTP commands to execute then receive
+            the results of (e.g. LIST, RETR)
         @param protocol: A L{Protocol} *instance* e.g. an
             L{FTPFileListProtocol}, or something that can be adapted to one.
             Typically this will be an L{IConsumer} implemenation.
@@ -1723,9 +1733,9 @@ class FTPClient(basic.LineReceiver):
         """
         protocol = IProtocol(protocol)
         wrapper = ProtocolWrapper(protocol, Deferred())
-        return self._openDataConnection(command, wrapper)
+        return self._openDataConnection(commands, wrapper)
 
-    def sendToConnection(self, command):
+    def sendToConnection(self, commands):
         """XXX
         
         @returns: A tuple of two L{Deferred}s:
@@ -1735,14 +1745,16 @@ class FTPClient(basic.LineReceiver):
                   - L{Deferred} list of control-connection responses.
         """
         s = SenderProtocol()
-        r = self._openDataConnection(command, s)
+        r = self._openDataConnection(commands, s)
         return (s.connectedDeferred, r)
 
-    def _openDataConnection(self, command, protocol):
+    def _openDataConnection(self, commands, protocol):
         """
         This method returns a DeferredList.
         """
-        cmd = FTPCommand(command, public=1)
+        cmds = [FTPCommand(command, public=1) for command in commands]
+        cmdsDeferred = DeferredList([cmd.deferred for cmd in cmds], 
+                                    fireOnOneErrback=True)
 
         if self.passive:
             # Hack: use a mutable object to sneak a variable out of the 
@@ -1758,7 +1770,8 @@ class FTPClient(basic.LineReceiver):
                         self.protocol.factory = self
                         return self.protocol
                     def clientConnectionFailed(self, connector, reason):
-                        self.protocol.connectionFailed()
+                        e = FTPError('Connection Failed', reason)
+                        self.protocol.deferred.errback(e)
                 f = _Factory()
                 f.protocol = protocol
                 _mutable[0] = reactor.connectTCP(host, port, f)
@@ -1767,8 +1780,8 @@ class FTPClient(basic.LineReceiver):
             self.queueCommand(pasvCmd)
             pasvCmd.deferred.addCallback(doPassive).addErrback(self.fail)
 
-            results = [cmd.deferred, pasvCmd.deferred, protocol.deferred]
-            d = DeferredList(results, fireOnOneErrback=1)
+            results = [cmdsDeferred, pasvCmd.deferred, protocol.deferred]
+            d = DeferredList(results, fireOnOneErrback=1, consumeErrors=1)
 
             # Ensure the connection is always closed
             def close(x, m=_mutable):
@@ -1800,12 +1813,13 @@ class FTPClient(basic.LineReceiver):
             portCmd.fail = lambda error: error
             
             # Ensure that the connection always gets closed
-            cmd.deferred.addErrback(lambda e, pc=portCmd: pc.fail(e) or e)
+            cmdsDeferred.addErrback(lambda e, pc=portCmd: pc.fail(e) or e)
 
-            results = [cmd.deferred, portCmd.deferred, portCmd.transferDeferred]
+            results = [cmdsDeferred, portCmd.deferred, portCmd.transferDeferred]
             d = DeferredList(results, fireOnOneErrback=1)
                               
-        self.queueCommand(cmd)
+        for cmd in cmds:
+            self.queueCommand(cmd)
         return d
 
     def generatePortCommand(self, portCmd):
@@ -1846,7 +1860,7 @@ class FTPClient(basic.LineReceiver):
         # Escape newline characters
         return string.replace(path, '\n', '\0')
 
-    def retrieveFile(self, path, protocol):
+    def retrieveFile(self, path, protocol, offset=0):
         """Retrieve a file from the given path
 
         This method issues the 'RETR' FTP command.
@@ -1856,14 +1870,18 @@ class FTPClient(basic.LineReceiver):
 
         @param path: path to file that you wish to receive.
         @param protocol: a L{Protocol} instance.
+        @param offset: offset to start downloading from
 
         @returns: L{Deferred}
         """
-        return self.receiveFromConnection('RETR ' + self.escapePath(path), protocol)
+        cmds = ['RETR ' + self.escapePath(path)]
+        if offset:
+            cmds.insert(0, ('REST ' + str(offset)))
+        return self.receiveFromConnection(cmds, protocol)
 
     retr = retrieveFile
 
-    def storeFile(self, path):
+    def storeFile(self, path, offset=0):
         """Store a file at the given path.
 
         This method issues the 'STOR' FTP command.
@@ -1874,8 +1892,10 @@ class FTPClient(basic.LineReceiver):
                     is completely transferred.
                   - L{Deferred} list of control-connection responses.
         """
-        
-        return self.sendToConnection('STOR ' + self.escapePath(path))
+        cmds = ['STOR ' + self.escapePath(path)]
+        if offset:
+            cmds.insert(0, ('REST ' + str(offset)))
+        return self.sendToConnection(cmds)
 
     stor = storeFile
         
@@ -1893,7 +1913,7 @@ class FTPClient(basic.LineReceiver):
         """
         if path is None:
             path = ''
-        return self.receiveFromConnection('LIST ' + self.escapePath(path), protocol)
+        return self.receiveFromConnection(['LIST ' + self.escapePath(path)], protocol)
         
     def nlst(self, path, protocol):
         """Retrieve a short file listing into the given protocol instance.
@@ -1907,7 +1927,7 @@ class FTPClient(basic.LineReceiver):
         """
         if path is None:
             path = ''
-        return self.receiveFromConnection('NLST ' + self.escapePath(path), protocol)
+        return self.receiveFromConnection(['NLST ' + self.escapePath(path)], protocol)
 
     def queueStringCommand(self, command, public=1):
         """Queues a string to be issued as an FTP command
@@ -1993,7 +2013,10 @@ class FTPClient(basic.LineReceiver):
             
         # Run the next command
         self.sendNextCommand()
-        
+
+    def connectionLost(self, reason):
+        self._fail(reason)
+
 
 class FTPFileListProtocol(basic.LineReceiver):
     """Parser for standard FTP file listings
@@ -2002,17 +2025,20 @@ class FTPFileListProtocol(basic.LineReceiver):
 
         -rw-r--r--   1 root     other        531 Jan 29 03:26 README
 
-    If you need different evil for a wacky FTP server, you can override this.
+    If you need different evil for a wacky FTP server, you can
+    override either C{fileLinePattern} or C{parseDirectoryLine()}.
 
     It populates the instance attribute self.files, which is a list containing
     dicts with the following keys (examples from the above line):
-        - filetype: e.g. 'd' for directories, or '-' for an ordinary file
-        - perms:    e.g. 'rw-r--r--'
-        - owner:    e.g. 'root'
-        - group:    e.g. 'other'
-        - size:     e.g. 531
-        - date:     e.g. 'Jan 29 03:26'
-        - filename: e.g. 'README'
+        - filetype:   e.g. 'd' for directories, or '-' for an ordinary file
+        - perms:      e.g. 'rw-r--r--'
+        - nlinks:     e.g. 1
+        - owner:      e.g. 'root'
+        - group:      e.g. 'other'
+        - size:       e.g. 531
+        - date:       e.g. 'Jan 29 03:26'
+        - filename:   e.g. 'README'
+        - linktarget: e.g. 'some/file'
 
     Note that the 'date' value will be formatted differently depending on the
     date.  Check U{http://cr.yp.to/ftp.html} if you really want to try to parse
@@ -2021,9 +2047,10 @@ class FTPFileListProtocol(basic.LineReceiver):
     @ivar files: list of dicts describing the files in this listing
     """
     fileLinePattern = re.compile(
-        r'^(?P<filetype>.)(?P<perms>.{9})\s+\d*\s*'
+        r'^(?P<filetype>.)(?P<perms>.{9})\s+(?P<nlinks>\d*)\s*'
         r'(?P<owner>\S+)\s+(?P<group>\S+)\s+(?P<size>\d+)\s+'
-        r'(?P<date>... .. ..:..)\s+(?P<filename>.*?)\r?$'
+        r'(?P<date>...\s+\d+\s+[\d:]+)\s+(?P<filename>([^ ]|\\ )*?)'
+        r'( -> (?P<linktarget>[^\r]*))?\r?$'
     )
     delimiter = '\n'
 
@@ -2031,11 +2058,56 @@ class FTPFileListProtocol(basic.LineReceiver):
         self.files = []
 
     def lineReceived(self, line):
+        d = self.parseDirectoryLine(line)
+        if d is None:
+            self.unknownLine(line)
+        else:
+            self.addFile(d)
+
+    def parseDirectoryLine(self, line):
+        """Return a dictionary of fields, or None if line cannot be parsed.
+
+        @param line: line of text expected to contain a directory entry
+        @type line: str
+
+        @return: dict
+        """
         match = self.fileLinePattern.match(line)
-        if match:
-            dict = match.groupdict()
-            dict['size'] = int(dict['size'])
-            self.files.append(dict)
+        if match is None:
+            return None
+        else:
+            d = match.groupdict()
+            d['filename'] = d['filename'].replace(r'\ ', ' ')
+            d['nlinks'] = int(d['nlinks'])
+            d['size'] = int(d['size'])
+            if d['linktarget']:
+                d['linktarget'] = d['linktarget'].replace(r'\ ', ' ')
+            return d
+
+    def addFile(self, info):
+        """Append file information dictionary to the list of known files.
+
+        Subclasses can override or extend this method to handle file
+        information differently without affecting the parsing of data
+        from the server.
+
+        @param info: dictionary containing the parsed representation
+                     of the file information
+        @type info: dict
+        """
+        self.files.append(info)
+
+    def unknownLine(self, line):
+        """Deal with received lines which could not be parsed as file
+        information.
+
+        Subclasses can override this to perform any special processing
+        needed.
+
+        @param line: unparsable line as received
+        @type line: str
+        """
+        pass
 
 def parsePWDResponse(response):
     """Returns the path from a response to a PWD command.
