@@ -33,7 +33,7 @@ Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
 # system imports
 from cStringIO import StringIO
 import tempfile
-import base64
+import base64, binascii
 import cgi
 import socket
 import math
@@ -46,9 +46,9 @@ import os
 import basic
 
 # twisted imports
-from twisted.internet import interfaces, reactor, protocol
+from twisted.internet import interfaces, reactor, protocol, address
 from twisted.protocols import policies
-from twisted.python import log
+from twisted.python import log, components
 try: # try importing the fast, C version
     from twisted.protocols._c_urlarg import unquote
 except ImportError:
@@ -268,13 +268,15 @@ def stringToDatetime(dateString):
     return int(timegm(year, month, day, hour, min, sec))
 
 def toChunk(data):
-    """Convert string to a chunk."""
-    return "%x\r\n%s\r\n" % (len(data), data)
+    """Convert string to a chunk.
+    
+    @returns: a tuple of strings representing the chunked encoding of data"""
+    return ("%x\r\n" % len(data), data, "\r\n")
 
 def fromChunk(data):
     """Convert chunk to string.
 
-    Returns tuple (result, remaining), may raise ValueError.
+    @returns: tuple (result, remaining), may raise ValueError.
     """
     prefix, rest = data.split('\r\n', 1)
     length = int(prefix, 16)
@@ -346,7 +348,8 @@ class HTTPClient(basic.LineReceiver):
             self.handleStatus(version, status, message)
             return
         if line:
-            key, val = line.split(': ', 1)
+            key, val = line.split(':', 1)
+            val = val.lstrip()
             self.handleHeader(key, val)
             if key.lower() == 'content-length':
                 self.length = int(val)
@@ -389,16 +392,20 @@ NO_BODY_CODES = (204, 304)
 class Request:
     """A HTTP request.
 
-    @cvar method: The HTTP method that was used.
-    @cvar uri: The full URI that was requested (includes arguments).
+    Subclasses should override the process() method to determine how
+    the request will be processed.
+    
+    @ivar method: The HTTP method that was used.
+    @ivar uri: The full URI that was requested (includes arguments).
     @ivar path: The path only (arguments not included).
     @ivar args: All of the arguments, including URL and POST arguments.
     @type args: A mapping of strings (the argument names) to lists of values.
                 i.e., ?foo=bar&foo=baz&quux=spam results in
                 {'foo': ['bar', 'baz'], 'quux': ['spam']}.
+    @ivar received_headers: All received headers
     """
 
-    __implements__ = interfaces.IConsumer
+    __implements__ = interfaces.IConsumer,
 
     producer = None
     finished = 0
@@ -412,7 +419,8 @@ class Request:
     sentLength = 0 # content-length of response, or total bytes sent via chunking
     etag = None
     lastModified = None
-
+    _forceSSL = 0
+    
     def __init__(self, channel, queued):
         """
         @param channel: the channel we're connected to.
@@ -586,7 +594,8 @@ class Request:
         self.transport.write('%s %s %s\r\n\r\n' % (self.clientproto, code, resp))
 
 
-    # http request methods
+    # The following is the public interface that people should be
+    # writing to.
 
     def getHeader(self, key):
         """Get a header that was sent from the network.
@@ -598,10 +607,6 @@ class Request:
         """
         return self.received_cookies.get(key)
 
-
-    # The following is the public interface that people should be
-    # writing to.
-
     def finish(self):
         """We are finished writing data."""
         if self.finished:
@@ -677,7 +682,7 @@ class Request:
         self.sentLength = self.sentLength + len(data)
         if data:
             if self.chunked:
-                self.transport.write(toChunk(data))
+                self.transport.writeSequence(toChunk(data))
             else:
                 self.transport.write(data)
 
@@ -827,32 +832,42 @@ class Request:
 
         This method is experimental.
         """
-        if ssl:
-            method = 'SSL'
-        else:
-            method = 'INET'
+        self._forceSSL = ssl
         self.received_headers["host"] = host
-        self.host = (method, host, port)
+        self.host = address.IPv4Address("TCP", host, port)
 
     def getClientIP(self):
-        if self.client[0] in ('INET', 'SSL'):
-            return self.client[1]
+        if isinstance(self.client, address.IPv4Address):
+            return self.client.host
         else:
             return None
 
     def isSecure(self):
-        return (self.host[0] == 'SSL')
+        if self._forceSSL:
+            return True
+        transport = getattr(getattr(self, 'channel', None), 'transport', None)
+        if interfaces.ISSLTransport(transport, default=None) is not None:
+            return True
+        return False
 
     def _authorize(self):
         # Authorization, (mostly) per the RFC
         try:
             authh = self.getHeader("Authorization")
+            if not authh:
+                self.user = self.password = ''
+                return
             bas, upw = authh.split()
+            if bas.lower() != "basic":
+                raise ValueError
             upw = base64.decodestring(upw)
-            self.user, self.password = upw.split(':')
-        except:
+            self.user, self.password = upw.split(':', 1)
+        except (binascii.Error, ValueError):
             self.user = self.password = ""
-
+        except:
+            log.err()
+            self.user = self.password = ""
+    
     def getUser(self):
         try:
             return self.user
