@@ -1448,12 +1448,18 @@ class NotificationClient(MSNEventBase):
         A default callback is added to the returned
         Deferred which updates the contacts attribute of
         the factory with the new contact information.
+        If you are adding a contact to the forward list
+        and you want to associate this contact with multiple
+        groups then you will need to call this method for each
+        group you would like to add them to, changing the groupID
+        parameter. The default callback will take care of updating
+        the group information on the factory's contact list.
 
         @param listType: (as defined by the *_LIST constants)
         @param userHandle: the user handle (passport) of the contact
                            that is being added
         @param groupID: the group ID for which to associate this contact
-                        with. (default 0 - no group). Groups are only
+                        with. (default 0 - default group). Groups are only
                         valid for FORWARD_LIST.
 
         @return: A Deferred, the callback for which will be called when
@@ -1486,7 +1492,12 @@ class NotificationClient(MSNEventBase):
         Used to remove a contact from the desired list.
         A default callback is added to the returned deferred
         which updates the contacts attribute of the factory
-        to reflect the new contact information.
+        to reflect the new contact information. If you are
+        removing from the forward list then you will need to
+        supply a groupID, if the contact is in more than one
+        group then they will only be removed from this group
+        and not the entire forward list, but if this is their
+        only group they will be removed from the whole list.
 
         @param listType: (as defined by the *_LIST constants)
         @param userHandle: the user handle (passport) of the
@@ -1514,8 +1525,14 @@ class NotificationClient(MSNEventBase):
             l = self.factory.contacts
             l.version = r[2]
             c = l.getContact(r[1])
-            c.removeFromList(r[0])
-            if c.lists == 0: l.remContact(c.userHandle)
+            group = r[3]
+            shouldRemove = 1
+            if group: # they may not have been removed from the list
+                c.groups.remove(group)
+                if c.groups: shouldRemove = 0
+            if shouldRemove:
+                c.removeFromList(r[0])
+                if c.lists == 0: l.remContact(c.userHandle)
             return r
         return d.addCallback(_cb)
 
@@ -2251,14 +2268,22 @@ errorCodes = {
     217 : "User not online",
     218 : "Already in mode",
     219 : "User is in the opposite list",
+    223 : "Too many groups",
+    224 : "Invalid group",
+    225 : "User not in group",
+    229 : "Group name too long",
+    230 : "Cannot remove group 0",
+    231 : "Invalid group",
     280 : "Switchboard failed",
     281 : "Transfer to switchboard failed",
 
     300 : "Required field missing",
+    301 : "Too many FND responses",
     302 : "Not logged in",
 
     500 : "Internal server error",
     501 : "Database server error",
+    502 : "Command disabled",
     510 : "File operation failed",
     520 : "Memory allocation failed",
     540 : "Wrong CHL value sent to server",
@@ -2268,18 +2293,34 @@ errorCodes = {
     602 : "Peer nameserver is down",
     603 : "Database connection failed",
     604 : "Server is going down",
+    605 : "Server unavailable",
 
     707 : "Could not create connection",
+    710 : "Invalid CVR parameters",
     711 : "Write is blocking",
     712 : "Session is overloaded",
     713 : "Too many active users",
     714 : "Too many sessions",
     715 : "Not expected",
     717 : "Bad friend file",
+    731 : "Not expected",
 
+    800 : "Requests too rapid",
+
+    910 : "Server too busy",
     911 : "Authentication failed",
+    912 : "Server too busy",
     913 : "Not allowed when offline",
+    914 : "Server too busy",
+    915 : "Server too busy",
+    916 : "Server too busy",
+    917 : "Server too busy",
+    918 : "Server too busy",
+    919 : "Server too busy",
     920 : "Not accepting new users",
+    921 : "Server too busy",
+    922 : "Server too busy",
+    923 : "No parent consent",
     924 : "Passport account not yet verified"
 
 }
@@ -2303,7 +2344,7 @@ statusCodes = {
 listIDToCode = {
 
     FORWARD_LIST : 'fl',
-    BLOCK_LIST : 'bl',
+    BLOCK_LIST   : 'bl',
     ALLOW_LIST   : 'al',
     REVERSE_LIST : 'rl'
 
