@@ -526,14 +526,22 @@ class XMLParser(Protocol):
 
     def begin_entityref(self, byte):
         self.erefbuf = ''
-
+        self.erefextra = '' # extra bit for lenient mode
+    
     def do_entityref(self, byte):
-        if byte.isspace():
+        if byte.isspace() or byte == "<":
             if self.beExtremelyLenient:
+                # '&foo' probably was '&amp;foo'
+                if self.erefbuf and self.erefbuf != "amp":
+                    self.erefextra = self.erefbuf
                 self.erefbuf = "amp"
-                return 'bodydata'
+                if byte == "<":
+                    return "tagstart"
+                else:
+                    self.erefextra += byte
+                    return 'spacebodydata'
             self._parseError("Bad entity reference")
-        if byte != ';':
+        elif byte != ';':
             self.erefbuf += byte
         else:
             return 'bodydata'
@@ -541,6 +549,14 @@ class XMLParser(Protocol):
     def end_entityref(self):
         self.gotEntityReference(self.erefbuf)
 
+    # hacky support for space after & in entityref in beExtremelyLenient
+    # state should only happen in that case
+    def begin_spacebodydata(self, byte):
+        self.bodydata = self.erefextra
+        self.erefextra = None
+    do_spacebodydata = do_bodydata
+    end_spacebodydata = end_bodydata
+    
     # Sorta SAX-ish API
     
     def gotTagStart(self, name, attributes):
