@@ -1,16 +1,16 @@
 
 # Twisted, the Framework of Your Internet
 # Copyright (C) 2001 Matthew W. Lefkowitz
-# 
+#
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of version 2.1 of the GNU Lesser General Public
 # License as published by the Free Software Foundation.
-# 
+#
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -20,11 +20,15 @@ Implements a AOL Instant Messenger TOC server and client, using the Twisted fram
 
 TODO:
 info,dir: see how gaim connects for this...it may never work if it tries to connect to the aim server automatically
+
+This module is stable, but deprecated.
+
+Maintainer: U{Paul Swartz<mailto:z3p@twistedmatrix.com>}
 """
 
 # twisted imports
-import protocol
-from twisted.internet import tcp
+from twisted.internet import reactor, protocol
+from twisted.python import log
 
 # base imports
 import struct
@@ -37,6 +41,8 @@ import StringIO
 SIGNON,DATA,ERROR,SIGNOFF,KEEP_ALIVE=range(1,6)
 PERMITALL,DENYALL,PERMITSOME,DENYSOME=range(1,5)
 
+DUMMY_CHECKSUM = -559038737 # 0xdeadbeef
+
 def quote(s):
     rep=['\\','$','{','}','[',']','(',')','"']
     for r in rep:
@@ -91,13 +97,13 @@ def roast(pw):
     return string.lower(ro)
 
 def checksum(b):
-    return 0xdeadbeef # do it like gaim does, since the checksum
+    return DUMMY_CHECKSUM # do it like gaim does, since the checksum
                       # formula doesn't work
 ##    # used in file transfers
 ##    check0 = check1 = 0x00ff
 ##    for i in range(len(b)):
 ##        if i%2:
-##            if ord(b[i])>check1: 
+##            if ord(b[i])>check1:
 ##                check1=check1+0x100 # wrap
 ##                if check0==0:
 ##                    check0=0x00ff
@@ -122,7 +128,7 @@ def checksum(b):
 ##    return checksum
 
 def checksum_file(f):
-    return 0xdeadbeef # do it like gaim does, since the checksum
+    return DUMMY_CHECKSUM # do it like gaim does, since the checksum
                       # formula doesn't work
 ##    check0=check1=0x00ff
 ##    i=0
@@ -132,7 +138,7 @@ def checksum_file(f):
 ##        for char in b:
 ##            i=not i
 ##            if i:
-##                if ord(char)>check1: 
+##                if ord(char)>check1:
 ##                    check1=check1+0x100 # wrap
 ##                    if check0==0:
 ##                        check0=0x00ff
@@ -185,14 +191,14 @@ class TOC(protocol.Protocol):
         self.signontime=0
         self.idletime=0
         self.userinfo="<br>"
-        self.userclass=" O" 
+        self.userclass=" O"
         self.away=""
         self.saved=None
 
     def _debug(self,data):
-        print data
+        log.msg(data)
 
-    def connectionLost(self):
+    def connectionLost(self, reason):
         self._debug("dropped connection from %s" % self.username)
         try:
             del self.factory.users[self.username]
@@ -215,7 +221,7 @@ class TOC(protocol.Protocol):
         self._debug(data)
         if type==DATA:
             data=data+"\000"
-        length=len(data) 
+        length=len(data)
         send=send+struct.pack("!BHH",type,self._ourseqnum,length)
         send=send+data
         self._ourseqnum=self._ourseqnum+1
@@ -250,7 +256,7 @@ class TOC(protocol.Protocol):
         the FLAP is the basic TOC message format, and is logically equivilant to a packet in TCP
         """
         if self._buf=='': return None
-        if self._buf[0]!="*": 
+        if self._buf[0]!="*":
             raise TOCParseError
         if len(self._buf)<6: return None
         foo,type,seqnum,length=struct.unpack("!BBHH",self._buf[:6])
@@ -268,7 +274,6 @@ class TOC(protocol.Protocol):
     #        get,username,http=string.split(line," ",2)
     #    except:
     #        return "Web" # not enough data
-    #    print get,username,http
     #    foo,type,username=string.split(username,"/")
     #    if type=="info":
     #        user=self.factory.users[username]
@@ -317,14 +322,14 @@ class TOC(protocol.Protocol):
         password=unroast(data[4])
         if not(self.authorize(data[1],int(data[2]),data[3],password)):
             self.sendError(BAD_NICKNAME)
-            self.connectionLost()
+            self.transport.loseConnection()
             return
         self.sendFlap(DATA,"SIGN_ON:TOC1.0")
         self.sendFlap(DATA,"NICK:%s"%self.saved.nick)
         self.sendFlap(DATA,"CONFIG:%s"%self.saved.config)
         # sending user configuration goes here
         return "Connected"
-   
+
     def authorize(self,server,port,username,password):
         if self.saved.password=="":
             self.saved.password=password
@@ -351,7 +356,7 @@ class TOC(protocol.Protocol):
         return "Connected"
 
     def toc_unknown(self,tocname,data):
-        self._debug("unknown! %s %s" % (tocname,data))  
+        self._debug("unknown! %s %s" % (tocname,data))
 
     def toc_init_done(self,data):
         """
@@ -415,7 +420,7 @@ class TOC(protocol.Protocol):
     def toc_add_buddy(self,data):
         """
         adds users to the buddy list
-        
+
         toc_add_buddy <buddyname1> [<buddyname2>] [<buddyname3>]...
         """
         buddies=map(normalize,string.split(data," "))
@@ -634,7 +639,7 @@ class TOC(protocol.Protocol):
     def canContact(self,user):
         if self.permitmode==PERMITALL: return 1
         elif self.permitmode==DENYALL: return 0
-        elif self.permitmode==PERMITSOME: 
+        elif self.permitmode==PERMITSOME:
             if user.username in self.permitlist: return 1
             else: return 0
         elif self.permitmode==DENYSOME:
@@ -717,7 +722,7 @@ class Chatroom:
 
     def join(self,user):
         if user in self.users:
-            return 
+            return
         self.users.append(user)
         user.chatJoin(self)
 
@@ -786,7 +791,7 @@ MAXARGS["ADMIN_PASSWD_STATUS"]=0
 
 class TOCClient(protocol.Protocol):
     def __init__(self,username,password,authhost="login.oscar.aol.com",authport=5190):
-        
+
         self.username=normalize(username) # our username
         self._password=password # our password
         self._mode="SendNick" # current mode
@@ -805,7 +810,7 @@ class TOCClient(protocol.Protocol):
         self._awaymessage=''
 
     def _debug(self,data):
-        print data
+        log.msg(data)
 
     def sendFlap(self,type,data):
         if type==DATA:
@@ -849,7 +854,7 @@ class TOCClient(protocol.Protocol):
         self._debug("connection made! %s" % self.transport)
         self.transport.write("FLAPON\r\n\r\n")
 
-    def connectionLost(self):
+    def connectionLost(self, reason):
         self._debug("connection lost!")
         self._online=0
 
@@ -872,6 +877,9 @@ class TOCClient(protocol.Protocol):
     def modeData(self,flap):
         if not flap[1]:
             return
+        if not ':' in flap[1]:
+            self._debug("bad SNAC:%s"%(flap[1]))
+            return
         command,rest=string.split(flap[1],":",1)
         if MAXARGS.has_key(command):
             maxsplit=MAXARGS[command]
@@ -913,11 +921,11 @@ class TOCClient(protocol.Protocol):
         """
         CONFIG:<config>
         format of config data:
-            g: group.  all users until next g or end of config are in this group
-            b: buddy
-            p: person on the permit list
-            d: person on the deny list
-            m: permit/deny mode (1: permit all, 2: deny all, 3: permit some, 4: deny some)
+            - g: group.  all users until next g or end of config are in this group
+            - b: buddy
+            - p: person on the permit list
+            - d: person on the deny list
+            - m: permit/deny mode (1: permit all, 2: deny all, 3: permit some, 4: deny some)
         """
         data=data[0]
         if data and data[0]=="{":data=data[1:-1]
@@ -1057,11 +1065,11 @@ class TOCClient(protocol.Protocol):
         self._cookies[cookie]=[user,SEND_FILE_UID,pip,port,{'name':name}]
         self.rvousProposal("send",cookie,user,vip,port,description=description,
                            name=name,files=numfiles,size=size)
-        
+
     def tocGET_FILE(self,user,cookie,seq,pip,vip,port,tlvs):
         return
         # XXX add this back in
-        #tcp.Client(pip,port,GetFileTransfer(self,cookie,os.path.expanduser("~")))
+        #reactor.clientTCP(pip,port,GetFileTransfer(self,cookie,os.path.expanduser("~")))
         #self.rvous_accept(user,cookie,GET_FILE_UID)
 
     def onLine(self):
@@ -1325,7 +1333,7 @@ class TOCClient(protocol.Protocol):
         message := the message, or '' to come back from awayness
         """
         self._awaymessage=message
-        if message: 
+        if message:
             message=' '+quote(message)
         self.sendFlap(2,"toc_set_away%s"%message)
 
@@ -1390,7 +1398,7 @@ class TOCClient(protocol.Protocol):
         self.sendFlap(2,"toc_rvous_accept %s %s %s" % (normalize(user),
                                                      cookie,uuid))
         if uuid==SEND_FILE_UID:
-            tcp.Client(pip,port,SendFileTransfer(self,cookie,user,d["name"]))
+            protocol.ClientCreator(reactor, SendFileTransfer,self,cookie,user,d["name"]).connectTCP(pip,port)
 
     def rvous_cancel(self,cookie):
         user,uuid,pip,port,d=self._cookies[cookie]
@@ -1432,7 +1440,7 @@ class SendFileTransfer(protocol.Protocol):
             self.hdr[2]=0x204
             self.hdr[7]=self.hdr[7]-1
             self.hdr[9]=self.hdr[9]-1
-            self.hdr[19]=0xdeadbeef # XXX really calculate this
+            self.hdr[19]=DUMMY_CHECKSUM # XXX really calculate this
             self.hdr[18]=self.hdr[18]+1
             self.hdr[21]="\000"
             self.transport.write(apply(struct.pack,[self.header_fmt]+self.hdr))
@@ -1440,7 +1448,7 @@ class SendFileTransfer(protocol.Protocol):
             if self.hdr[7]==0:
                 self.transport.loseConnection()
 
-        
+
 class GetFileTransfer(protocol.Protocol):
     header_fmt="!4s 2H 8s 6H 10I 32s 3c 69s 16s 2H 64s"
     def __init__(self,client,cookie,dir):
@@ -1469,7 +1477,7 @@ class GetFileTransfer(protocol.Protocol):
              checksum(self.listing),0,0,0,0,0,0,"OFT_Windows ICBMFT V1.1 32",
              "\002",chr(0x1a),chr(0x10),"","",0,0,""]
         self.transport.write(apply(struct.pack,[self.header_fmt]+hdr))
-                    
+
     def dataReceived(self,data):
         self.buf=self.buf+data
         while len(self.buf)>=256:
@@ -1494,12 +1502,12 @@ class GetFileTransfer(protocol.Protocol):
                 hdr[18]=hdr[19]=0
                 hdr[21]=chr(0x20)
                 self.transport.write(apply(struct.pack,[self.header_fmt]+hdr))
-                print "got file request for %s"%file,hex(hdr[13])
+                log.msg("got file request for %s"%file,hex(hdr[13]))
             elif hdr[2]==0x0202:
-                print "sending file"
+                log.msg("sending file")
                 self.transport.registerProducer(self,0)
             elif hdr[2]==0x0204:
-                print "real checksum: %s"%hex(hdr[19])
+                log.msg("real checksum: %s"%hex(hdr[19]))
                 del self.file
             elif hdr[2]==0x0205: # resume
                 already=hdr[18]
@@ -1507,19 +1515,19 @@ class GetFileTransfer(protocol.Protocol):
                     data=self.file.read(already)
                 else:
                     data=""
-                print "restarting at %s"%already
+                log.msg("restarting at %s"%already)
                 hdr[2]=0x0106
                 hdr[19]=checksum(data)
                 self.transport.write(apply(struct.pack,[self.header_fmt]+hdr))
             elif hdr[2]==0x0207:
                 self.transport.registerProducer(self,0)
             else:
-                print "don't understand 0x%04x"%hdr[2]
-                print hdr
+                log.msg("don't understand 0x%04x"%hdr[2])
+                log.msg(hdr)
 
     def resumeProducing(self):
         data=self.file.read(4096)
-        print len(data)
+        log.msg(len(data))
         if not data:
             self.transport.unregisterProducer()
         self.transport.write(data)
@@ -1587,4 +1595,4 @@ STD_MESSAGE[BAD_NICKNAME]="Incorrect nickname or password"
 STD_MESSAGE[SERVICE_TEMP_UNAVAILABLE]="The service is temporarily unavailable"
 STD_MESSAGE[WARNING_TOO_HIGH]="Your warning level is currently too high to sign on"
 STD_MESSAGE[CONNECTING_TOO_QUICK]="You have been connecting and disconnecting too frequently.  Wait 10 minutes and try again.  If you continue to try, you will need to wait even longer."
-STD_MESSAGE[UNKNOWN_SIGNON]="An unknown signon error has occured %s"
\ No newline at end of file
+STD_MESSAGE[UNKNOWN_SIGNON]="An unknown signon error has occurred %s"
