@@ -67,8 +67,9 @@ def callWithLogger(logger, func, *args, **kw):
         err(system=lp)
 
 def write(stuff):
-    """Write some data to the log."""
-    warnings.warn("What the hell is wrong with you?", DeprecationWarning, stacklevel=2)
+    """Write some data to the log.
+    DEPRECATED. Use L{msg} instead."""
+    warnings.warn("Use log.msg, not log.write.", DeprecationWarning, stacklevel=2)
     msg(str(stuff))
 
 def debug(*stuff,**otherstuff):
@@ -182,7 +183,7 @@ logOwner = EscapeFromTheMeaninglessConfinesOfCapital()
 
 class LogPublisher:
     """Class for singleton log message publishing."""
-    
+
     synchronized = ['msg']
 
     def __init__(self):
@@ -194,6 +195,7 @@ class LogPublisher:
         Observers are callable objects that will be called with each new log
         message (a dict).
         """
+        assert callable(other)
         self.observers.append(other)
 
     def removeObserver(self, other):
@@ -206,9 +208,13 @@ class LogPublisher:
         actualEventDict.update(kw)
         actualEventDict['message'] = message
         actualEventDict['time'] = time.time()
-        for o in self.observers:
-            o(actualEventDict)
-
+        for i in xrange(len(self.observers) - 1, -1, -1):
+            try:
+                self.observers[i](actualEventDict)
+            except:
+                o = self.observers.pop(i)
+                msg("Log observer %s failed, removing from observer list." % (o,))
+                err()
 
 try:
     theLogPublisher
@@ -230,8 +236,12 @@ threadable.whenThreaded(initThreads)
 
 
 class FileLogObserver:
-    """Log observer that writes to a file-like object."""
+    """Log observer that writes to a file-like object.
     
+    @ivar timeFormat: Format string passed to strftime()
+    """
+    timeFormat = "%Y/%m/%d %H:%M %Z"
+
     def __init__(self, f):
         self.write = f.write
         self.flush = f.flush
@@ -248,10 +258,11 @@ class FileLogObserver:
                 return
         else:
             text = ' '.join(map(str, edm))
-        y,mon,d,h,min, iigg,nnoo,rree,daylight = time.localtime(eventDict['time'])
-        self.write("%0.4d/%0.2d/%0.2d %0.2d:%0.2d %s [%s] %s\n" %
-                   (y, mon, d, h, min, time.tzname[daylight],
-                    eventDict['system'], text.replace("\n","\n\t")))
+        
+        timeStr = time.strftime(self.timeFormat, time.localtime(eventDict['time']))
+        fmtDict = {'system': eventDict['system'], 'text': text.replace("\n", "\n\t")}
+        msgStr = " [%(system)s] %(text)s\n" % fmtDict
+        self.write(timeStr + msgStr)
         self.flush()                    # hoorj!
 
     def start(self):
@@ -265,7 +276,7 @@ class FileLogObserver:
 
 class StdioOnnaStick:
     """Class that pretends to be stout/err."""
-    
+
     closed = 0
     softspace = 0
     mode = 'wb'
@@ -317,8 +328,8 @@ def startLogging(file, *a, **kw):
     startLoggingWithObserver(flo.emit, *a, **kw)
 
 def startLoggingWithObserver(observer, setStdout=1):
-    """Initialize logging to a specified observer. If setStdout is true 
-       (defaults to yes), also redirect sys.stdout and sys.stderr 
+    """Initialize logging to a specified observer. If setStdout is true
+       (defaults to yes), also redirect sys.stdout and sys.stderr
        to the specified file.
     """
     global defaultObserver, _oldshowwarning
@@ -373,7 +384,7 @@ class DefaultObserver:
                 text = " ".join([str(m) for m in eventDict["message"]]) + "\n"
             sys.stderr.write(text)
             sys.stderr.flush()
-    
+
     def start(self):
         addObserver(self._emit)
 
