@@ -16,21 +16,17 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 """
-I am a support module for making TOC servers with mktap.
+Support module for making TOC servers with mktap.
 """
 
 from twisted.protocols import toc
-from twisted.internet import tcp
 from twisted.python import usage
-import sys
+from twisted.application import strports
 
 class Options(usage.Options):
     synopsis = "Usage: mktap toc [-p <port>]"
-    optStrings = [["port", "p", 5190]]
-
+    optParameters = [["port", "p", "5190"]]
     longdesc = "Makes a TOC server."
 
-def updateApplication(app, config):
-    t = toc.TOCFactory()
-    portno = int(config.port)
-    app.listenTCP(portno, t)
+def makeService(config):
+    return strports.service(config['port'], toc.TOCFactory())
