@@ -1,26 +1,28 @@
 # Twisted, the Framework of Your Internet
 # Copyright (C) 2001-2002 Matthew W. Lefkowitz
-# 
+#
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of version 2.1 of the GNU Lesser General Public
 # License as published by the Free Software Foundation.
-# 
+#
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-# 
+#
 
 """Tests for twisted.enterprise."""
 
 from twisted.trial import unittest
 
 import os
+import stat
 import random
+import tempfile
 
 from twisted.enterprise.row import RowObject
 from twisted.enterprise.reflector import *
@@ -44,6 +46,12 @@ except: PgSQL = None
 try: import MySQLdb
 except: MySQLdb = None
 
+try: import psycopg
+except: psycopg = None
+
+try: import kinterbasdb
+except: kinterbasdb = None
+
 tableName = "testTable"
 childTableName = "childTable"
 
@@ -163,7 +171,6 @@ class ReflectorTestCase:
 
         # save it
         deferredResult(self.reflector.insertRow(row))
-        row = None # drop the reference so it is no longer cached
 
         # now load it back in
         whereClause = [("key_string", EQUAL, "first")]
@@ -279,6 +286,17 @@ class ReflectorTestCase:
 
         self.failUnless(len(self.data) == 0, "rows were not deleted")
 
+        # create one row to work with
+        row = TestRow()
+        row.assignKeyAttr("key_string", "first")
+        values = self.randomizeRow(row)
+
+        # save it
+        deferredResult(self.reflector.insertRow(row))
+
+        # delete it
+        deferredResult(self.reflector.deleteRow(row))
+
     def gotData(self, data):
         self.data = data
 
@@ -307,6 +325,7 @@ class SQLReflectorTestCase(ReflectorTestCase):
     DB_PASS = 'twisted_test'
 
     can_rollback = 1
+    test_failures = 1
 
     reflectorClass = SQLReflector
 
@@ -327,11 +346,12 @@ class SQLReflectorTestCase(ReflectorTestCase):
         self.stopDB()
 
     def testPool(self):
-        # make sure failures are raised correctly
-        deferredError(self.dbpool.runQuery("select * from NOTABLE"))
-        deferredError(self.dbpool.runOperation("delete from * from NOTABLE"))
-        deferredError(self.dbpool.runInteraction(self.bad_interaction))
-        log.flushErrors()
+        if self.test_failures:
+            # make sure failures are raised correctly
+            deferredError(self.dbpool.runQuery("select * from NOTABLE"))
+            deferredError(self.dbpool.runOperation("deletexxx from NOTABLE"))
+            deferredError(self.dbpool.runInteraction(self.bad_interaction))
+            log.flushErrors()
 
         # verify simple table is empty
         sql = "select count(1) from simple"
@@ -387,7 +407,7 @@ class SQLReflectorTestCase(ReflectorTestCase):
         # should test this, but gadfly throws an exception instead
         #self.failUnless(transaction.fetchone() is None, "Too many rows")
         return "done"
-    
+
     def bad_interaction(self, transaction):
         if self.can_rollback:
             transaction.execute("insert into simple(x) values(0)")
@@ -453,6 +473,15 @@ class PostgresTestCase(SQLReflectorTestCase, unittest.TestCase):
                               user=self.DB_USER, password=self.DB_PASS,
                               cp_min=0)
 
+class PsycopgTestCase(SQLReflectorTestCase, unittest.TestCase):
+    """Test cases for the SQL reflector using psycopg for Postgres.
+    """
+
+    def makePool(self):
+        return ConnectionPool('psycopg', database=self.DB_NAME,
+                              user=self.DB_USER, password=self.DB_PASS,
+                              cp_min=0)
+
 
 class MySQLTestCase(SQLReflectorTestCase, unittest.TestCase):
     """Test cases for the SQL reflector using MySQL.
@@ -466,6 +495,35 @@ class MySQLTestCase(SQLReflectorTestCase, unittest.TestCase):
                               user=self.DB_USER, passwd=self.DB_PASS)
 
 
+class FirebirdTestCase(SQLReflectorTestCase, unittest.TestCase):
+    """Test cases for the SQL reflector using Firebird/Interbase."""
+
+    count = 2 # CHANGEME
+    test_failures = 0 # failure testing causes problems
+    reflectorClass = NoSlashSQLReflector
+    DB_DIR = tempfile.mktemp()
+    DB_NAME = os.path.join(DB_DIR, SQLReflectorTestCase.DB_NAME)
+
+    def startDB(self):
+        os.chmod(self.DB_DIR, stat.S_IRWXU + stat.S_IRWXG + stat.S_IRWXO)
+        sql = 'create database "%s" user "%s" password "%s"'
+        sql %= (self.DB_NAME, self.DB_USER, self.DB_PASS);
+        conn = kinterbasdb.create_database(sql)
+        conn.close()
+        os.chmod(self.DB_NAME, stat.S_IRWXU + stat.S_IRWXG + stat.S_IRWXO)
+
+    def makePool(self):
+        return ConnectionPool('kinterbasdb', database=self.DB_NAME,
+                              host='localhost', user=self.DB_USER,
+                              password=self.DB_PASS)
+
+    def stopDB(self):
+        conn = kinterbasdb.connect(database=self.DB_NAME,
+                                   host='localhost', user=self.DB_USER,
+                                   password=self.DB_PASS)
+        conn.drop_database()
+        conn.close()
+
 class QuotingTestCase(unittest.TestCase):
 
     def testQuoting(self):
@@ -492,6 +550,16 @@ else:
     except Exception, e:
         PostgresTestCase.skip = "Connection to PgSQL server failed: " + str(e)
 
+if psycopg is None: PsycopgTestCase.skip = "psycopg module not available"
+else:
+    try:
+        conn = psycopg.connect(database=PsycopgTestCase.DB_NAME,
+                               user=PsycopgTestCase.DB_USER,
+                               password=PsycopgTestCase.DB_PASS)
+        conn.close()
+    except Exception, e:
+        PsycopgTestCase.skip = "Connection to PostgreSQL using psycopg failed: " + str(e)
+
 if MySQLdb is None: MySQLTestCase.skip = "MySQLdb module not available"
 else:
     try:
@@ -501,3 +569,13 @@ else:
         conn.close()
     except Exception, e:
         MySQLTestCase.skip = "Connection to MySQL server failed: " + str(e)
+
+if kinterbasdb is None:
+    FirebirdTestCase.skip = "kinterbasdb module not available"
+else:
+    try:
+        testcase = FirebirdTestCase()
+        testcase.startDB()
+        testcase.stopDB()
+    except Exception, e:
+        FirebirdTestCase.skip = "Connection to Firebase server failed: " + str(e)
