@@ -16,7 +16,9 @@
 
 """FTP tests.
 
-Maintainer: slyphon (Jonathan D. Simms)
+Server Test Maintainer: slyphon (Jonathan D. Simms)
+Client Test Maintainer: spiv
+
 """
 
 from __future__ import  nested_scopes
@@ -29,7 +31,7 @@ from twisted.trial import unittest
 from twisted.protocols import basic
 from twisted.internet import reactor, protocol, defer, interfaces
 from twisted.cred import error, portal, checkers, credentials
-from twisted.python import log, components
+from twisted.python import log, components, failure
 
 from twisted.protocols import ftp, loopback
 
@@ -349,12 +351,12 @@ class TestUtilityFunctions(unittest.TestCase):
         evilPaths = [r"..\/*/foobar/ding//dong/\\**", 
         r"../../\\**/*/fhet/*/..///\\//..///#$#221./*"]
         exorcisedPaths = ["..\/foobar/ding/dong","../../../#$#221."]
-        for x, ep in enumerate(evilPaths):
-            cp = ftp.cleanPath(ep)
-            self.failUnlessEqual(cp, exorcisedPaths[x])
+        for i in range(len(evilPaths)):
+            cp = ftp.cleanPath(evilPaths[i])
+            self.failUnlessEqual(cp, exorcisedPaths[i])
             log.msg(cp)
 
-    testCleanPath.todo = 'this test needs more work'
+    testCleanPath.skip = 'this test needs more work'
 
 class TestFTPFactory(FTPTestCase):
     def testBuildProtocol(self):
@@ -378,7 +380,7 @@ class TestFTPFactory(FTPTestCase):
         self.failUnlessEqual(len(ftpf.instances), 1)
         self.failUnlessEqual(ftpf.instances[0], p)
 
-    testBuildProtocol.todo = "add test for maxProtocolInstances=None"
+    testBuildProtocol.skip = "add test for maxProtocolInstances=None"
         
 class TestFTPServer(FTPTestCase):
     def testNotLoggedInReply(self):
@@ -458,7 +460,7 @@ class TestFTPServer(FTPTestCase):
         self.failUnlessEqual(rx.count('\r\n'), 2, "more than 2 \\r\\n's ")
         self.fail('test is not complete')
 
-    testTYPE.todo = 'rework tests to make sure only binary is supported'
+    testTYPE.skip = 'rework tests to make sure only binary is supported'
 
     def testRETR(self):
         cli, sr, iop, send = self.cnx.getCSTuple()
@@ -486,7 +488,7 @@ class TestFTPServer(FTPTestCase):
         rx = ''.join(dc.rawData)
         self.failUnlessEqual(rx, avatar.sentfile.getvalue())
         
-    #testRETR.todo = 'not quite there yet'
+    testRETR.skip = 'awaiting MASSIVE refactoring of ftp.py'
 
     def testSYST(self):
         cli, sr, iop, send = self.cnx.getCSTuple()
@@ -576,7 +578,7 @@ class TestAnonymousAvatar(FTPTestCase):
         got = c.lines[-1]
         self.assertEquals(wanted, got, "wanted: %s\n\ngot: %s" % (wanted,got))
 
-    testAnonymousLogin.todo = 'this test needs to be refactored' 
+    testAnonymousLogin.skip = 'this test needs to be refactored' 
     
     def doAnonymousLogin(self,c,s,pump):
         c, s, pump, send = self.cnx.getCSTuple()
@@ -594,7 +596,7 @@ class TestAnonymousAvatar(FTPTestCase):
         pump.flush()
         self.assertEquals(c.lines[-1], '257 "/" is current directory.')
 
-    testPWDOnLogin.todo = 'need to implement fake filesystem for testing' 
+    testPWDOnLogin.skip = 'need to implement fake filesystem for testing' 
 
     def testCWD(self):
         import warnings
@@ -623,7 +625,7 @@ class TestAnonymousAvatar(FTPTestCase):
         send('PWD'); flush()
         send('CWD ../../../'); flush()
 
-    testCWD.todo = 'need to implement fake filesystem for testing' 
+    testCWD.skip = 'need to implement fake filesystem for testing' 
 
 
     def testCDUP(self):
@@ -649,7 +651,7 @@ class TestAnonymousAvatar(FTPTestCase):
         send('PWD'); flush()
         self.assertEquals(c.lines[-1], '257 "/" is current directory.')
 
-    testCDUP.todo = 'need to implement fake filesystem for testing' 
+    testCDUP.skip = 'need to implement fake filesystem for testing' 
 
 #    def testWelcomeMessage(self):
 #        c, s, pump, send = self.cnx.getCSTuple()
@@ -684,17 +686,36 @@ class ChildProcessProtocol(protocol.ProcessProtocol):
 # -- Client Tests -----------------------------------------------------------
 
 
+class PrintLines(protocol.Protocol):
+    """Helper class used by FTPFileListingTests."""
+
+    def __init__(self, lines):
+        self._lines = lines
+
+    def connectionMade(self):
+        for line in self._lines:
+            self.transport.write(line + "\r\n")
+        self.transport.loseConnection()
+
+class MyFTPFileListProtocol(ftp.FTPFileListProtocol):
+    def __init__(self):
+        self.other = []
+        ftp.FTPFileListProtocol.__init__(self)
+
+    def unknownLine(self, line):
+        self.other.append(line)
 
 class FTPFileListingTests(unittest.TestCase):
+    def getFilesForLines(self, lines):
+        fileList = MyFTPFileListProtocol()
+        loopback.loopback(PrintLines(lines), fileList)
+        return fileList.files, fileList.other
+
     def testOneLine(self):
         # This example line taken from the docstring for FTPFileListProtocol
-        fileList = ftp.FTPFileListProtocol()
-        class PrintLine(protocol.Protocol):
-            def connectionMade(self):
-                self.transport.write('-rw-r--r--   1 root     other        531 Jan 29 03:26 README\n')
-                self.transport.loseConnection()
-        loopback.loopback(PrintLine(), fileList)
-        file = fileList.files[0]
+        line = '-rw-r--r--   1 root     other        531 Jan 29 03:26 README'
+        (file,), other = self.getFilesForLines([line])
+        self.failIf(other, 'unexpect unparsable lines: %s' % repr(other))
         self.failUnless(file['filetype'] == '-', 'misparsed fileitem')
         self.failUnless(file['perms'] == 'rw-r--r--', 'misparsed perms')
         self.failUnless(file['owner'] == 'root', 'misparsed fileitem')
@@ -702,6 +723,54 @@ class FTPFileListingTests(unittest.TestCase):
         self.failUnless(file['size'] == 531, 'misparsed fileitem')
         self.failUnless(file['date'] == 'Jan 29 03:26', 'misparsed fileitem')
         self.failUnless(file['filename'] == 'README', 'misparsed fileitem')
+        self.failUnless(file['nlinks'] == 1, 'misparsed nlinks')
+        self.failIf(file['linktarget'], 'misparsed linktarget')
+
+    def testVariantLines(self):
+        line1 = 'drw-r--r--   2 root     other        531 Jan  9  2003 A'
+        line2 = 'lrw-r--r--   1 root     other          1 Jan 29 03:26 B -> A'
+        line3 = 'woohoo! '
+        (file1, file2), (other,) = self.getFilesForLines([line1, line2, line3])
+        self.failUnless(other == 'woohoo! \r', 'incorrect other line')
+        # file 1
+        self.failUnless(file1['filetype'] == 'd', 'misparsed fileitem')
+        self.failUnless(file1['perms'] == 'rw-r--r--', 'misparsed perms')
+        self.failUnless(file1['owner'] == 'root', 'misparsed owner')
+        self.failUnless(file1['group'] == 'other', 'misparsed group')
+        self.failUnless(file1['size'] == 531, 'misparsed size')
+        self.failUnless(file1['date'] == 'Jan  9  2003', 'misparsed date')
+        self.failUnless(file1['filename'] == 'A', 'misparsed filename')
+        self.failUnless(file1['nlinks'] == 2, 'misparsed nlinks')
+        self.failIf(file1['linktarget'], 'misparsed linktarget')
+        # file 2
+        self.failUnless(file2['filetype'] == 'l', 'misparsed fileitem')
+        self.failUnless(file2['perms'] == 'rw-r--r--', 'misparsed perms')
+        self.failUnless(file2['owner'] == 'root', 'misparsed owner')
+        self.failUnless(file2['group'] == 'other', 'misparsed group')
+        self.failUnless(file2['size'] == 1, 'misparsed size')
+        self.failUnless(file2['date'] == 'Jan 29 03:26', 'misparsed date')
+        self.failUnless(file2['filename'] == 'B', 'misparsed filename')
+        self.failUnless(file2['nlinks'] == 1, 'misparsed nlinks')
+        self.failUnless(file2['linktarget'] == 'A', 'misparsed linktarget')
+
+    def testUnknownLine(self):
+        files, others = self.getFilesForLines(['ABC', 'not a file'])
+        self.failIf(files, 'unexpected file entries')
+        self.failUnless(others == ['ABC\r', 'not a file\r'],
+                        'incorrect unparsable lines: %s' % repr(others))
+
+    def testYear(self):
+        # This example derived from bug description in issue 514.
+        fileList = ftp.FTPFileListProtocol()
+        class PrintLine(protocol.Protocol):
+            def connectionMade(self):
+                self.transport.write('-rw-r--r--   1 root     other        531 Jan 29 2003 README\n')
+                self.transport.loseConnection()
+        loopback.loopback(PrintLine(), fileList)
+        file = fileList.files[0]
+        self.failUnless(file['size'] == 531, 'misparsed fileitem')
+        self.failUnless(file['date'] == 'Jan 29 2003', 'misparsed fileitem')
+        self.failUnless(file['filename'] == 'README', 'misparsed fileitem')
 
 class FTPClientTests(unittest.TestCase):
     def testFailedRETR(self):
@@ -748,7 +817,7 @@ class FTPClientTests(unittest.TestCase):
                 reactor.iterate()
             except:
                 pass
-
+    
     def timeout(self):
         reactor.crash()
         self.fail('Timed out')
@@ -757,4 +826,15 @@ class FTPClientTests(unittest.TestCase):
         for response in responses:
             reactor.callLater(0.1, protocol.lineReceived, response)
 
+    def testErrbacksUponDisconnect(self):
+        ftpClient = ftp.FTPClient()
+        d = ftpClient.list('some path', Dummy())
+        m = []
+        def _eb(failure):
+            m.append(failure)
+            return None
+        d.addErrback(_eb)
+        from twisted.internet.main import CONNECTION_LOST
+        ftpClient.connectionLost(failure.Failure(CONNECTION_LOST))
+        self.failUnless(1, len(m))
 
