@@ -46,6 +46,8 @@ import twisted.cred.checkers
 import twisted.cred.credentials
 import twisted.cred.portal
 
+from twisted.test.proto_helpers import StringTransport
+
 try:
     from ssl_helpers import ClientTLSContext, ServerTLSContext
 except ImportError:
@@ -80,7 +82,7 @@ class IMAP4UTF7TestCase(unittest.TestCase):
         for (input, output) in self.tests:
             # XXX - Piece of *crap* 2.1
             self.assertEquals(input, imap4.decoder(output)[0])
-    
+
     def testPrintableSingletons(self):
         # All printables represent themselves
         for o in range(0x20, 0x26) + range(0x27, 0x7f):
@@ -670,10 +672,13 @@ class IMAP4HelperTestCase(unittest.TestCase):
                 "len(%r) = %r != %r" % (input, L, expected))
 
 class SimpleMailbox:
+    __implements__ = imap4.IMailboxInfo, imap4.IMailbox, imap4.ICloseableMailbox
+
     flags = ('\\Flag1', 'Flag2', '\\AnotherSysFlag', 'LastFlag')
     messages = []
     mUID = 0
     rw = 1
+    closed = False
 
     def __init__(self):
         self.listeners = []
@@ -735,6 +740,9 @@ class SimpleMailbox:
             self.messages.remove(i)
         return [i[3] for i in delete]
 
+    def close(self):
+        self.closed = True
+
 class Account(imap4.MemoryAccount):
     def _emptyMailbox(self, name, id):
         return SimpleMailbox()
@@ -826,7 +834,7 @@ class IMAP4ServerTestCase(IMAP4HelperMixin, unittest.TestCase):
         self.connected.addCallback(strip(getCaps)).addErrback(self._ebGeneral)
         self.loopback()
 
-        self.assertEquals({'IMAP4rev1': None, 'NAMESPACE': None}, caps)
+        self.assertEquals({'IMAP4rev1': None, 'NAMESPACE': None, 'IDLE': None}, caps)
 
     def testCapabilityWithAuth(self):
         caps = {}
@@ -839,7 +847,8 @@ class IMAP4ServerTestCase(IMAP4HelperMixin, unittest.TestCase):
         self.connected.addCallback(strip(getCaps)).addErrback(self._ebGeneral)
         self.loopback()
 
-        self.assertEquals({'IMAP4rev1': None, 'NAMESPACE': None, 'AUTH': ['CRAM-MD5']}, caps)
+        expCap = {'IMAP4rev1': None, 'NAMESPACE': None, 'IDLE': None, 'AUTH': ['CRAM-MD5']}
+        self.assertEquals(expCap, caps)
 
     def testLogout(self):
         self.loggedOut = 0
@@ -1305,6 +1314,7 @@ class IMAP4ServerTestCase(IMAP4HelperMixin, unittest.TestCase):
 
         self.assertEquals(len(m.messages), 1)
         self.assertEquals(m.messages[0], ('Message 2', ('AnotherFlag',), None, 1))
+        self.failUnless(m.closed)
 
     def testExpunge(self):
         m = SimpleMailbox()
@@ -1576,34 +1586,6 @@ class UnsolicitedResponseTestCase(IMAP4HelperMixin, unittest.TestCase):
         E = self.client.events
         self.assertEquals(E, [['newMessages', 20, None], ['newMessages', None, 10]])
 
-class StringTransport:
-    disconnecting = 0
-
-    def __init__(self):
-        self.clear()
-
-    def clear(self):
-        self.io = StringIO()
-
-    def value(self):
-        return self.io.getvalue()
-
-    def write(self, data):
-        self.io.write(data)
-
-    def writeSequence(self, data):
-        self.io.write(''.join(data))
-
-    def loseConnection(self):
-        pass
-
-    def getPeer(self):
-        return ('StringIO', repr(self.io))
-
-    def getHost(self):
-        return ('StringIO', repr(self.io))
-
-
 class HandCraftedTestCase(unittest.TestCase):
     def testTrailingLiteral(self):
         transport = StringTransport()
@@ -2091,6 +2073,10 @@ class FetchSearchStoreTestCase(unittest.TestCase, IMAP4HelperMixin):
         self.server_received_uid = uid
         return self.expected
 
+    def addListener(self, *a, **kw):
+        pass
+    removeListener = addListener
+
     def _searchWork(self, uid):
         def search():
             return self.client.search(self.query, uid=uid)
@@ -2182,65 +2168,93 @@ class FakeMailbox:
 
 class FeaturefulMessage:
     __implements__ = imap4.IMessageFile,
-    
+
     def getFlags(self):
         return 'flags'
-    
+
     def getInternalDate(self):
         return 'internaldate'
-    
+
     def open(self):
         return StringIO("open")
 
+class MessageCopierMailbox:
+    __implements__ = imap4.IMessageCopier,
+
+    def __init__(self):
+        self.msgs = []
+
+    def copy(self, msg):
+        self.msgs.append(msg)
+        return len(self.msgs)
+
 class CopyWorkerTestCase(unittest.TestCase):
     def testFeaturefulMessage(self):
         s = imap4.IMAP4Server()
-        
+
         # Yes.  I am grabbing this uber-non-public method to test it.
         # It is complex.  It needs to be tested directly!
         # Perhaps it should be refactored, simplified, or split up into
         # not-so-private components, but that is a task for another day.
+
+        # Ha ha! Addendum!  Soon it will be split up, and this test will
+        # be re-written to just use the default adapter for IMailbox to
+        # IMessageCopier and call .copy on that adapter.
         f = s._IMAP4Server__cbCopy
 
         m = FakeMailbox()
         d = f([(i, FeaturefulMessage()) for i in range(1, 11)], 'tag', m)
         r = unittest.deferredResult(d)
-        
+
         for a in m.args:
             self.assertEquals(a[0].read(), "open")
             self.assertEquals(a[1], "flags")
             self.assertEquals(a[2], "internaldate")
-        
+
         for (status, result) in r:
             self.failUnless(status)
             self.assertEquals(result, None)
 
     def testUnfeaturefulMessage(self):
         s = imap4.IMAP4Server()
-        
+
         # See above comment
         f = s._IMAP4Server__cbCopy
-        
+
         m = FakeMailbox()
         msgs = [FakeyMessage({'Header-Counter': str(i)}, (), 'Date', 'Body %d' % (i,), i + 10, None) for i in range(1, 11)]
         d = f([im for im in zip(range(1, 11), msgs)], 'tag', m)
         r = unittest.deferredResult(d)
-        
+
         seen = []
         for a in m.args:
             seen.append(a[0].read())
             self.assertEquals(a[1], ())
             self.assertEquals(a[2], "Date")
-        
+
         seen.sort()
         exp = ["Header-Counter: %d\r\n\r\nBody %d" % (i, i) for i in range(1, 11)]
         exp.sort()
         self.assertEquals(seen, exp)
-        
+
         for (status, result) in r:
             self.failUnless(status)
             self.assertEquals(result, None)
-        
+
+    def testMessageCopier(self):
+        s = imap4.IMAP4Server()
+
+        # See above comment
+        f = s._IMAP4Server__cbCopy
+
+        m = MessageCopierMailbox()
+        msgs = [object() for i in range(1, 11)]
+        d = f([im for im in zip(range(1, 11), msgs)], 'tag', m)
+        r = unittest.deferredResult(d)
+
+        self.assertEquals(r, zip([1] * 10, range(1, 11)))
+        for (orig, new) in zip(msgs, m.msgs):
+            self.assertIdentical(orig, new)
 
 class TLSTestCase(IMAP4HelperMixin, unittest.TestCase):
     serverCTX = ServerTLSContext and ServerTLSContext()
