@@ -23,6 +23,7 @@ from twisted.trial import unittest
 
 from twisted.internet import protocol, reactor, defer
 from twisted.internet import error
+from twisted.internet.address import IPv4Address
 
 
 class ClosingProtocol(protocol.Protocol):
@@ -107,7 +108,7 @@ class ListeningTestCase(PortCleanerUpper):
     def testStopListening(self):
         f = MyServerFactory()
         port = reactor.listenTCP(0, f, interface="127.0.0.1")
-        n = port.getHost()[2]
+        n = port.getHost().port
         self.ports.append(port)
         l = []
         defer.maybeDeferred(port.stopListening).addCallback(l.append)
@@ -137,7 +138,7 @@ class LoopbackTestCase(PortCleanerUpper):
     def testClosePortInProtocolFactory(self):
         f = ClosingFactory()
         port = reactor.listenTCP(0, f, interface="127.0.0.1")
-        self.n = port.getHost()[2]
+        self.n = port.getHost().port
         self.ports.append(port)
         f.port = port
         clientF = MyClientFactory()
@@ -156,7 +157,7 @@ class LoopbackTestCase(PortCleanerUpper):
     def testTcpNoDelay(self):
         f = MyServerFactory()
         port = reactor.listenTCP(0, f, interface="127.0.0.1")
-        self.n = port.getHost()[2]
+        self.n = port.getHost().port
         self.ports.append(port)
         clientF = MyClientFactory()
         reactor.connectTCP("localhost", self.n, clientF)
@@ -181,7 +182,7 @@ class LoopbackTestCase(PortCleanerUpper):
     def testTcpKeepAlive(self):
         f = MyServerFactory()
         port = reactor.listenTCP(0, f, interface="127.0.0.1")
-        self.n = port.getHost()[2]
+        self.n = port.getHost().port
         self.ports.append(port)
         clientF = MyClientFactory()
         reactor.connectTCP("localhost", self.n, clientF)
@@ -219,7 +220,7 @@ class LoopbackTestCase(PortCleanerUpper):
         serv = socket.getservbyname
         s = MyServerFactory()
         port = reactor.listenTCP(0, s, interface="127.0.0.1")
-        self.n = port.getHost()[2]
+        self.n = port.getHost().port
         socket.getservbyname = lambda s, p,n=self.n: s == 'http' and p == 'tcp' and n or 10
         self.ports.append(port)
         try:
@@ -279,7 +280,7 @@ class FactoryTestCase(PortCleanerUpper):
 
         # listen on port
         p1 = reactor.listenTCP(0, f, interface='127.0.0.1')
-        self.n1 = p1.getHost()[2]
+        self.n1 = p1.getHost().port
         self.ports.append(p1)
         reactor.iterate()
         reactor.iterate()
@@ -287,10 +288,10 @@ class FactoryTestCase(PortCleanerUpper):
         
         # listen on two more ports
         p2 = reactor.listenTCP(0, f, interface='127.0.0.1')
-        self.n2 = p2.getHost()[2]
+        self.n2 = p2.getHost().port
         self.ports.append(p2)
         p3 = reactor.listenTCP(0, f, interface='127.0.0.1')
-        self.n3 = p3.getHost()[2]
+        self.n3 = p3.getHost().port
         self.ports.append(p3)
         reactor.iterate()
         reactor.iterate()
@@ -312,7 +313,7 @@ class FactoryTestCase(PortCleanerUpper):
     def testClientStartStop(self):
         f = ClosingFactory()
         p = reactor.listenTCP(0, f, interface="127.0.0.1")
-        self.n = p.getHost()[2]
+        self.n = p.getHost().port
         self.ports.append(p)
         f.port = p
         reactor.iterate()
@@ -333,7 +334,7 @@ class ConnectorTestCase(PortCleanerUpper):
     def testConnectorIdentity(self):
         f = ClosingFactory()
         p = reactor.listenTCP(0, f, interface="127.0.0.1")
-        n = p.getHost()[2]
+        n = p.getHost().port
         self.ports.append(p)
         f.port = p
         reactor.iterate()
@@ -356,7 +357,7 @@ class ConnectorTestCase(PortCleanerUpper):
     def testUserFail(self):
         f = MyServerFactory()
         p = reactor.listenTCP(0, f, interface="127.0.0.1")
-        n = p.getHost()[2]
+        n = p.getHost().port
         self.ports.append(p)
         
         def startedConnecting(connector):
@@ -379,7 +380,7 @@ class ConnectorTestCase(PortCleanerUpper):
     def testReconnect(self):
         f = ClosingFactory()
         p = reactor.listenTCP(0, f, interface="127.0.0.1")
-        n = p.getHost()[2]
+        n = p.getHost().port
         self.ports.append(p)
         f.port = p
         reactor.iterate()
@@ -409,7 +410,7 @@ class CannotBindTestCase(PortCleanerUpper):
         f = MyServerFactory()
 
         p1 = reactor.listenTCP(0, f, interface='127.0.0.1')
-        n = p1.getHost()[2]
+        n = p1.getHost().port
         self.ports.append(p1)
         self.assertEquals(p1.getHost(), ("INET", "127.0.0.1", n,))
         
@@ -424,15 +425,15 @@ class CannotBindTestCase(PortCleanerUpper):
         self.ports.append(p)
         
         factory = MyClientFactory()
-        reactor.connectTCP("127.0.0.1", p.getHost()[2], factory, bindAddress=("127.0.0.1", 0))
+        reactor.connectTCP("127.0.0.1", p.getHost().port, factory, bindAddress=("127.0.0.1", 0))
         while not factory.protocol:
             reactor.iterate()
         
         self.assertEquals(factory.protocol.made, 1)
 
-        port = factory.protocol.transport.getHost()[2]
+        port = factory.protocol.transport.getHost().port
         f2 = MyClientFactory()
-        reactor.connectTCP("127.0.0.1", p.getHost()[2], f2, bindAddress=("127.0.0.1", port))
+        reactor.connectTCP("127.0.0.1", p.getHost().port, f2, bindAddress=("127.0.0.1", port))
         reactor.iterate()
         reactor.iterate()
         
@@ -447,11 +448,11 @@ class CannotBindTestCase(PortCleanerUpper):
         reactor.iterate()
         self.assertEquals(factory.stopped, 1)
 
-
 class MyOtherClientFactory(protocol.ClientFactory):
     def buildProtocol(self, address):
         self.address = address
-        return MyProtocol()
+        self.protocol = MyProtocol()
+        return self.protocol
 
 
 class LocalRemoteAddressTestCase(PortCleanerUpper):
@@ -461,7 +462,7 @@ class LocalRemoteAddressTestCase(PortCleanerUpper):
     def testHostAddress(self):
         f1 = MyServerFactory()
         p1 = reactor.listenTCP(0, f1, interface='127.0.0.1')
-        n = p1.getHost()[2]
+        n = p1.getHost().port
         self.ports.append(p1)
         
         f2 = MyOtherClientFactory()
@@ -471,10 +472,10 @@ class LocalRemoteAddressTestCase(PortCleanerUpper):
             reactor.iterate(0.01)
 
         self.assertEquals(p1.getHost(), f2.address)
-        self.assertEquals(p1.getHost(), p2.transport.getPeer())
+        self.assertEquals(p1.getHost(), f2.protocol.transport.getPeer())
 
         p1.stopListening()
-        p2.transport.loseConnection()
+        p2.disconnect()
     
 
 class WriterProtocol(protocol.Protocol):
@@ -526,7 +527,7 @@ class WriteDataTestCase(PortCleanerUpper):
         f.done = 0
         f.problem = 0
         p = reactor.listenTCP(0, f, interface="127.0.0.1")
-        n = p.getHost()[2]
+        n = p.getHost().port
         self.ports.append(p)
         clientF = WriterClientFactory()
         reactor.connectTCP("localhost", n, clientF)
@@ -564,7 +565,7 @@ class ProperlyCloseFilesTestCase(unittest.TestCase):
         f.protocol.master = self
         
         def connector():
-            p = self.listener.getHost()[2]
+            p = self.listener.getHost().port
             return reactor.connectTCP('127.0.0.1', p, f)
         self.connector = connector
 
@@ -588,6 +589,68 @@ class ProperlyCloseFilesTestCase(unittest.TestCase):
     def tearDown(self):
         self.listener.stopListening()
 
+
+class AProtocol(protocol.Protocol):
+
+    def connectionMade(self):
+        reactor.callLater(0.1, self.transport.loseConnection)
+        self.factory.testcase.assertEquals(self.transport.getHost(),
+                          IPv4Address("TCP", self.transport.getHost().host, self.transport.getHost().port))
+        self.factory.testcase.assertEquals(self.transport.getPeer(),
+                          IPv4Address("TCP", self.transport.getPeer().host, self.transport.getPeer().port))
+        self.factory.testcase.assertEquals(self.transport.getPeer(), self.factory.ipv4addr)
+        self.factory.testcase.ran = 1
+
+class AClientFactory(protocol.ClientFactory):
+
+    def __init__(self, testcase, ipv4addr):
+        self.testcase = testcase
+        self.ipv4addr = ipv4addr
+
+    def buildProtocol(self, addr):
+        self.testcase.assertEquals(addr, self.ipv4addr)
+        self.testcase.assertEquals(addr, ('INET', self.ipv4addr.host, self.ipv4addr.port))
+        p = AProtocol()
+        p.factory = self
+        return p
+        
+class AServerFactory(protocol.ServerFactory):
+
+    def __init__(self, testcase, ipv4addr):
+        self.testcase = testcase
+        self.ipv4addr = ipv4addr
+    
+    def buildProtocol(self, addr):
+        self.testcase.assertEquals(addr, self.ipv4addr)
+        self.testcase.assertEquals(addr, (self.ipv4addr.host, self.ipv4addr.port))
+        p = AProtocol()
+        p.factory = self
+        return p
+
+class AddressTestCase(unittest.TestCase):
+
+    def getFreePort(self):
+        """Get an empty port."""
+        p = reactor.listenTCP(0, protocol.ServerFactory())
+        reactor.iterate(); reactor.iterate()
+        port = p.getHost().port
+        p.stopListening()
+        reactor.iterate(); reactor.iterate()
+        return port
+    
+    def testBuildProtocol(self):
+        portno = self.getFreePort()
+        p = reactor.listenTCP(0, AServerFactory(self, IPv4Address('TCP', '127.0.0.1', portno)))
+        reactor.iterate()
+        reactor.connectTCP("127.0.0.1", p.getHost().port,
+                           AClientFactory(self, IPv4Address("TCP", "127.0.0.1", p.getHost().port)),
+                           bindAddress=("127.0.0.1", portno))
+        self.runReactor(0.4, True)
+        p.stopListening()
+        self.assert_(hasattr(self, "ran"))
+        del self.ran
+
+
 try:
     import resource
 except ImportError:
