@@ -17,9 +17,10 @@ from twisted.trial import unittest
 import string, random, copy
 
 from twisted.web import server, resource, util
-from twisted.internet import app, defer
+from twisted.internet import defer, interfaces
 from twisted.protocols import http, loopback
 from twisted.python import log, reflect
+from twisted.internet.address import IPv4Address
 
 class DummyRequest:
     uri='http://dummy/'
@@ -76,6 +77,7 @@ class ResourceTestCase(unittest.TestCase):
         r = resource.Resource()
         self.failUnlessEqual([], r.listEntities())
         
+
 class SimpleResource(resource.Resource):
     def render(self, request):
         if http.CACHED in (request.setLastModified(10),
@@ -92,6 +94,7 @@ class SiteTest(unittest.TestCase):
         site = server.Site(sres1)
         assert site.getResourceFor(DummyRequest([''])) is sres2, "Got the wrong resource."
 
+
 # Conditional requests:
 # If-None-Match, If-Modified-Since
 
@@ -250,11 +253,12 @@ resource = Data('dynamic world','text/plain')
 class DummyChannel:
     class Baz:
         port = 80
-        type = 'TCP'
         def getPeer(self):
-            return None
+            return IPv4Address("TCP", 'client.example.com', 12344)
         def getHost(self):
-            return self.type, 'example.com', self.port
+            return IPv4Address("TCP", 'example.com', self.port)
+    class SSLBaz(Baz):
+        __implements__ = interfaces.ISSLTransport,
     transport = Baz()
     site = server.Site(resource.Resource())
 
@@ -299,9 +303,8 @@ class TestRequest(unittest.TestCase):
 
     def testPrePathURLSSLPortAndSSL(self):
         d = DummyChannel()
-        d.transport = DummyChannel.Baz()
+        d.transport = DummyChannel.SSLBaz()
         d.transport.port = 443
-        d.transport.type = 'SSL'
         request = server.Request(d, 1)
         request.setHost('example.com', 443)
         request.gotLength(0)
@@ -310,9 +313,8 @@ class TestRequest(unittest.TestCase):
 
     def testPrePathURLHTTPPortAndSSL(self):
         d = DummyChannel()
-        d.transport = DummyChannel.Baz()
+        d.transport = DummyChannel.SSLBaz()
         d.transport.port = 80
-        d.transport.type = 'SSL'
         request = server.Request(d, 1)
         request.setHost('example.com', 80)
         request.gotLength(0)
@@ -321,15 +323,24 @@ class TestRequest(unittest.TestCase):
 
     def testPrePathURLSSLNonDefault(self):
         d = DummyChannel()
-        d.transport = DummyChannel.Baz()
+        d.transport = DummyChannel.SSLBaz()
         d.transport.port = 81
-        d.transport.type = 'SSL'
         request = server.Request(d, 1)
         request.setHost('example.com', 81)
         request.gotLength(0)
         request.requestReceived('GET', '/foo/bar', 'HTTP/1.0')
         self.assertEqual(request.prePathURL(), 'https://example.com:81/foo/bar')
 
+    def testPrePathURLSetSSLHost(self):
+        d = DummyChannel()
+        d.transport = DummyChannel.Baz()
+        d.transport.port = 81
+        request = server.Request(d, 1)
+        request.setHost('foo.com', 81, 1)
+        request.gotLength(0)
+        request.requestReceived('GET', '/foo/bar', 'HTTP/1.0')
+        self.assertEqual(request.prePathURL(), 'https://foo.com:81/foo/bar')
+        
 class RootResource(resource.Resource):
     isLeaf=0
     def getChildWithDefault(self, name, request):
@@ -359,7 +370,52 @@ class RememberURLTest(unittest.TestCase):
             request.gotLength(0)
             request.requestReceived('GET', '/foo/bar/baz', 'HTTP/1.0')
             self.assertEqual(request.getRootURL(), "http://example.com/foo")
-        
+
+
+class NewRenderResource(resource.Resource):
+    def render_GET(self, request):
+        return "hi hi"
+
+    def render_HEH(self, request):
+        return "ho ho"
+
+
+class NewRenderTestCase(unittest.TestCase):
+    def _getReq(self):
+        d = DummyChannel()
+        d.site.resource.putChild('newrender', NewRenderResource())
+        d.transport = DummyChannel.Baz()
+        d.transport.port = 81
+        request = server.Request(d, 1)
+        request.setHost('example.com', 81)
+        request.gotLength(0)
+        return request
+
+    def testGoodMethods(self):
+        req = self._getReq()
+        req.requestReceived('GET', '/newrender', 'HTTP/1.0')
+        self.assertEquals(req.transport.getvalue().splitlines()[-1], 'hi hi')
+
+        req = self._getReq()
+        req.requestReceived('HEH', '/newrender', 'HTTP/1.0')
+        self.assertEquals(req.transport.getvalue().splitlines()[-1], 'ho ho')
+
+    def testBadMethods(self):
+        req = self._getReq()
+        req.requestReceived('CONNECT', '/newrender', 'HTTP/1.0')
+        self.assertEquals(req.code, 501)
+
+        req = self._getReq()
+        req.requestReceived('hlalauguG', '/newrender', 'HTTP/1.0')
+        self.assertEquals(req.code, 501)
+
+    def testImplicitHead(self):
+        req = self._getReq()
+        req.requestReceived('HEAD', '/newrender', 'HTTP/1.0')
+        self.assertEquals(req.code, 200)
+        self.assertEquals(-1, req.transport.getvalue().find('hi hi'))
+
+
 class SDResource(resource.Resource):
     def __init__(self,default):  self.default=default
     def getChildWithDefault(self,name,request):
