@@ -54,17 +54,27 @@ class SUXTest(TestCase):
 
 class MicroDOMTest(TestCase):
 
+    def testCaseSensitiveSoonCloser(self):
+	s = """
+	      <HTML><BODY>
+	      <P ALIGN="CENTER">
+		<A HREF="http://www.apache.org/"><IMG SRC="/icons/apache_pb.gif"></A>
+	      </P>
+
+	      <P>
+		This is an insane set of text nodes that should NOT be gathered under
+		the A tag above.
+	      </P>
+	      </BODY></HTML>
+	    """
+	d = microdom.parseString(s, beExtremelyLenient=1)
+        l = domhelpers.findNodesNamed(d.documentElement, 'a')
+	n = domhelpers.gatherTextNodes(l[0],1).replace('&nbsp;',' ')
+	self.assertEquals(n.find('insane'), -1)
+
     def testEmptyError(self):
         self.assertRaises(sux.ParseError, microdom.parseString, "")
 
-    def testEatingWhitespace(self):
-        s = """<hello>
-        </hello>"""
-        d = microdom.parseString(s)
-        self.failUnless(not d.documentElement.hasChildNodes(),
-                        d.documentElement.childNodes)
-        self.failUnless(d.isEqualToDocument(microdom.parseString('<hello></hello>')))
-
     def testTameDocument(self):
         s = """
         <test>
@@ -81,7 +91,6 @@ class MicroDOMTest(TestCase):
         self.assertEquals(
             domhelpers.gatherTextNodes(d.documentElement).strip() ,'test')
 
-
     def testAwfulTagSoup(self):
         s = """
         <html>
@@ -119,7 +128,7 @@ alert("I hate you");
         d = microdom.parseString(s, beExtremelyLenient=1)
         self.assertEquals(d.firstChild().firstChild().firstChild().data,
                           "(foo < bar) and (bar > foo)")
-        self.assertEquals(d.firstChild().childNodes[1].firstChild().data,
+        self.assertEquals(d.firstChild().getElementsByTagName("script")[1].firstChild().data,
                           "foo </scrip bar ")
 
     def testScriptLeniencyIntelligence(self):
@@ -129,6 +138,10 @@ alert("I hate you");
         self.assertEquals(microdom.parseString(s, beExtremelyLenient=1).firstChild().toxml(), s)
         s = """<script><![CDATA[lalal]]></script>"""
         self.assertEquals(microdom.parseString(s, beExtremelyLenient=1).firstChild().toxml(), s)
+        s = """<script> // <![CDATA[
+        lalal
+        //]]></script>"""
+        self.assertEquals(microdom.parseString(s, beExtremelyLenient=1).firstChild().toxml(), s)
         
     def testPreserveCase(self):
         s = '<eNcApSuLaTe><sUxor></sUxor><bOrk><w00T>TeXt</W00t></BoRk></EnCaPsUlAtE>'
@@ -350,7 +363,40 @@ alert("I hate you");
         self.failIf(d6.documentElement.hasAttribute('a'))
         self.assertEquals(d4.documentElement.toxml(), '<foo A="b">x</foo>')
         self.assertEquals(d5.documentElement.toxml(), '<foo a="b">x</foo>')
+    def testEatingWhitespace(self):
+        s = """<hello>
+        </hello>"""
+        d = microdom.parseString(s)
+        self.failUnless(not d.documentElement.hasChildNodes(),
+                        d.documentElement.childNodes)
+        self.failUnless(d.isEqualToDocument(microdom.parseString('<hello></hello>')))
 
+    def testLenientAmpersand(self):
+        prefix = "<?xml version='1.0'?>"
+        # we use <pre> so space will be preserved
+        for i, o in [("&", "&amp;"),
+                     ("& ", "&amp; "),
+                     ("&amp;", "&amp;"),
+                     ("&hello monkey", "&amp;hello monkey")]:
+            d = microdom.parseString("%s<pre>%s</pre>" % (prefix, i), beExtremelyLenient=1)
+            self.assertEquals(d.documentElement.toxml(), "<pre>%s</pre>" % o)
+        # non-space preserving
+        d = microdom.parseString("%s<t>hello & there</t>", beExtremelyLenient=1)
+        self.assertEquals(d.documentElement.toxml(), "<t>hello &amp; there</t>")
+    
+    def testInsensitiveLenient(self):
+        # testing issue #537
+        d = microdom.parseString("<?xml version='1.0'?><bar><xA><y>c</Xa> <foo></bar>", beExtremelyLenient=1)
+        self.assertEquals(d.documentElement.firstChild().toxml(), "<xa><y>c</y></xa>")
+
+    def testSpacing(self):
+        # testing issue #414
+        s = "<?xml version='1.0'?><p><q>smart</q> <code>HairDryer</code></p>"
+        d = microdom.parseString(s, beExtremelyLenient=1)
+        self.assertEquals(d.documentElement.toxml(), "<p><q>smart</q> <code>HairDryer</code></p>")
+
+    testSpacing.todo = "AAARGH white space swallowing screws this up"
+    
     def testUnicodeTolerance(self):
         import struct
         s = '<foo><bar><baz /></bar></foo>'
@@ -388,7 +434,18 @@ alert("I hate you");
         self.assertEquals(j3.toxml(), hdr+
                           u'<foo><div>\u221a</div><!--\u221a--></foo>'.encode('utf8'))
 
-        
+    def testNamedChildren(self):
+        tests = {"<foo><bar /><bar unf='1' /><bar>asdfadsf</bar>"
+                         "<bam/></foo>" : 3,
+                 '<foo>asdf</foo>' : 0,
+                 '<foo><bar><bar></bar></bar></foo>' : 1,
+                 }
+        for t in tests.keys():
+            node = microdom.parseString(t).documentElement
+            result = domhelpers.namedChildren(node, 'bar')
+            self.assertEquals(len(result), tests[t])
+            if result:
+                self.assert_(hasattr(result[0], 'tagName'))
 
     def testCloneNode(self):
         s = '<foo a="b"><bax>x</bax></foo>'
@@ -456,15 +513,15 @@ alert("I hate you");
         s2 = d.toprettyxml()
         self.assertEquals(d.documentElement.namespace,
                           "base")
-        self.assertEquals(d.documentElement.childNodes[0].namespace,
+        self.assertEquals(d.documentElement.getElementsByTagName("y")[0].namespace,
                           "base")
-        self.assertEquals(d.documentElement.childNodes[1].getAttributeNS('base','q'),
+        self.assertEquals(d.documentElement.getElementsByTagName("y")[1].getAttributeNS('base','q'),
                           '1')
         
         d2 = microdom.parseString(s2)
         self.assertEquals(d2.documentElement.namespace,
                           "base")
-        self.assertEquals(d2.documentElement.childNodes[0].namespace,
+        self.assertEquals(d2.documentElement.getElementsByTagName("y")[0].namespace,
                           "base")
-        self.assertEquals(d2.documentElement.childNodes[1].getAttributeNS('base','q'),
+        self.assertEquals(d2.documentElement.getElementsByTagName("y")[1].getAttributeNS('base','q'),
                           '1')
