@@ -35,7 +35,7 @@ Microdom mainly focuses on working with HTML and XHTML.
 from __future__ import nested_scopes
 
 # System Imports
-import copy
+import copy, re
 from cStringIO import StringIO
 
 # Twisted Imports
@@ -382,11 +382,11 @@ class Element(Node):
         self._markpos = markpos
         self.namespace = namespace
 
-    def addPrefixes(self, **kw):
+    def addPrefixes(self, pfxs):
         if self.nsprefixes is None:
-            self.nsprefixes = kw
+            self.nsprefixes = pfxs
         else:
-            self.nsprefixes.update(kw)
+            self.nsprefixes.update(pfxs)
 
     def endTag(self, endTagName):
         if not self.preserveCase:
@@ -499,8 +499,10 @@ class Element(Node):
                 else:
                     prefix = genprefix()
                     newprefixes[ns] = prefix
+                assert val is not None
                 writeattr(prefix+':'+key,val)
             else:
+                assert val is not None
                 writeattr(attr, val)
         if newprefixes:
             for ns, prefix in newprefixes.iteritems():
@@ -619,21 +621,41 @@ class MicroDOMParser(XMLParser):
             return self.elementstack[-1]
         else:
             return None
-        
+
+    COMMENT = re.compile(r"\s*/[/*]\s*")
+    
     def _fixScriptElement(self, el):
+        # this deals with case where there is comment or CDATA inside
+        # <script> tag and we want to do the right thing with it
         if not self.beExtremelyLenient or not len(el.childNodes) == 1:
             return
         c = el.firstChild()
         if isinstance(c, Text):
+            # deal with nasty people who do stuff like:
+            #   <script> // <!--
+            #      x = 1;
+            #   // --></script>
+            # tidy does this, for example.
+            prefix = ""
+            oldvalue = c.value
+            match = self.COMMENT.match(oldvalue)
+            if match:
+                prefix = match.group()
+                oldvalue = oldvalue[len(prefix):]
+            
+            # now see if contents are actual node and comment or CDATA
             try:
-                e = parseString("<a>%s</a>" % c.value).childNodes[0]
+                e = parseString("<a>%s</a>" % oldvalue).childNodes[0]
             except (ParseError, MismatchedTags):
                 return
             if len(e.childNodes) != 1:
                 return
             e = e.firstChild()
             if isinstance(e, (CDATASection, Comment)):
-                el.childNodes = [e]
+                el.childNodes = []
+                if prefix:
+                    el.childNodes.append(Text(prefix))
+                el.childNodes.append(e)
     
     def gotDoctype(self, doctype):
         self._mddoctype = doctype
@@ -674,7 +696,7 @@ class MicroDOMParser(XMLParser):
                      preserveCase=self.preserveCase,
                      namespace=namespaces.get(''))
         revspaces = _reverseDict(newspaces)
-        el.addPrefixes(**revspaces)
+        el.addPrefixes(revspaces)
         
         if newspaces:
             rscopy = self.nsstack[-1][2].copy()
@@ -683,7 +705,7 @@ class MicroDOMParser(XMLParser):
         self.elementstack.append(el)
         if parent:
             parent.appendChild(el)
-        if (self.beExtremelyLenient and name in self.soonClosers):
+        if (self.beExtremelyLenient and el.tagName in self.soonClosers):
             self.gotTagEnd(name)
 
     def _gotStandalone(self, factory, data):
@@ -742,7 +764,7 @@ class MicroDOMParser(XMLParser):
                 if self.elementstack:
                     lastEl = self.elementstack[0]
                     for idx in xrange(len(self.elementstack)):
-                        if self.elementstack[-(idx+1)].tagName == name:
+                        if self.elementstack[-(idx+1)].tagName == cname:
                             self.elementstack[-(idx+1)].endTag(name)
                             break
                     else:
