@@ -67,13 +67,13 @@ class ProxyClient(http.HTTPClient):
 
     def handleEndHeaders(self):
         self.father.transport.write("\r\n")
-
-    def connectionLost(self, reason):
-        self.father.transport.loseConnection()
     
-    def handleResponse(self, buffer):
+    def handleResponsePart(self, buffer):
         self.father.transport.write(buffer)
-        self.transport.loseConnection()    
+
+    def handleResponseEnd(self):
+        self.transport.loseConnection()
+        self.father.channel.transport.loseConnection()
 
 
 class ProxyClientFactory(protocol.ClientFactory):
@@ -102,7 +102,7 @@ class ProxyClientFactory(protocol.ClientFactory):
 
 
 class ProxyRequest(http.Request):
-    """Used by Proxy to impelement a simple web proxy."""
+    """Used by Proxy to implement a simple web proxy."""
 
     protocols = {'http': ProxyClientFactory}
     ports = {'http': 80}
@@ -185,11 +185,11 @@ class ReverseProxyResource(resource.Resource):
     def render(self, request):
         request.received_headers['host'] = self.host
         request.content.seek(0, 0)
-	qs = urlparse.urlparse(request.uri)[4]
-	if qs:
-	    rest = self.path + '?' + qs
-	else:
-	    rest = self.path
+        qs = urlparse.urlparse(request.uri)[4]
+        if qs:
+            rest = self.path + '?' + qs
+        else:
+            rest = self.path
         clientFactory = ProxyClientFactory(request.method, rest, 
                                      request.clientproto, 
                                      request.getAllHeaders(),
