@@ -168,22 +168,46 @@ class Resource:
     
     def putChild(self, path, child):
         """Register a static child.
+
+        You almost certainly don't want '/' in your path. If you
+        intended to have the root of a folder, e.g. /foo/, you want
+        path to be ''.
         """
         self.children[path] = child
         child.server = self.server
 
     def render(self, request):
-        """Render a given resource.
+        """Render a given resource. See L{IResource}'s render method.
 
-        This must be implemented in all subclasses of Resource.
+        I delegate to methods of self with the form 'render_METHOD'
+        where METHOD is the HTTP that was used to make the
+        request. Examples: render_GET, render_HEAD, render_POST, and
+        so on. Generally you should implement those methods instead of
+        overriding this one.
 
-        The return value of this method will be the rendered page, unless the
-        return value is twisted.web.server.NOT_DONE_YET, in which case it is
-        this class's responsibility to write the results to
+        render_METHOD methods are expected to return a string which
+        will be the rendered page, unless the return value is
+        twisted.web.server.NOT_DONE_YET, in which case it is this
+        class's responsibility to write the results to
         request.write(data), then call request.finish().
+
+        Old code that overrides render() directly is likewise expected
+        to return a string or NOT_DONE_YET.
         """
-        raise NotImplementedError("%s.render called" %
-                                  reflect.qual(self.__class__))
+        m = getattr(self, 'render_' + request.method, None)
+        if not m:
+            from twisted.web.server import UnsupportedMethod
+            raise UnsupportedMethod(getattr(self, 'allowedMethods', ()))
+        return m(request)
+
+    def render_HEAD(self, request):
+        """Default handling of HEAD method.
+        
+        I just return self.render_GET(request). When method is HEAD,
+        the framework will handle this correctly.
+        """
+        return self.render_GET(request)
+        
 
 
 #t.w imports
