@@ -86,7 +86,8 @@ class UnsupportedMethod(Exception):
 
     def __init__(self, allowedMethods, *args):
         Exception.__init__(self, allowedMethods, *args)
-
+        self.allowedMethods = allowedMethods
+        
         if not operator.isSequenceType(allowedMethods):
             why = "but my first argument is not a sequence."
             s = ("First argument must be a sequence of"
@@ -116,6 +117,11 @@ class Request(pb.Copyable, http.Request, components.Componentized):
         self.content.seek(0, 0)
         x['content_data'] = self.content.read()
         x['remote'] = pb.ViewPoint(issuer, self)
+
+        # Address objects aren't jellyable
+        x['host'] = tuple(x['host'])
+        x['client'] = tuple(x['client'])
+
         return x
 
     # HTML generation helpers
@@ -235,10 +241,15 @@ class Request(pb.Copyable, http.Request, components.Componentized):
 
     def processingFailed(self, reason):
         log.err(reason)
-        body = ("<html><head><title>web.Server Traceback (most recent call last)</title></head>"
-                "<body><b>web.Server Traceback (most recent call last):</b>\n\n"
-                "%s\n\n</body></html>\n"
-                % webutil.formatFailure(reason))
+        if self.site.displayTracebacks:
+            body = ("<html><head><title>web.Server Traceback (most recent call last)</title></head>"
+                    "<body><b>web.Server Traceback (most recent call last):</b>\n\n"
+                    "%s\n\n</body></html>\n"
+                    % webutil.formatFailure(reason))
+        else:
+            body = ("<html><head><title>Processing Failed</title></head><body>"
+                  "<b>Processing Failed</b></body></html>")
+
         self.setResponseCode(http.INTERNAL_SERVER_ERROR)
         self.setHeader('content-type',"text/html")
         self.setHeader('content-length', str(len(body)))
@@ -336,7 +347,7 @@ class Request(pb.Copyable, http.Request, components.Componentized):
         return self.session
 
     def prePathURL(self):
-        inet, addr, port = self.getHost()
+        port = self.getHost().port
         if self.isSecure():
             default = 443
         else:
@@ -430,7 +441,8 @@ class Site(http.HTTPFactory):
 
     counter = 0
     requestFactory = Request
-
+    displayTracebacks = True
+    
     def __init__(self, resource, logPath=None, timeout=60*60*12):
         """Initialize.
         """
