@@ -70,10 +70,16 @@ class Data(resource.Resource):
         return self.data
 
 def addSlash(request):
-    return "http%s://%s%s/" % (
+    qs = ''
+    qindex = string.find(request.uri, '?')
+    if qindex != -1:
+        qs = request.uri[qindex:]
+        
+    return "http%s://%s%s/%s" % (
         request.isSecure() and 's' or '',
         request.getHeader("host"),
-        (string.split(request.uri,'?')[0]))
+        (string.split(request.uri,'?')[0]),
+        qs)
 
 class Redirect(resource.Resource):
     def __init__(self, request):
@@ -139,7 +145,9 @@ def loadMimeTypes(mimetype_locations=['/etc/mime.types']):
     # files (this defaults to ['/etc/mime.types']).
     for location in mimetype_locations:
         if os.path.exists(location):
-            contentTypes.update(mimetypes.read_mime_types(location))
+            more = mimetypes.read_mime_types(location)
+            if more is not None:
+                contentTypes.update(more)
             
     return contentTypes
 
@@ -270,8 +278,7 @@ class File(resource.Resource, styles.Versioned, filepath.FilePath):
                 return self.directoryListing()
 
         if not fpath.exists():
-            searchNames = [(path + ext) for ext in self.ignoredExts]
-            fpath = self.childSearchPreauth(*searchNames)
+            fpath = fpath.siblingExtensionSearch(*self.ignoredExts)
             if fpath is None:
                 return self.childNotFound
 
@@ -306,7 +313,7 @@ class File(resource.Resource, styles.Versioned, filepath.FilePath):
                                                           self.defaultType)
 
         if not self.exists():
-            return error.NoResource("File not found.").render(request)
+            return self.childNotFound.render(request)
 
         if self.isdir():
             return self.redirect(request)
@@ -402,6 +409,7 @@ class File(resource.Resource, styles.Versioned, filepath.FilePath):
         # refactoring by steps, here - constructor should almost certainly take these
         f.processors = self.processors
         f.indexNames = self.indexNames[:]
+        f.childNotFound = self.childNotFound
         return f
 
 
