@@ -1,19 +1,61 @@
+# -*- test-case-name: twisted.test.test_web -*-
+#
+# Twisted, the Framework of Your Internet
+# Copyright (C) 2001 Matthew W. Lefkowitz
+#
+# This library is free software; you can redistribute it and/or
+# modify it under the terms of version 2.1 of the GNU Lesser General Public
+# License as published by the Free Software Foundation.
+#
+# This library is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# Lesser General Public License for more details.
+#
+# You should have received a copy of the GNU Lesser General Public
+# License along with this library; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""This module is DEPRECATED."""
+
+import warnings
+warnings.warn("This module is deprecated, please use Woven instead.", DeprecationWarning)
 
 # System Imports
-import string, time, types, traceback, copy, pprint, sys, os, string
+import string, time, types, traceback, pprint, sys, os
 import linecache
 import re
 from cStringIO import StringIO
 
 # Twisted Imports
-from twisted.python import defer, rebuild, reflect, failure, log
+from twisted.python import failure, log, rebuild, reflect, util
+from twisted.internet import defer
 from twisted.protocols import http
 
 # Sibling Imports
 import html, resource, error
+import util as webutil
+
+#backwards compatibility
+from util import formatFailure, htmlrepr, htmlUnknown, htmlDict, htmlList,\
+                 htmlInst, htmlString, htmlReprTypes
+
+
+
 from server import NOT_DONE_YET
 
+True = (1==1)
+False = not True
+
 """A twisted web component framework.
+
+This code is mainly old stuff which is being kept around for historical
+reasons.  It is being very gradually deprecated in favor of twisted.web.woven;
+however, woven is still maturing and does not yet have equivalent functionality
+in all areas.  (Though in some ways, it's already much more featureful.)  This
+module will be removed when woven has reached feature- and documentation-parity
+with it, and all Twisted functionality that uses it has been rewritten.
+
 """
 
 # magic value that sez a widget needs to take over the whole page.
@@ -28,110 +70,21 @@ def _ellipsize(x):
         return y[:1024]+"..."
     return y
 
-def htmlrepr(x):
-    return htmlReprTypes.get(type(x), htmlUnknown)(x)
-
-def htmlUnknown(x):
-    return '<CODE>'+html.escape(repr(x))+'</code>'
-
-def htmlDict(d):
-    io = StringIO()
-    w = io.write
-    w('<table bgcolor="#cccc99"><tr><th colspan="2" align="left">Dictionary %d</th></tr>' % id(d))
-    for k, v in d.items():
-        if k == '__builtins__':
-            v = 'builtin dictionary'
-        w('\n<tr bgcolor="#ffff99"><td valign="top"><b>%s<b></td>\n<td>%s</td></tr>\n' % (htmlrepr(k), htmlrepr(v)))
-    w('</table>')
-    return io.getvalue()
-
-def htmlList(l):
-    io = StringIO()
-    w = io.write
-    w('<table bgcolor="#7777cc"><tr><th colspan="2" align="left">List %d</th></tr>' % id(l))
-    for i in l:
-        w('<tr bgcolor="#9999ff"><td>%s</td></tr>\n' % htmlrepr(i))
-    w('</table>\n')
-    return io.getvalue()
-
-def htmlInst(i):
-    if hasattr(i, "__html__"):
-        s = i.__html__()
-    else:
-        s = '<CODE>'+html.escape(repr(i))+'</code>'
-    return '''<table bgcolor="#cc7777"><tr><td><b>%s</b> instance</td></tr>
-              <tr bgcolor="#ff9999"><td>%s</td></tr>
-              </table>
-              ''' % (i.__class__, s)
-
-def htmlString(s):
-    return html.escape(repr(s))
-
-htmlReprTypes = {types.DictType: htmlDict,
-                 types.ListType: htmlList,
-                 types.InstanceType: htmlInst,
-                 types.StringType: htmlString}
-
-def formatFailure(myFailure):
-    if not isinstance(myFailure, failure.Failure):
-        return html.PRE(str(myFailure))
-    io = StringIO()
-    w = io.write
-    w("<table>")
-    w('<th align="left" colspan="3"><font color="red">%s: %s</font></th>' % (myFailure.type, myFailure.value))
-    line = 0
-    for method, filename, lineno, localVars, globalVars in myFailure.frames:
-        # Cheat to make tracebacks shorter.
-        if filename == '<string>':
-            continue
-        # file, line number
-        w('<tr bgcolor="#%s"><td colspan="2" valign="top">%s, line %s in <b>%s</b><br><table width="100%%">' % (["bbbbbb", "cccccc"][line % 2], filename, lineno, method))
-        snippet = ''
-        for snipLineNo in range(lineno-2, lineno+2):
-            snipLine = linecache.getline(filename, snipLineNo)
-            snippet = snippet + snipLine
-            snipLine = string.replace(
-                string.replace(html.escape(string.rstrip(snipLine)),
-                               '  ','&nbsp;'),
-                '\t', '&nbsp; &nbsp; &nbsp; &nbsp; ')
-
-
-            if snipLineNo == lineno:
-                color = 'bgcolor="#ffffff"'
-            else:
-                color = ''
-            w('<tr %s><td>%s</td><td><code>%s</code></td></tr>' % (color, snipLineNo,snipLine))
-        w('</table></td></tr>')
-        w('<tr bgcolor="#%s">' % (["bbbbbb", "cccccc"][line % 2]))
-        # local vars
-        for nm, varList in ('Locals', localVars), ('Globals', globalVars):
-            w('<td valign="top"><table><tr><th align="left" colspan="2">'
-              '%s'
-              '</th></tr>' % nm)
-            for name, var in varList:
-                if re.search(r'\W'+name+r'\W', snippet):
-                    w('<tr><td valign="top"><b>%s</b></td>'
-                      '<td>%s</td></tr>' % (name, htmlrepr(var)))
-            w('</table></td>')
-        w('</td>')
-        w('</tr>')
-        line = line + 1
-    return io.getvalue()
 
 class Widget:
     """A component of a web page.
     """
     title = None
     def getTitle(self, request):
-        return self.title or str(self.__class__)
+        return self.title or reflect.qual(self.__class__)
 
     def display(self, request):
         """Implement me to represent your widget.
 
-        I must return a list of strings and twisted.python.defer.Deferred
+        I must return a list of strings and twisted.internet.defer.Deferred
         instances.
         """
-        raise NotImplementedError("%s.display" % str(self.__class__))
+        raise NotImplementedError("%s.display" % reflect.qual(self.__class__))
 
 class StreamWidget(Widget):
     """A 'streamable' component of a webpage.
@@ -140,7 +93,7 @@ class StreamWidget(Widget):
     def stream(self, write, request):
         """Call 'write' multiple times with a string argument to represent this widget.
         """
-        raise NotImplementedError("%s.stream" % self.__class__)
+        raise NotImplementedError("%s.stream" % reflect.qual(self.__class__))
 
     def display(self, request):
         """Produce a list containing a single string.
@@ -152,7 +105,7 @@ class StreamWidget(Widget):
                 return result
             return l
         except:
-            return [formatFailure(failure.Failure())]
+            return [webutil.formatFailure(failure.Failure())]
 
 class WidgetMixin(Widget):
     """A mix-in wrapper for a Widget.
@@ -231,9 +184,8 @@ class Presentation(Widget):
                 try:
                     x = eval(elem, namespace, namespace)
                 except:
-                    io = StringIO()
                     log.deferr()
-                    tm.append(formatFailure(failure.Failure()))
+                    tm.append(webutil.formatFailure(failure.Failure()))
                 else:
                     if isinstance(x, types.ListType):
                         tm.extend(x)
@@ -250,39 +202,63 @@ class Presentation(Widget):
 
 
 def htmlFor_hidden(write, name, value):
-    write('<INPUT TYPE="hidden" NAME="%s" VALUE="%s">' % (name, value))
+    write('<INPUT TYPE="hidden" NAME="%s" VALUE="%s" />' % (name, value))
+
 def htmlFor_file(write, name, value):
-    write('<INPUT SIZE=60 TYPE="file" NAME="%s">' % name)
+    write('<INPUT SIZE="60" TYPE="file" NAME="%s" />' % name)
+
 def htmlFor_string(write, name, value):
-    write('<INPUT SIZE=60 TYPE="text" NAME="%s" VALUE="%s">' % (name, value))
+    write('<INPUT SIZE="60" TYPE="text" NAME="%s" VALUE="%s" />' % (name, value))
+
 def htmlFor_password(write, name, value):
-    write('<INPUT SIZE=60 TYPE="password" NAME=%s>' % name)
+    write('<INPUT SIZE="60" TYPE="password" NAME="%s" />' % name)
+
 def htmlFor_text(write, name, value):
-    write('<TEXTAREA COLS="60" ROWS="10" NAME="%s" WRAP="virtual">%s</textarea>' % (name, value))
-def htmlFor_menu(write, name, value):
-    "Value of the format [(optionName, displayName), ...]"
-    write('  <SELECT NAME="%s">\n' % name)
-    for optionName, displayName in value:
-        write('    <OPTION VALUE="%s">%s</option>\n' % (optionName, displayName))
+    write('<textarea COLS="60" ROWS="10" NAME="%s" WRAP="virtual">%s</textarea>' % (name, value))
+
+def htmlFor_menu(write, name, value, allowMultiple=False):
+    "Value of the format [(optionName, displayName[, selected]), ...]"
+
+    write('  <select NAME="%s"%s>\n' %
+          (name, (allowMultiple and " multiple") or ''))
+
+    for v in value:
+        optionName, displayName, selected = util.padTo(3, v)
+        selected = (selected and " selected") or ''
+        write('    <option VALUE="%s"%s>%s</option>\n' %
+              (optionName, selected, displayName))
+    if not value:
+        write('    <option VALUE=""></option>\n')
     write("  </select>\n")
+
+def htmlFor_multimenu(write, name, value):
+    "Value of the format [(optionName, displayName[, selected]), ...]"
+    return htmlFor_menu(write, name, value, True)
+
 def htmlFor_checkbox(write, name, value):
     "A checkbox."
     if value:
-        value = 'checked'
+        value = 'checked = "1"'
     else:
         value = ''
-    write('<INPUT TYPE="checkbox" NAME="__checkboxes__" VALUE="%s" %s>\n' % (name, value))
+    write('<INPUT TYPE="checkbox" NAME="__checkboxes__" VALUE="%s" %s />\n' % (name, value))
 
 def htmlFor_checkgroup(write, name, value):
     "A check-group."
     for optionName, displayName, checked in value:
-        checked = (checked and 'checked') or ''
-        write('<INPUT TYPE="checkbox" NAME="%s" VALUE="%s" %s>%s<br>\n' % (name, optionName, checked, displayName))
+        checked = (checked and 'checked = "1"') or ''
+        write('<INPUT TYPE="checkbox" NAME="%s" VALUE="%s" %s />%s<br />\n' % (name, optionName, checked, displayName))
+
+def htmlFor_radio(write, name, value):
+    "A radio button group."
+    for optionName, displayName, checked in value:
+        checked = (checked and 'checked = "1"') or ''
+        write('<INPUT TYPE="radio" NAME="%s" VALUE="%s" %s />%s<br />\n' % (name, optionName, checked, displayName))
 
 class FormInputError(Exception):
     pass
 
-class Form(StreamWidget):
+class Form(Widget):
     """I am a web form.
 
     In order to use me, you probably want to set self.formFields (or override
@@ -292,10 +268,12 @@ class Form(StreamWidget):
       |  from twisted.web import widgets
       |  class HelloForm(widgets.Form):
       |      formFields = [
-      |          ['string', 'Who to greet?', 'whoToGreet', 'World'],
+      |          ['string', 'Who to greet?', 'whoToGreet', 'World',
+      |            'This is for choosing who to greet.'],
       |          ['menu', 'How to greet?', 'how', [('cheerfully', 'with a smile'),
       |                                            ('sullenly', 'without enthusiasm'),
-      |                                            ('spontaneously', 'on the spur of the moment')]]]
+      |                                            ('spontaneously', 'on the spur of the moment')]]
+      |            'This is for choosing how to greet them.']
       |      def process(self, write, request, submit, whoToGreet, how):
       |          write('The web wakes up and %s says, \"Hello, %s!\"' % (how, whoToGreet))
 
@@ -310,47 +288,60 @@ class Form(StreamWidget):
         'file': htmlFor_file,
         'string': htmlFor_string,
         'int': htmlFor_string,
+        'float': htmlFor_string,
         'text': htmlFor_text,
         'menu': htmlFor_menu,
+        'multimenu': htmlFor_multimenu,
         'password': htmlFor_password,
         'checkbox': htmlFor_checkbox,
-        'checkgroup': htmlFor_checkgroup
+        'checkgroup': htmlFor_checkgroup,
+        'radio': htmlFor_radio,
     }
 
     formParse = {
-        'int': int
+        'int': int,
+        'float': float,
     }
 
     formFields = [
     ]
 
+    # do we raise an error when we get extra args or not?
+    formAcceptExtraArgs = 0
+
     def getFormFields(self, request, fieldSet = None):
-        """I return a list of lists describing this form.
+        """I return a list of lists describing this form, or a Deferred.
 
         This information is used both to display the form and to process it.
         The list is in the following format::
 
-          | [['Input Type',   'Display Name',   'Input Name',   'Input Value'],
-          |  ['Input Type 2', 'Display Name 2', 'Input Name 2', 'Input Value 2']
+          | [['Input Type',   'Display Name',   'Input Name',   'Input Value', 'Description'],
+          |  ['Input Type 2', 'Display Name 2', 'Input Name 2', 'Input Value 2', 'Description 2']
           |  ...]
 
         Valid values for 'Input Type' are:
 
-          * 'hidden': a hidden field that contains a string that the user won't change
+          - 'hidden': a hidden field that contains a string that the user won't change
 
-          * 'string': a short string
+          - 'string': a short string
 
-          * 'int': an integer
+          - 'int': an integer, e.g. 1, 0, 25 or -23
 
-          * 'text': a longer text field, suitable for entering paragraphs
+          - 'float': a float, e.g. 1.0, 2, -3.45, or 28.4324231
 
-          * 'menu': an HTML SELECT input, a list of choices
+          - 'text': a longer text field, suitable for entering paragraphs
 
-          * 'checkgroup': a group of checkboxes
+          - 'menu': an HTML SELECT input, a list of choices
 
-          * 'password': a 'string' field where the contents are not visible as the user types
+          - 'multimenu': an HTML SELECT input allowing multiple choices
 
-          * 'file': a file-upload form (EXPERIMENTAL)
+          - 'checkgroup': a group of checkboxes
+
+          - 'radio': a group of radio buttons
+
+          - 'password': a 'string' field where the contents are not visible as the user types
+
+          - 'file': a file-upload form (EXPERIMENTAL)
 
         'Display Name' is a descriptive string that will be used to
         identify the field to the user.
@@ -362,8 +353,11 @@ class Form(StreamWidget):
         The 'Input Value' is usually a string, but its value can depend on the
         'Input Type'.  'int' it is an integer, 'menu' it is a list of pairs of
         strings, representing (value, name) pairs for the menu options.  Input
-        value for 'checkgroup' should be a list of ('inputName', 'Display
-        Name', 'checked') triplets.
+        value for 'checkgroup' and 'radio' should be a list of ('inputName',
+        'Display Name', 'checked') triplets.
+
+        The 'Description' field is an (optional) string which describes the form
+        item to the user.
 
         If this result is statically determined for your Form subclass, you can
         assign it to FormSubclass.formFields; if you need to determine it
@@ -385,7 +379,14 @@ class Form(StreamWidget):
             fieldSet = self.formFields
         if not self.shouldProcess(request):
             return fieldSet
-        for inputType, displayName, inputName, inputValue in fieldSet:
+
+        for field in fieldSet:
+            if len(field)==5:
+                inputType, displayName, inputName, inputValue, description = field
+            else:
+                inputType, displayName, inputName, inputValue = field
+                description = ""
+
             if inputType == 'checkbox':
                 if request.args.has_key('__checkboxes__'):
                     if inputName in request.args['__checkboxes__']:
@@ -394,7 +395,7 @@ class Form(StreamWidget):
                         inputValue = 0
                 else:
                     inputValue = 0
-            elif inputType == 'checkgroup':
+            elif inputType in ('checkgroup', 'radio'):
                 if request.args.has_key(inputName):
                     keys = request.args[inputName]
                 else:
@@ -406,37 +407,46 @@ class Form(StreamWidget):
                     inputValue.append([optionName, optionDisplayName, checked])
             elif request.args.has_key(inputName):
                 iv = request.args[inputName][0]
-                if inputType == 'menu':
+                if inputType in ['menu', 'multimenu']:
                     if iv in inputValue:
                         inputValue.remove(iv)
                         inputValue.insert(0, iv)
                 else:
                     inputValue = iv
-            fields.append([inputType, displayName, inputName, inputValue])
+            fields.append([inputType, displayName, inputName, inputValue, description])
         return fields
 
     submitNames = ['Submit']
+    actionURI = ''
 
     def format(self, form, write, request):
         """I display an HTML FORM according to the result of self.getFormFields.
         """
-        write('<FORM ENCTYPE="multipart/form-data" METHOD="post" ACTION="%s">\n'
-              '<TABLE BORDER="0">\n' % request.uri)
-        for inputType, displayName, inputName, inputValue in form:
-            write('<TR>\n<TD ALIGN="right" VALIGN="top"><B>%s</b></td>\n'
-                  '<TD VALIGN="%s">\n' %
+        write('<form ENCTYPE="multipart/form-data" METHOD="post" ACTION="%s">\n'
+              '<table BORDER="0">\n' % (self.actionURI or request.uri))
+
+        for field in form:
+            if len(field) == 5:
+                inputType, displayName, inputName, inputValue, description = field
+            else:
+                inputType, displayName, inputName, inputValue = field
+                description = ""
+            write('<tr>\n<td ALIGN="right" VALIGN="top"><B>%s</B></td>\n'
+                  '<td VALIGN="%s">\n' %
                   (displayName, ((inputType == 'text') and 'top') or 'middle'))
             self.formGen[inputType](write, inputName, inputValue)
-            write('</td>\n</tr>\n')
-        write('<TR><TD></TD><TD ALIGN="left"><hr>\n')
+            write('\n<br />\n<font size="-1">%s</font></td>\n</tr>\n' % description)
+
+
+        write('<tr><td></td><td ALIGN="left"><hr />\n')
         for submitName in self.submitNames:
-            write('<INPUT TYPE="submit" NAME="submit" VALUE="%s">\n' % submitName)
+            write('<INPUT TYPE="submit" NAME="submit" VALUE="%s" />\n' % submitName)
         write('</td></tr>\n</table>\n'
-              '<INPUT TYPE="hidden" NAME="__formtype__" VALUE="%s">\n'
-              % (str(self.__class__)))
+              '<INPUT TYPE="hidden" NAME="__formtype__" VALUE="%s" />\n'
+              % (reflect.qual(self.__class__)))
         fid = self.getFormID()
         if fid:
-            write('<INPUT TYPE="hidden" NAME="__formid__" VALUE="%s">\n' % fid)
+            write('<INPUT TYPE="hidden" NAME="__formid__" VALUE="%s" />\n' % fid)
         write("</form>\n")
 
     def getFormID(self):
@@ -470,14 +480,15 @@ class Form(StreamWidget):
         The remainder of my arguments must be correctly named.  They will each be named after one of the
 
         """
-        write("Submit: %s <br> %s" % (submit, html.PRE(pprint.PrettyPrinter().pformat(kw))))
+        write("<pre>Submit: %s <br /> %s</pre>" % (submit, html.PRE(pprint.PrettyPrinter().pformat(kw))))
 
     def _doProcess(self, form, write, request):
         """(internal) Prepare arguments for self.process.
         """
-        args = copy.copy(request.args)
+        args = request.args.copy()
         kw = {}
-        for inputType, displayName, inputName, inputValue in form:
+        for field in form:
+            inputType, displayName, inputName, inputValue = field[:4]
             if inputType == 'checkbox':
                 if request.args.has_key('__checkboxes__'):
                     if inputName in request.args['__checkboxes__']:
@@ -486,7 +497,7 @@ class Form(StreamWidget):
                         formData = 0
                 else:
                     formData = 0
-            elif inputType == 'checkgroup':
+            elif inputType in ['checkgroup', 'radio', 'multimenu']:
                 if args.has_key(inputName):
                     formData = args[inputName]
                     del args[inputName]
@@ -502,7 +513,10 @@ class Form(StreamWidget):
                 formData = formData[0]
                 method = self.formParse.get(inputType)
                 if method:
-                    formData = method(formData)
+                    try:
+                        formData = method(formData)
+                    except:
+                        raise FormInputError("%s: %s" % (displayName, "error"))
             kw[inputName] = formData
         submitAction = args.get('submit')
         if submitAction:
@@ -510,7 +524,7 @@ class Form(StreamWidget):
         for field in ['submit', '__formtype__', '__checkboxes__']:
             if args.has_key(field):
                 del args[field]
-        if args:
+        if args and not self.formAcceptExtraArgs:
             raise FormInputError("unknown fields: %s" % repr(args))
         return apply(self.process, (write, request, submitAction), kw)
 
@@ -519,14 +533,14 @@ class Form(StreamWidget):
 
         By default, this will make the message appear in red, bold italics.
         """
-        return '<FONT COLOR=RED><B><I>%s</i></b></font><br>\n' % error
+        return '<font color="#f00"><b><i>%s</i></b></font><br />\n' % error
 
     def shouldProcess(self, request):
         args = request.args
         fid = self.getFormID()
         return (args and # there are arguments to the request
                 args.has_key('__formtype__') and # this is a widgets.Form request
-                args['__formtype__'][0] == str(self.__class__) and # it is for a form of my type
+                args['__formtype__'][0] == reflect.qual(self.__class__) and # it is for a form of my type
                 ((not fid) or # I am only allowed one form per page
                  (args.has_key('__formid__') and # if I distinguish myself from others, the request must too
                   args['__formid__'][0] == fid))) # I am in fact the same
@@ -544,17 +558,37 @@ class Form(StreamWidget):
         self.format(self.getFormFields(req), w, req)
         return l
 
-    def stream(self, write, request):
-        """Render the results of displaying or processing the form.
-        """
-        args = request.args
+    def display(self, request):
+        """Display the form."""
         form = self.getFormFields(request)
-        if self.shouldProcess(request):
-            try:
-                return self._doProcess(form, write, request)
-            except FormInputError, fie:
-                write(self.formatError(str(fie)))
-        return self.format(form, write, request)
+        if isinstance(form, defer.Deferred):
+            if self.shouldProcess(request):
+                form.addCallback(lambda form, f=self._displayProcess, r=request: f(r, form))
+            else:
+                form.addCallback(lambda form, f=self._displayFormat, r=request: f(r, form))
+            return [form]
+        else:
+            if self.shouldProcess(request):
+                return self._displayProcess(request, form)
+            else:
+                return self._displayFormat(request, form)
+
+    def _displayProcess(self, request, form):
+        l = []
+        write = l.append
+        try:
+            val = self._doProcess(form, write, request)
+            if val:
+                l.extend(val)
+        except FormInputError, fie:
+            write(self.formatError(str(fie)))
+        return l
+
+    def _displayFormat(self, request, form):
+        l = []
+        self.format(form, l.append, request)
+        return l
+
 
 
 class DataWidget(Widget):
@@ -616,19 +650,15 @@ class RenderSession:
         self.request = request
         self.needsHeaders = 0
         self.beforeBody = 1
-
-        toArm = []
+        self.forgotten = 0
+        self.pauseList = []
         for i in range(len(self.lst)):
             item = self.lst[i]
             if isinstance(item, defer.Deferred):
-                self.lst[i] = (self._addDeferred(item), item)
-                toArm.append(item)
-
+                self._addDeferred(item, self.lst, i)
         self.keepRendering()
-        for item in toArm:
-            item.arm()
 
-    def _addDeferred(self, deferred):
+    def _addDeferred(self, deferred, lst, idx):
         sentinel = self.Sentinel()
         if hasattr(deferred, 'needsHeader'):
             # You might want to set a header from a deferred, in which
@@ -637,11 +667,16 @@ class RenderSession:
             args = (sentinel, 1)
         else:
             args = (sentinel, 0)
+        lst[idx] = sentinel, deferred
+        deferred.pause()
+        self.pauseList.append(deferred)
         deferred.addCallbacks(self.callback, self.callback,
                               callbackArgs=args, errbackArgs=args)
-        return sentinel
+
 
     def callback(self, result, sentinel, decNeedsHeaders):
+        if self.forgotten:
+            return
         if result != FORGET_IT:
             self.needsHeaders = self.needsHeaders - decNeedsHeaders
         else:
@@ -651,9 +686,11 @@ class RenderSession:
         if not type(result) in (types.ListType, types.TupleType):
             result = [result]
 
-        # If the deferred does not wish to produce it's result all at
+        # If the deferred does not wish to produce its result all at
         # once, it can give us a partial result as
         #  (NOT_DONE_YET, partial_result)
+        ## XXX: How would a deferred go about producing the result in multiple
+        ## stages?? --glyph
         if result[0] is NOT_DONE_YET:
             done = 0
             result = result[1]
@@ -662,12 +699,10 @@ class RenderSession:
         else:
             done = 1
 
-        toArm = []
         for i in xrange(len(result)):
             item = result[i]
             if isinstance(item, defer.Deferred):
-                result[i] = (self._addDeferred(item), item)
-                toArm.append(item)
+                self._addDeferred(item, result, i)
 
         for position in range(len(self.lst)):
             item = self.lst[position]
@@ -675,27 +710,24 @@ class RenderSession:
                 if item[0] is sentinel:
                     break
         else:
-            raise 'Sentinel for Deferred not found!'
+            raise AssertionError('Sentinel for Deferred not found!')
 
         if done:
             self.lst[position:position+1] = result
         else:
             self.lst[position:position] = result
 
-        # Consolidate strings to minimize length of list,
-        # Good try, but this seems really buggy
-##        for i in range(1, len(self.lst)):
-##            last = type(self.lst[i-1]) is types.StringType
-##            this = type(self.lst[i]) is types.StringType
-##            if last and this:
-##                self.lst[i-1:i+1] = [self.lst[i-1] + self.lst[i]]
-
         self.keepRendering()
-        for r in toArm:
-            r.arm()
 
 
     def keepRendering(self):
+        while self.pauseList:
+            pl = self.pauseList
+            self.pauseList = []
+            for deferred in pl:
+                deferred.unpause()
+            return
+
         if self.needsHeaders:
             # short circuit actual rendering process until we're sure no
             # more deferreds need to set headers...
@@ -704,9 +736,10 @@ class RenderSession:
         assert self.lst is not None, "This shouldn't happen."
         while 1:
             item = self.lst[0]
-            if self.beforeBody and item == FORGET_IT:
+            if self.beforeBody and FORGET_IT in self.lst:
                 # If I haven't moved yet, and the widget wants to take
                 # over the page, let it do so!
+                self.forgotten = 1
                 return
 
             if isinstance(item, types.StringType):
@@ -716,7 +749,7 @@ class RenderSession:
                 if isinstance(item[0], self.Sentinel):
                     return
             elif isinstance(item, failure.Failure):
-                self.request.write(formatFailure(item))
+                self.request.write(webutil.formatFailure(item))
             else:
                 self.beforeBody = 0
                 unknown = html.PRE(repr(item))
@@ -733,6 +766,7 @@ class RenderSession:
 class WidgetResource(resource.Resource):
     def __init__(self, widget):
         self.widget = widget
+        resource.Resource.__init__(self)
 
     def render(self, request):
         RenderSession(self.widget.display(request), request)
@@ -757,41 +791,42 @@ class WidgetPage(Page):
     Widget wrapped up in a simple HTML template.
     """
     stylesheet = '''
-    A
+    a
     {
-        font-family: Lucida, Verdana, Helvetica, Arial;
-        color: #336699;
+        font-family: Lucida, Verdana, Helvetica, Arial, sans-serif;
+        color: #369;
         text-decoration: none;
     }
 
-    TH
+    th
     {
-        font-family: Lucida, Verdana, Helvetica, Arial;
+        font-family: Lucida, Verdana, Helvetica, Arial, sans-serif;
         font-weight: bold;
         text-decoration: none;
+        text-align: left;
     }
 
-    PRE, CODE
+    pre, code
     {
-        font-family: Courier New, Courier;
+        font-family: "Courier New", Courier, monospace;
     }
 
-    P, BODY, TD, OL, UL, MENU, BLOCKQUOTE, DIV
+    p, body, td, ol, ul, menu, blockquote, div
     {
-        font-family: Lucida, Verdana, Helvetica, Arial;
-        color: #000000;
+        font-family: Lucida, Verdana, Helvetica, Arial, sans-serif;
+        color: #000;
     }
     '''
 
-    template = '''
-    <html>
+    template = '''<html>
+    <head>
+    <title>%%%%self.title%%%%</title>
     <style>
     %%%%self.stylesheet%%%%
     </style>
-    <head>
-    <title>%%%%self.title%%%%</title>
     <base href="%%%%request.prePathURL()%%%%">
     </head>
+
     <body>
     <h1>%%%%self.title%%%%</h1>
     %%%%self.widget%%%%
@@ -821,7 +856,6 @@ class Gadget(resource.Resource):
     self.pageFactory should be a Resource that takes a Widget in its
     constructor. The default is twisted.web.widgets.WidgetPage.
     """
-    page = WidgetPage
 
     isLeaf = 0
 
@@ -834,8 +868,10 @@ class Gadget(resource.Resource):
 
     def render(self, request):
         #Redirect to view this entity as a collection.
-        request.setResponseCode(http.MOVED_PERMANENTLY)
-        request.setHeader("location","http://%s%s/" % (
+        request.setResponseCode(http.FOUND)
+        # TODO who says it's not https?
+        request.setHeader("location","http%s://%s%s/" % (
+            request.isSecure() and 's' or '',
             request.getHeader("host"),
             (string.split(request.uri,'?')[0])))
         return "NO DICE!"
@@ -855,8 +891,8 @@ class Gadget(resource.Resource):
         Add a static path to this Gadget. This method is obsolete, use
         Gadget.putPath instead.
         """
-        
-        print "Gadget.addFile() is deprecated."
+
+        log.msg("Gadget.addFile() is deprecated.")
         self.paths[path] = path
 
     def putPath(self, path, pathname):
@@ -877,9 +913,12 @@ class Gadget(resource.Resource):
         is only for backwards-compatibility -- you should set the 'pageFactory'
         attribute on your Gadget inside of its __init__ method.
         """
+        #XXX: delete this after a while.
         if hasattr(self, "page"):
-            print "Gadget.page is deprecated, use Gadget.pageFactory instead"
+            log.msg("Gadget.page is deprecated, use Gadget.pageFactory instead")
             return apply(self.page, args, kwargs)
+        else:
+            return apply(WidgetPage, args, kwargs)
 
     def getChild(self, path, request):
         if path == '':
@@ -899,11 +938,11 @@ class Gadget(resource.Resource):
             if prefix:
                 prefix = os.path.abspath(os.path.dirname(prefix))
             return static.File(os.path.join(prefix, self.paths[path]))
-        
+
         elif path == '__reload__':
             return self.pageFactory(Reloader(map(reflect.namedModule, [self.__module__] + self.modules)))
         else:
-            return error.NoResource()
+            return error.NoResource("No such child resource in gadget.")
 
 
 class TitleBox(Presentation):
@@ -945,13 +984,12 @@ class Reloader(Presentation):
         self.modules = modules
 
     def reload(self, request):
-        request.setHeader("location", "..")
-        request.setResponseCode(http.MOVED_PERMANENTLY)
+        request.redirect("..")
         x = []
         write = x.append
         for module in self.modules:
             rebuild.rebuild(module)
-            write('<li>reloaded %s<br>' % module.__name__)
+            write('<li>reloaded %s<br />' % module.__name__)
         return x
 
 class Sidebar(StreamWidget):
@@ -979,7 +1017,7 @@ class Sidebar(StreamWidget):
         return self.bar
 
     def stream(self, write, request):
-        write("<TABLE width=120 cellspacing=1 cellpadding=1 border=0>")
+        write("<table width=120 cellspacing=1 cellpadding=1 border=0>")
         for each in self.getList():
             if each[0] == self.highlightHeading:
                 headingColor = self.activeHeadingColor
@@ -989,7 +1027,8 @@ class Sidebar(StreamWidget):
                 headingColor = self.headingColor
                 headingTextColor = self.headingTextColor
                 canHighlight = 0
-            write('<TR><TD colspan=2 bgcolor="#%s"><FONT COLOR="%s"><b>%s</b>'
+            write('<tr><td colspan=2 bgcolor="#%s"><font color="%s">'
+                  '<strong>%s</strong>'
                   '</font></td></td></tr>\n' % (headingColor, headingTextColor, each[0]))
             for name, link in each[1:]:
                 if canHighlight and (name == self.highlightSection):
@@ -998,10 +1037,33 @@ class Sidebar(StreamWidget):
                 else:
                     sectionColor = self.sectionColor
                     sectionTextColor = self.sectionTextColor
-                write('<TR><td align=right bgcolor="#%s" width=6>-</td>'
-                      '<TD bgcolor="#%s"><A HREF="%s"><FONT COLOR="#%s">%s'
+                write('<tr><td align=right bgcolor="#%s" width=6>-</td>'
+                      '<td bgcolor="#%s"><a href="%s"><font color="#%s">%s'
                       '</font></a></td></tr>'
                        % (sectionColor, sectionColor, request.sibLink(link), sectionTextColor, name))
         write("</table>")
 
+# moved from template.py
+from twisted.web.woven import template
+from twisted.python import components
+
+class WebWidgetNodeMutator(template.NodeMutator):
+    """A WebWidgetNodeMutator replaces the node that is passed in to generate
+    with the result of generating the twisted.web.widget instance it adapts.
+    """
+    def generate(self, request, node):
+        widget = self.data
+        displayed = widget.display(request)
+        try:
+            html = string.join(displayed)
+        except:
+            pr = Presentation()
+            pr.tmpl = displayed
+            #strList = pr.display(request)
+            html = string.join(displayed)
+        stringMutator = template.StringNodeMutator(html)
+        return stringMutator.generate(request, node)
+
+components.registerAdapter(WebWidgetNodeMutator, Widget, template.INodeMutator)
+
 import static
