@@ -122,16 +122,30 @@ class FormFillerWidget(widgets.Widget):
                              value=str(self.getValue(request, model)))
 
     def input_submit(self, request, content, model, templateAttributes={}):
+        arguments = {}
+        val = model.getHint("onClick", templateAttributes.get("onClick", None))
+        if val:
+            arguments["onClick"] = val
+        arguments["type"] = "submit"
+        arguments["name"] = model.name
         div = content.div()
         for tag, value, desc in model.choices:
-            div.input(type="submit", name=model.name, value=tag)
+            args = arguments.copy()
+            args["value"] = tag
+            div.input(**args)
             div.text(" ")
         if model.reset:
             div.input(type="reset")
         return div
 
     def input_choice(self, request, content, model, templateAttributes={}):
-        s = content.select(name=model.name)
+        # am I not evil? allow onChange js events
+        arguments = {}
+        val = model.getHint("onChange", templateAttributes.get("onChange", None))
+        if val:
+            arguments["onChange"] = val
+        arguments["name"] = model.name
+        s = content.select(**arguments)
         default = self.getValues(request, model)
         for tag, value, desc in model.choices:
             kw = {}
@@ -249,6 +263,8 @@ class FormFillerWidget(widgets.Widget):
     def input_verifiedpassword(self, request, content, model, templateAttributes={}):
         breakLines = model.getHint('breaklines', 1)
         values = self.getValues(request, model)
+        if isinstance(values, (str, unicode)):
+            values = (values, values)
         if not values:
             p1, p2 = "", ""
         elif len(values) == 1:
