@@ -15,6 +15,11 @@
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+"""IRC interface to L{twisted.words}.
+
+I implement an IRC server so you can connect to a L{twisted.words}
+service with a regular IRC client.
+"""
 
 # XXX FIXME This will need to be fixed to work with multi-server twisted.words
 # and database access
@@ -25,16 +30,16 @@ import string
 import time
 
 # Twisted Imports
-from twisted.protocols import irc, protocol
+from twisted.protocols import irc
+from twisted.internet import protocol
 from twisted.spread import pb
 from twisted.python import log
 from twisted import copyright
-from twisted.manhole import coil
 
 # sibling imports
 import service
 
-class IRCChatter(irc.IRC, service.WordsClientInterface):
+class IRCChatter(irc.IRC, service.WordsClient):
     nickname = '*'
     passwd = None
     participant = None
@@ -82,7 +87,7 @@ class IRCChatter(irc.IRC, service.WordsClientInterface):
         #>> :glyph_!glyph@adsl-64-123-27-108.dsl.austtx.swbell.net PRIVMSG glyph_ :hello
         #>> :glyph!glyph@adsl-64-123-27-108.dsl.austtx.swbell.net PRIVMSG glyph_ :hello
 
-        if metadata is not None:
+        if metadata is not None and metadata.has_key('style'):
             message = irc.ctcpStringify([(wordsToCtcp(metadata['style']),
                                           message)])
 
@@ -93,7 +98,7 @@ class IRCChatter(irc.IRC, service.WordsClientInterface):
 
 
     def receiveGroupMessage(self, sender, group, message, metadata=None):
-        if metadata is not None:
+        if metadata is not None and metadata.has_key('style'):
             message = irc.ctcpStringify([(wordsToCtcp(metadata['style']),
                                           message)])
 
@@ -113,7 +118,7 @@ class IRCChatter(irc.IRC, service.WordsClientInterface):
                                 prefix="%s@%s!%s" %
                                 (sender, sender, self.servicename))
 
-    def connectionLost(self):
+    def connectionLost(self, reason):
         log.msg( "%s lost connection" % self.nickname )
         if self.participant:
             self.participant.detached(self, self.identity)
@@ -207,21 +212,27 @@ class IRCChatter(irc.IRC, service.WordsClientInterface):
         self.pendingPassword = password
         req = participant.getIdentityRequest()
         req.addCallbacks(self.loggedInAs, self.notLoggedIn)
-        req.arm()
 
     def loggedInAs(self, ident):
         """Successfully logged in.
         """
-        if ident.verifyPlainPassword(self.pendingPassword):
-            self.identity = ident
-            self.pendingLogin.attached(self, self.identity)
-            self.participant = self.pendingLogin
-            self.sendMessage('NOTICE', ":Authentication accepted.  "
-                             "Thank you.", prefix='NickServ!services@%s'
-                             % (self.servicename,))
-            self.sendMessage('376', ':No /MOTD.',prefix=self.servicename)
-        else:
-            self.notLoggedIn("unauthorized")
+        pwrq = ident.verifyPlainPassword(self.pendingPassword)
+        pwrq.addCallback(self.successfulLogin, ident)
+        pwrq.addErrback(self.failedLogin)
+
+    def successfulLogin(self, msg, ident):
+        self.identity = ident
+        self.pendingLogin.attached(self, self.identity)
+        self.participant = self.pendingLogin
+        self.sendMessage('NOTICE', ":Authentication accepted.  "
+                         "Thank you.", prefix='NickServ!services@%s'
+                         % (self.servicename,))
+        self.sendMessage('376', ':No /MOTD.',prefix=self.servicename)
+        del self.pendingLogin
+        del self.pendingPassword
+
+    def failedLogin(self, error):
+        self.notLoggedIn(error)
         del self.pendingLogin
         del self.pendingPassword
 
@@ -257,7 +268,7 @@ class IRCChatter(irc.IRC, service.WordsClientInterface):
         """User mode message
 
         Parameters: <nickname>
-            *( ( "+" / "-" ) *( "i" / "w" / "o" / "O" / "r" ) )
+        *( ( "+" / "-" ) *( "i" / "w" / "o" / "O" / "r" ) )
 
 
         [REQUIRED]
@@ -396,12 +407,27 @@ class IRCChatter(irc.IRC, service.WordsClientInterface):
         channame = params[-1][1:]
         group = self.service.groups.get(channame)
         if group:
-            # XXX - need to make sure this doesn't exceed the 512
-            # character limit.  If so, send it in multiple lines.
-            self.sendMessage(irc.RPL_NAMREPLY, "=", "#" + channame,
-                          ":" +
-                          string.join(map(lambda member: member.name,
-                                          group.members)))
+            # :servicename XYZ theirNick = #channel :names\r\n
+            prefixLength = len(self.servicename) + len(self.nickname) + 11
+            namesLength = 512 - prefixLength
+            names = [member.name for member in group.members]
+            lastName = 0
+            curLength = 0
+            for i, L in zip(range(len(names)), map(len, names)):
+                if curLength + L > namesLength:
+                    self.sendMessage(
+                        irc.RPL_NAMREPLY, "=", "#" + channame,
+                        ":" + ' '.join(names[lastName:i])
+                    )
+                    lastName = i
+                    curLength = 0
+                else:
+                    curLength = curLength + L
+            if curLength:
+                self.sendMessage(
+                    irc.RPL_NAMREPLY, "=", "#" + channame,
+                    ":" + ' '.join(names[lastName:])
+                )
         self.sendMessage(irc.RPL_ENDOFNAMES,
                          "#" + channame, ":End of /NAMES list.")
 
@@ -483,7 +509,7 @@ class IRCChatter(irc.IRC, service.WordsClientInterface):
                         msgMethod(name_, data, metadata)
                     except pb.Error, e:
                         self.receiveDirectMessage("*error*", str(e))
-                        print 'error sending CTCP query:',str(e)
+                        log.msg('error sending CTCP query:',str(e))
 
                 if not m['normal']:
                     return
@@ -491,7 +517,7 @@ class IRCChatter(irc.IRC, service.WordsClientInterface):
                 text = string.join(m['normal'], ' ')
 
             ## 'bot' handling
-            if name == 'ContactServ':
+            if string.lower(name) == 'contactserv':
                 # crude contacts interface
                 cmds = string.split(text, ' ', 1)
                 if cmds[0] == 'add':
@@ -507,7 +533,7 @@ class IRCChatter(irc.IRC, service.WordsClientInterface):
                     self.participant.groupMessage(channame, text)
                 except pb.Error, e:
                     self.receiveDirectMessage("*error*", str(e))
-                    print 'error chatting to channel:',str(e)
+                    log.msg('error chatting to channel:',str(e))
             else:
                 try:
                     self.participant.directMessage(name, text)
@@ -518,7 +544,7 @@ class IRCChatter(irc.IRC, service.WordsClientInterface):
                                      ":%s" % (e,))
         else:
 
-            if name == 'NickServ':
+            if string.lower(name) == 'nickserv':
                 self.logInAs(self.pendingLogin, text)
             else:
                 self.receiveDirectMessage("NickServ", "You haven't logged in yet.")
@@ -912,10 +938,11 @@ class IRCChatter(irc.IRC, service.WordsClientInterface):
             # self.sendMessage(irc.RPL_ISON, ":" + string.join(ison))
             pass
 
-class IRCGateway(protocol.Factory, coil.Configurable):
-    configCreatable = 0
+class IRCGateway(protocol.Factory):
+
     def __init__(self, service):
         self.service = service
+
     def buildProtocol(self, connection):
         """Build an IRC protocol to talk to my chat service.
         """
@@ -936,5 +963,3 @@ def ctcpToWords(query_tag):
 
 def wordsToCtcp(style):
     return mapWordsToCtcp.get(style, style)
-
-coil.registerClass(IRCGateway)
