@@ -1,3 +1,4 @@
+# -*- test-case-name: twisted.test.test_words -*-
 
 # Twisted, the Framework of Your Internet
 # Copyright (C) 2001 Matthew W. Lefkowitz
@@ -16,14 +17,90 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+"""
+
+Twisted Words Service objects.  Chat and messaging for Twisted.
+
+Twisted words is a general-purpose chat and instant messaging system designed
+to be a suitable replacement both for Instant Messenger systems and
+conferencing systems like IRC.
+
+Currently it provides presence notification, web-based account creation, and a
+simple group-chat abstraction.
+
+Stability: incendiary
+
+Maintainer: Maintainer: U{Glyph Lefkowitz<mailto:glyph@twistedmatrix.com>}
+
+Future Plans: Woah boy.  This module is incredibly unstable.  It has an
+incredible deficiency of features.  There are also several features which are
+pretty controvertial.  As far as stability goes, it is lucky that the current
+interfaces are really simple: at least the uppermost external ones will almost
+certainly be preserved, but there is a lot of plumbing work.
+
+First of all the fact that users must have accounts generated through a web
+interface to sign in is a serious annoyance, especially to people who are
+familiar with IRC's semantics.  The following features are proposed to
+mitigate this annoyance:
+
+  - account creation through the various client interfaces available to Words
+    users.
+
+  - guest accounts, so that users who join for an hour once don't pollute the
+    authentication database with huge amounts of cruft.
+
+  - 'mood' metadata for users.  Since you can't change nicks, you need a way to
+    do the equivalent thing on IRC where people will sign in multiple times and
+    have foo_work and foo_home
+
+There is no plan to make it possible to log-in without an account.  This is
+simply a broken behavior of IRC; all possible convenience features that mimic
+this should be integrated, but authentication is an important part of chat.
+
+There are also certain things that are just missing.
+
+  - restricted group operations.  Typical IRC-style stuff, except you don't
+    ever see the @.  Permimssions should be grantable in a capability style,
+    rather than with a single bit.
+
+  - server-to-server communication.  As much as possible this should be
+    decentralized and not have the notion of 'hub' servers; rooms have
+    'physical' locality.  This is really hard to integrate with IRC client
+    protocol stuff, so it may end up that this feature requires a rewrite of
+    Twisted Words so that servers that present an IRC gateway are treated as
+    leaf nodes, and the recommended mode of operation is for the user to run a
+    lightweight proxy locally.
+
+  - a serious logging, monitoring, and routing framework
+
+Then there's a whole bunch of things that would be nice to have.
+
+  - public key authentication
+
+  - robust wire-level security
+
+  - integrated consensus web authoring tools
+
+  - management tools and guidelines for community leaders
+
+  - interface to operator functionality through 'bot' interface with
+    per-channel personality configuration
+
+  - graphical extensions to clients to allow formatted text (but detect
+    obviously annoying or abusive formatting)
+
+  - rate limiting, simple DoS protection, firewall integration
+
+  - basically everything OPN wants to be able to do, but better
+
+"""
+
 # System Imports
-import string
-import types
+import types, time
 
 # Twisted Imports
 from twisted.spread import pb
-from twisted.python import log, roots
-from twisted.manhole import coil
+from twisted.python import log, roots, components
 from twisted.persisted import styles
 from twisted import copyright
 from twisted.cred import authorizer
@@ -36,7 +113,7 @@ AWAY = 2
 
 statuses = ["Offline","Online","Away"]
 
-class WordsError(pb.Error):
+class WordsError(pb.Error, KeyError):
     pass
 
 class NotInCollectionError(WordsError):
@@ -84,7 +161,7 @@ class WrongStatusError(WordsError):
         return s
 
 
-class WordsClientInterface:
+class IWordsClient(components.Interface):
     """A client to a perspective on the twisted.words service.
 
     I attach to that participant with Participant.attached(),
@@ -134,6 +211,51 @@ class WordsClientInterface:
         """Tells me a member has left a group.
         """
 
+class WordsClient:
+    __implements__ = IWordsClient
+    """A stubbed version of L{IWordsClient}.
+
+    Useful for partial implementations.
+    """
+
+    def receiveContactList(self, contactList): pass
+    def notifyStatusChanged(self, name, status): pass
+    def receiveGroupMembers(self, names, group): pass
+    def setGroupMetadata(self, metadata, name): pass
+    def receiveDirectMessage(self, sender, message, metadata=None): pass
+    def receiveGroupMessage(self, sender, group, message, metadata=None): pass
+    def memberJoined(self, member, group): pass
+    def memberLeft(self, member, group): pass
+
+
+class Transcript:
+    """I am a transcript of a conversation between multiple parties.
+    """
+    def __init__(self, voice, name):
+        self.chat = []
+        self.voice = voice
+        self.name = name
+    def logMessage(self, voiceName, message, metadata):
+        self.chat.append((time.time(), voiceName, message, metadata))
+    def endTranscript(self):
+        self.voice.stopTranscribing(self.name)
+
+class IWordsPolicy(components.Interface):
+    def getNameFor(self, participant):
+        """Give a name for a participant, based on the current policy."""
+    def lookUpParticipant(self, nick):
+        """ Get a Participant, given a name."""
+
+class NormalPolicy:
+    __implements__ = IWordsPolicy
+
+    def __init__(self, participant):
+        self.participant = participant
+    def getNameFor(self, participant):
+        return participant.name
+
+    def lookUpParticipant(self, nick):
+        return self.participant.service.getPerspectiveNamed(nick)
 
 class Participant(pb.Perspective, styles.Versioned):
     def __init__(self, name):
@@ -144,25 +266,30 @@ class Participant(pb.Perspective, styles.Versioned):
         self.reverseContacts = []
         self.groups = []
         self.client = None
-        self.info = ""
+        self.loggedNames = {}
 
-    persistenceVersion = 1
+        self.policy = NormalPolicy(self)
+    persistenceVersion = 2
+
+    def upgradeToVersion2(self):
+        self.loggedNames = {}
 
     def __getstate__(self):
         state = styles.Versioned.__getstate__(self)
         # Assumptions:
         # * self.client is a RemoteReference, or otherwise represents
         #   a transient presence.
-        state["client"] = None
-        # * Because we have no client, we are not online.
-        state["status"] = OFFLINE
-        # * Because we are not online, we are in no groups.
-        state["groups"] = []
+        if isinstance(state["client"], styles.Ephemeral):
+            state["client"] = None
+            # * Because we have no client, we are not online.
+            state["status"] = OFFLINE
+            # * Because we are not online, we are in no groups.
+            state["groups"] = []
 
         return state
 
     def attached(self, client, identity):
-        """Attach a client which implements WordsClientInterface to me.
+        """Attach a client which implements L{IWordsClient} to me.
         """
         if ((self.client is not None)
             and self.client.__class__ != styles.Ephemeral):
@@ -175,6 +302,14 @@ class Participant(pb.Perspective, styles.Versioned):
         self.changeStatus(ONLINE)
         return self
 
+    def transcribeConversationWith(self, voiceName):
+        t  = Transcript(self, voiceName)
+        self.loggedNames[voiceName] = t
+        return t
+
+    def stopTranscribing(self, voiceName):
+        del self.loggedNames[voiceName]
+
     def changeStatus(self, newStatus):
         self.status = newStatus
         for contact in self.reverseContacts:
@@ -228,62 +363,55 @@ class Participant(pb.Perspective, styles.Versioned):
         raise NotInGroupError(name)
 
     def getGroupMembers(self, groupName):
-        for group in self.groups:
-            if group.name == groupName:
-                self.client.callRemote('receiveGroupMembers', map(lambda m: m.name,
-                                                                  group.members),
-                                       group.name)
-        raise NotInGroupError(groupName)
+        if self.client:
+            for group in self.groups:
+                if group.name == groupName:
+                    self.client.callRemote('receiveGroupMembers',
+                                           map(lambda m: m.name,
+                                               group.members),
+                                           group.name)
+                    return
+            raise NotInGroupError(groupName)
 
     def getGroupMetadata(self, groupName):
-        for group in self.groups:
-            if group.name == groupName:
-                self.client.callRemote('setGroupMetadata', group.metadata, group.name)
+        if self.client:
+            for group in self.groups:
+                if group.name == groupName:
+                    self.client.callRemote('setGroupMetadata', group.metadata, group.name)
 
     def receiveDirectMessage(self, sender, message, metadata):
         if self.client:
-            if metadata:
-                d = self.client.callRemote('receiveDirectMessage', sender.name, message,
-                                           metadata)
-                #If the client doesn't support metadata, call this function
-                #again with no metadata, so none is sent
-                #
-                #note on the 'if d:' - the IRC service is in-process and
-                #won't return a Deferred, but rather it returns None.
-                #silently ignore it. (This is really evil and terrible.)
-                if d:
-                    d.addErrback(self.receiveDirectMessage,
-                                 sender.name, message, None)
-            else:
-                self.client.callRemote('receiveDirectMessage', sender.name, message)
+            # is this wrong?
+            # nick = self.policy.getNameFor(sender)
+            nick = sender.name
+            if self.loggedNames.has_key(nick):
+                self.loggedNames[nick].logMessage(sender.name, message,
+                                                         metadata)
+            self.client.callRemote('receiveDirectMessage', nick,
+                                   message, metadata)
         else:
             raise WrongStatusError(self.status, self.name)
 
 
     def receiveGroupMessage(self, sender, group, message, metadata):
         if sender is not self and self.client:
-            if metadata:
-                d = self.client.receiveGroupMessage(sender.name, group.name,
-                                                    message, metadata)
-                if d:
-                    d.addErrback(self.receiveGroupMessage,
-                                 sender, group, message, None)
-            else:
-                self.client.receiveGroupMessage(sender.name, group.name,
-                                                message)
+            self.client.callRemote('receiveGroupMessage',sender.name, group.name,
+                                   message, metadata)
 
     def memberJoined(self, member, group):
-        self.client.memberJoined(member.name, group.name)
+        if self.client:
+            self.client.callRemote('memberJoined', member.name, group.name)
 
     def memberLeft(self, member, group):
-        self.client.memberLeft(member.name, group.name)
+        if self.client:
+            self.client.callRemote('memberLeft', member.name, group.name)
 
     def directMessage(self, recipientName, message, metadata=None):
-        # XXX getPerspectiveNamed is misleading here -- this ought to look up
-        # the user to make sure they're *online*, and if they're not, it may
-        # need to query a database.
-        recipient = self.service.getPerspectiveNamed(recipientName)
+        recipient = self.policy.lookUpParticipant(recipientName)
         recipient.receiveDirectMessage(self, message, metadata or {})
+        if self.loggedNames.has_key(recipientName):
+            self.loggedNames[recipientName].logMessage(self.name, message, metadata)
+
 
     def groupMessage(self, groupName, message, metadata=None):
         for group in self.groups:
@@ -327,7 +455,21 @@ class Participant(pb.Perspective, styles.Versioned):
         return s
 
 class Group(styles.Versioned):
+    """
+    This class represents a group of people engaged in a chat session
+    with one another.
 
+    @type name:            C{string}
+    @ivar name:            The name of the group
+    @type members:         C{list}
+    @ivar members:         The members of the group
+    @type metadata:        C{dictionary}
+    @ivar metadata:        Metadata that describes the group.  Common
+                           keys are:
+                             - C{'topic'}: The topic string for the group.
+                             - C{'topic_author'}: The name of the user who
+                               last set the topic.
+    """
     def __init__(self, name):
         self.name = name
         self.members = []
@@ -379,56 +521,32 @@ class Group(styles.Versioned):
         del self.topic
         self.metadata['topic_author'] = 'admin'
 
-class Service(pb.Service, styles.Versioned, coil.Configurable):
+
+class Service(pb.Service, styles.Versioned):
     """I am a chat service.
     """
-    def __init__(self, name, app):
-        pb.Service.__init__(self, name, app)
-        self.participants = {}
+
+    perspectiveClass = Participant
+
+    def __init__(self, name, parent=None, auth=None):
+        pb.Service.__init__(self, name, parent, auth)
         self.groups = {}
-        self._setConfigDispensers()
-
-    # Configuration stuff.
-    def _setConfigDispensers(self):
-        import ircservice, webwords
-        self.configDispensers = [
-            ['makeIRCGateway', ircservice.IRCGateway, "IRC chat gateway to %s" % self.serviceName],
-            ['makeWebAccounts', webwords.WordsGadget, "Public Words Website for %s" % self.serviceName]
-            ]
-
-    def makeWebAccounts(self):
-        import webwords
-        return webwords.WordsGadget(self)
-
-    def makeIRCGateway(self):
-        import ircservice
-        return ircservice.IRCGateway(self)
-
-    def configInit(self, container, name):
-        self.__init__(name, container.app)
-
-    def getConfiguration(self):
-        return {"name": self.serviceName}
-
-    configTypes = {
-        'name': types.StringType
-        }
-
-    configName = 'Twisted Words PB Service'
-
-    def config_name(self, name):
-        raise coil.InvalidConfiguration("You can't change a Service's name.")
+        self.bots = []
 
     ## Persistence versioning.
-    persistenceVersion = 2
+    persistenceVersion = 4
 
     def upgradeToVersion1(self):
         from twisted.internet.app import theApplication
         styles.requireUpgrade(theApplication)
         pb.Service.__init__(self, 'twisted.words', theApplication)
 
-    def upgradeToVersion2(self):
-        self._setConfigDispensers()
+    def upgradeToVersion3(self):
+        self.perspectives = self.participants
+        del self.participants
+
+    def upgradeToVersion4(self):
+        self.bots = []
 
     ## Service functionality.
 
@@ -439,26 +557,38 @@ class Service(pb.Service, styles.Versioned, coil.Configurable):
             self.groups[name] = group
         return group
 
-    def createParticipant(self, name):
-        if not self.participants.has_key(name):
-            log.msg("Created New Participant: %s" % name)
-            p = Participant(name)
-            p.setService(self)
-            self.participants[name] = p
-            return p
+    def createPerspective(self, name):
+        if self.perspectives.has_key(name):
+            raise KeyError("Participant already exists: %s." % name)
+        log.msg("Creating New Participant: %s" % name)
+        return pb.Service.createPerspective(self, name)
 
     def getPerspectiveNamed(self, name):
         try:
-            p = self.participants[name]
+            return pb.Service.getPerspectiveNamed(self, name)
         except KeyError:
             raise UserNonexistantError(name)
-        else:
-            return p
+
+    def addBot(self, name, bot):
+        try:
+            p = self.getPerspectiveNamed(name)
+        except UserNonexistantError:
+            p = self.createPerspective(name)
+
+        bot.setupBot(p) # XXX this method needs a better name
+        from twisted.spread.util import LocalAsyncForwarder
+        p.attached(LocalAsyncForwarder(bot, IWordsClient, 1), None)
+        self.bots.append(bot)
+
+    def deleteBot(self, bot):
+        bot.voice.detached(bot, None)
+        self.bots.remove(bot)
+        del self.perspectives[bot.voice.perspectiveName]
+
+    createParticipant = createPerspective
 
     def __str__(self):
         s = "<%s in app '%s' at %x>" % (self.serviceName,
                                         self.application.name,
                                         id(self))
         return s
-
-coil.registerClass(Service)
