@@ -1,4 +1,5 @@
-
+# -*- test-case-name: twisted.test.test_tendril -*-
+# $Id: tendril.py,v 1.32 2003/01/08 10:34:29 acapnotic Exp $
 # Twisted, the Framework of Your Internet
 # Copyright (C) 2001 Matthew W. Lefkowitz
 #
@@ -15,26 +16,174 @@
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+"""Tendril between Words and IRC servers.
+
+A Tendril, attached to a Words service, signs on as a user to an IRC
+server.  It can then relay traffic for one or more channels/groups
+between the two servers.  Anything it hears on a Words group it will
+repeat as a user in an IRC channel; anyone it hears on IRC will appear
+to be logged in to the Words service and speaking in a group there.
+
+How to Start a Tendril
+======================
+
+In manhole::
+
+  from twisted.internet import reactor as R
+  from twisted.internet.app import theApplication as A
+  from twisted.words import tendril as T
+
+  w = A.getServiceNamed('twisted.words')
+  f = T.TendrilFactory(w)
+  # Maybe do some customization of f here, i.e.
+  ## f.nickname = 'PartyLink'
+  ## f.groupList = ['this', 'that', 'other']
+  R.connectTCP(irchost, 6667, f)
+
+Stability: No more stable than L{words<twisted.words.service>}.
+
+Future plans: Use \"L{Policy<twisted.words.service.Policy>}\" to get
+Perspectives.
+
+@author: U{Kevin Turner<acapnotic@twistedmatrix.com>}
+"""
+
+from twisted import copyright
+from twisted.cred import authorizer, error
+from twisted.internet import defer, protocol
 from twisted.persisted import styles
 from twisted.protocols import irc
-from twisted.python import log
+from twisted.python import log, reflect
 from twisted.words import service
-from twisted import copyright
-from twisted.cred import authorizer
+from twisted.spread.util import LocalAsyncForwarder
 
 wordsService = service
 del service
 
 import string
-import sys
 import traceback
+import types
 
-_LOGALL = 0
+True = (1==1)
+False = not True
+
+_LOGALL = False
 
 # XXX FIXME -- This will need to be fixed to work asynchronously in order to
 # support multiple-server twisted.words and database access to accounts
 
-class ProxiedParticipant(wordsService.WordsClientInterface,
+class TendrilFactory(protocol.ReconnectingClientFactory, reflect.Accessor):
+    """I build Tendril clients for a words service.
+
+    All of a tendril's configurable state is stored here with me.
+    """
+
+    wordsService = None
+    wordsclient = None
+
+    networkSuffix = None
+    nickname = None
+    perspectiveName = None
+
+    protocol = None # will be set to TendrilIRC as soon as it's defined.
+    _groupList = ['tendril_test']
+    _errorGroup = 'TendrilErrors'
+
+    helptext = (
+        "Hi, I'm a Tendril bridge between here and %(service)s.",
+        "You can send a private message to someone like this:",
+        "/msg %(myNick)s msg theirNick Hi there!",
+        )
+
+    def __init__(self, service):
+        """Initialize this factory with a words service."""
+        self.reallySet('wordsService', service)
+
+    def startFactory(self):
+        self.wordsclient = TendrilWords(
+            service=self.wordsService, ircFactory=self,
+            nickname=self.nickname, perspectiveName=self.perspectiveName,
+            networkSuffix=self.networkSuffix, groupList=self.groupList,
+            errorGroup=self.errorGroup)
+
+    def buildProtocol(self, addr):
+        if self.wordsclient.irc:
+            log.msg("Warning: building a new %s protocol while %s is still active."
+                    % (self.protocol, self.wordsclient.irc))
+
+        proto = protocol.ClientFactory.buildProtocol(self, addr)
+        self.wordsclient.setIrc(proto)
+
+        # Ermm.
+        ## self.protocol.__dict__.update(self.getConfiguration())
+        for k in ('nickname', 'helptext'):
+            setattr(proto, k, getattr(self, k))
+
+        return proto
+
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        try:
+            del state["wordsclient"]
+        except KeyError:
+            pass
+        return state
+
+    def set_wordsService(self, service):
+        raise TypeError, "%s.wordsService is a read-only attribute." % (repr(self),)
+
+    def set_groupList(self, groupList):
+        if self.wordsclient:
+            oldlist = self.wordsclient.groupList
+            if groupList != oldlist:
+                newgroups = filter(lambda g, ol=oldlist: g not in ol,
+                                   groupList)
+                deadgroups = filter(lambda o, gl=groupList: o not in gl,
+                                    oldlist)
+
+                self.wordsclient.groupList[:] = groupList
+                if self.wordsclient.irc:
+                    for group in newgroups:
+                        self.wordsclient.irc.join(groupToChannelName(group))
+                    for group in deadgroups:
+                        self.wordsclient.irc.part(groupToChannelName(group))
+        self._groupList = groupList
+
+    def get_groupList(self):
+        if self.wordsclient:
+            return self.wordsclient.groupList
+        else:
+            return self._groupList
+
+    def set_nickname(self, nick):
+        if self.wordsclient and self.wordsclient.irc:
+            self.wordsclient.irc.setNick(nick)
+        self.reallySet('nickname', nick)
+
+    def set_errorGroup(self, errorGroup):
+        if self.wordsclient:
+            oldgroup = self.wordsclient.errorGroup
+            if oldgroup != errorGroup:
+                self.wordsclient.joinGroup(errorGroup)
+                self.wordsclient.errorGroup = errorGroup
+                self.wordsclient.leaveGroup(oldgroup)
+        self._errorGroup = errorGroup
+
+    def get_errorGroup(self):
+        if self.wordsclient:
+            return self.wordsclient.errorGroup
+        else:
+            return self._errorGroup
+
+    def set_helptext(self, helptext):
+        if isinstance(helptext, types.StringType):
+            helptext = string.split(helptext, '\n')
+        if self.wordsclient and self.wordsclient.irc:
+            self.wordsclient.irc.helptext = helptext
+        self.reallySet('helptext', helptext)
+
+
+class ProxiedParticipant(wordsService.WordsClient,
                          styles.Ephemeral):
     """I'm the client of a participant who is connected through Tendril.
     """
@@ -56,138 +205,43 @@ class ProxiedParticipant(wordsService.WordsClientInterface,
                                   sender, message, metadata)
 
 
-class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
+class TendrilIRC(irc.IRCClient, styles.Ephemeral):
     """I connect to the IRC server and broker traffic.
     """
 
-    networkSuffix = '@opn'
-    nickname = 'tl'
-    groupList = ['tendril_test']
-
-    participants = None
-
-    errorGroup = 'TendrilErrors'
-    perspectiveName = nickname + networkSuffix
-
     realname = 'Tendril'
     versionName = 'Tendril'
-    versionNum = '$Revision: 1.14 $'[11:-2]
+    versionNum = '$Revision: 1.32 $'[11:-2]
     versionEnv = copyright.longversion
 
-    helptext = (
-        "Hi, I'm a Tendril bridge between here and %(service)s.",
-        "You can send a private message to someone like this:",
-        "/msg %(myNick)s msg theirNick Hi there!",
-        )
+    helptext = TendrilFactory.helptext
 
-    def __init__(self, service, groupList=None,
-                 nickname=None, networkSuffix=None, perspectiveName=None):
-        """Create a new Tendril client.
+    words = None
 
-        service -- a twisted.words.service.Service, or at least
-        something with a 'serviceName' attribute and 'createParticipant'
-        and 'getPerspectiveNamed' methods which work like a
-        words..Service.
-
-        groupList -- a list of strings naming groups on the Words
-        service to join and bridge to their counterparts on the IRC
-        server.
-
-        nickname -- a string to use as my nickname on the IRC network.
-
-        networkSuffix -- a string to append to the nickname of the
-        Participants I bring in through IRC, e.g. \"@opn\".
-
-        perspectiveName -- the name of my perspective with this
-        service.  Defaults to nickname + networkSuffix.
-
-        To connect me to an IRC server, pass me as the 'protocol' when
-        constructing a tcp.Client.
-        """
-        self.participants = {}
+    def __init__(self):
+        """Create a new Tendril IRC client."""
         self.dcc_sessions = {}
 
-        if nickname:
-            self.nickname = nickname
-        if networkSuffix:
-            self.networkSuffix = networkSuffix
-
-        if not perspectiveName:
-            perspectiveName = self.nickname + self.networkSuffix
-
-        self.perspectiveName = perspectiveName
-
-        if groupList:
-            self.groupList = list(groupList)
-        else:
-            self.groupList = list(TendrilClient.groupList)
-
-        self.service = service
-        self.realname = 'Tendril to %s' % (service.serviceName,)
-
-        self.attachToWords()
-
-        self.joinGroup(self.errorGroup)
-
-
-    def attachToWords(self):
-        """Get my perspective on the Words service; attach as a client.
-        """
-
-        try:
-            self.perspective = (
-                self.service.getPerspectiveNamed(self.perspectiveName))
-        except wordsService.UserNonexistantError:
-            self.perspective = (
-                self.service.createParticipant(self.perspectiveName))
-            if not self.perspective:
-                raise RuntimeError, ("service %s won't give me my "
-                                     "perspective named %s"
-                                     % (self.service,
-                                        self.perspectiveName))
-
-        if self.perspective.client is self:
-            log.msg("I seem to be already attached.")
-            return
-
-        try:
-            self.attach()
-        except authorizer.Unauthorized:
-            if self.perspective.client:
-                log.msg("%s is attached to my perspective: "
-                        "kicking it off." % (self.perspective.client,))
-                self.perspective.detached(self.perspective.client, None)
-                self.attach()
-            else:
-                raise
-
-    def __getstate__(self):
-        dct = self.__dict__.copy()
-        # Don't save my imaginary friends.
-        dct["participants"] = {}
-        return dct
-
-
     ### Protocol-level methods
 
-    def connectionLost(self):
+    def connectionLost(self, reason):
         """When I lose a connection, log out all my IRC participants.
         """
-        self.log("%s: Connection lost." % (self.transport,), 'info')
-        for nick in self.participants.keys()[:]:
-            self.logoutParticipant(nick)
+        self.log("%s: Connection lost: %s" % (self.transport, reason), 'info')
+        self.words.ircConnectionLost()
 
     ### Protocol LineReceiver-level methods
 
     def lineReceived(self, line):
-        self.log(line, 'dump')
-
         try:
             irc.IRCClient.lineReceived(self, line)
         except:
+            # If you *don't* catch exceptions here, any unhandled exception
+            # raised by anything lineReceived calls (which is most of the
+            # client code) ends up making Connection Lost happen, which
+            # is almost certainly not necessary for us.
             log.deferr()
 
-
     def sendLine(self, line):
         """Send a line through my transport, unless my transport isn't up.
         """
@@ -197,7 +251,6 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         self.log(line, 'dump')
         irc.IRCClient.sendLine(self, line)
 
-
     ### Protocol IRCClient server->client methods
 
     def irc_JOIN(self, prefix, params):
@@ -206,9 +259,9 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         nick = string.split(prefix,'!')[0]
         groupName = channelToGroupName(params[0])
         if nick == self.nickname:
-            self.joinGroup(groupName)
+            self.words.joinGroup(groupName)
         else:
-            self._getParticipant(nick).joinGroup(groupName)
+            self.words._getParticipant(nick).joinGroup(groupName)
 
     def irc_NICK(self, prefix, params):
         """When an IRC user changes their nickname
@@ -221,7 +274,7 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         if old_nick == self.nickname:
             self.nickname = new_nick
         else:
-            self.changeParticipantNick(old_nick, new_nick)
+            self.words.changeParticipantNick(old_nick, new_nick)
 
     def irc_PART(self, prefix, params):
         """Parting IRC members leave the correspoding group.
@@ -230,28 +283,21 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         channel = params[0]
         groupName = channelToGroupName(channel)
         if nick == self.nickname:
-            self.groupMessage(groupName, "I've left %s" % (channel,))
-            self.leaveGroup(groupName)
-            self.evacuateGroup(groupName)
+            self.words.groupMessage(groupName, "I've left %s" % (channel,))
+            self.words.leaveGroup(groupName)
+            self.words.evacuateGroup(groupName)
             return
-
-        participant = self._getParticipant(nick)
-        try:
-            participant.leaveGroup(groupName)
-        except wordsService.NotInGroupError:
-            pass
-
-        if not participant.groups:
-            self.logoutParticipant(nick)
+        else:
+            self.words.ircPartParticipant(nick, groupName)
 
     def irc_QUIT(self, prefix, params):
         """When a user quits IRC, log out their participant.
         """
         nick = string.split(prefix,'!')[0]
         if nick == self.nickname:
-            self.detach()
+            self.words.detach()
         else:
-            self.logoutParticipant(nick)
+            self.words.logoutParticipant(nick)
 
     def irc_KICK(self, prefix, params):
         """Kicked?  Who?  Not me, I hope.
@@ -262,8 +308,8 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         group = channelToGroupName(channel)
         if string.lower(kicked) == string.lower(self.nickname):
             # Yikes!
-            if self.participants.has_key(nick):
-                wordsname = " (%s)" % (self._getParticipant(nick).name,)
+            if self.words.participants.has_key(nick):
+                wordsname = " (%s)" % (self.words._getParticipant(nick).name,)
             else:
                 wordsname = ''
             if len(params) > 2:
@@ -271,17 +317,14 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
             else:
                 reason = ''
 
-            self.groupMessage(group, '%s%s kicked me off!%s'
+            self.words.groupMessage(group, '%s%s kicked me off!%s'
                               % (prefix, wordsname, reason))
             self.log("I've been kicked from %s: %s %s"
                      % (channel, prefix, params), 'NOTICE')
-            self.evacuateGroup(group)
+            self.words.evacuateGroup(group)
 
         else:
-            try:
-                self._getParticipant(kicked).leaveGroup(group)
-            except wordsService.NotInGroupError:
-                pass
+            self.words.ircPartParticipant(kicked, group)
 
     def irc_INVITE(self, prefix, params):
         """Accept an invitation, if it's in my groupList.
@@ -290,7 +333,7 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         if group in self.groupList:
             self.log("I'm accepting the invitation to join %s from %s."
                      % (group, prefix), 'NOTICE')
-            self.join(groupToChannelName(group))
+            self.words.join(groupToChannelName(group))
 
     def irc_TOPIC(self, prefix, params):
         """Announce the new topic.
@@ -300,10 +343,10 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         nick = string.split(prefix,'!')[0]
         channel = params[0]
         topic = params[1]
-        self.groupMessage(channelToGroupName(channel),
-                          "%s has just decreed the topic to be: %s"
-                          % (self._getParticipant(nick).name,
-                             topic))
+        self.words.groupMessage(channelToGroupName(channel),
+                                "%s has just decreed the topic to be: %s"
+                                % (self.words._getParticipant(nick).name,
+                                   topic))
 
     def irc_ERR_BANNEDFROMCHAN(self, prefix, params):
         """When I can't get on a channel, report it.
@@ -343,7 +386,7 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         self.notice(nick, "Got your DCC %s"
                     % (irc.dccDescribe(data),))
 
-        pName = self._getParticipant(nick).name
+        pName = self.words._getParticipant(nick).name
         self.dcc_sessions[pName] = (user, dcc_text, data)
 
         self.notice(nick, "If I should pass it on to another user, "
@@ -356,7 +399,8 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         """Join my groupList once I've signed on.
         """
         self.log("Welcomed by IRC server.", 'info')
-        for group in self.groupList:
+        self.factory.resetDelay()
+        for group in self.words.groupList:
             self.join(groupToChannelName(group))
 
     def privmsg(self, user, channel, message):
@@ -382,11 +426,7 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         else:
             # The message isn't to me, so it must be to a group.
             group = channelToGroupName(channel)
-            try:
-                self._getParticipant(nick).groupMessage(group, message)
-            except wordsService.NotInGroupError:
-                self._getParticipant(nick).joinGroup(group)
-                self._getParticipant(nick).groupMessage(group, message)
+            self.words.ircParticipantMsg(nick, group, message)
 
     def noticed(self, user, channel, message):
         """Pass channel notices on to the group.
@@ -403,24 +443,14 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         else:
             # The message isn't to me, so it must be to a group.
             group = channelToGroupName(channel)
-            try:
-                self._getParticipant(nick).groupMessage(group, message)
-            except wordsService.NotInGroupError:
-                self._getParticipant(nick).joinGroup(group)
-                self._getParticipant(nick).groupMessage(group, message)
+            self.words.ircParticipantMsg(nick, group, message)
 
     def action(self, user, channel, message):
         """Speak about a participant in third-person.
         """
         group = channelToGroupName(channel)
         nick = string.split(user,'!',1)[0]
-        try:
-            self._getParticipant(nick).groupMessage(group, message,
-                                                    {'style': 'emote'})
-        except wordsService.NotInGroupError:
-            self._getParticipant(nick).joinGroup(group)
-            self._getParticipant(nick).groupMessage(group, message,
-                                                    {'style': 'emote'})
+        self.words.ircParticipantMsg(nick, group, message, emote=True)
 
     ### Bot event methods
 
@@ -430,7 +460,7 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         (nick, message) = string.split(params, ' ', 1)
         sender = string.split(sender, '!', 1)[0]
         try:
-            self._getParticipant(sender).directMessage(nick, message)
+            self.words._getParticipant(sender).directMessage(nick, message)
         except wordsService.WordsError, e:
             self.notice(sender, "msg to %s failed: %s" % (nick, e))
 
@@ -439,7 +469,7 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         for l in self.helptext:
             self.notice(nick, l % {
                 'myNick': self.nickname,
-                'service': self.service.serviceName,
+                'service': self.factory.wordsService,
                 })
 
     def botUnknown(self, user, channel, message):
@@ -473,7 +503,7 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         DCC FORGET -- forget any DCC requests you offered to me.
         """
         nick = string.split(user,"!")[0]
-        pName = self._getParticipant(nick).name
+        pName = self.words._getParticipant(nick).name
 
         if not params:
             # Do I have a DCC from you?
@@ -500,8 +530,8 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
 
                 ctcpMsg = irc.ctcpStringify([('DCC',orig_data)])
                 try:
-                    self._getParticipant(nick).directMessage(dst,
-                                                            ctcpMsg)
+                    self.words._getParticipant(nick).directMessage(dst,
+                                                                   ctcpMsg)
                 except wordsService.WordsError, e:
                     self.notice(nick, "DCC offer to %s failed: %s"
                                 % (dst, e))
@@ -525,6 +555,121 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
                         "DCC, DCC PASSTO <nick>, DCC FORGET")
         return
 
+
+    ### Utility
+
+    def log(self, message, priority=None):
+        """I need to give Twisted a prioritized logging facility one of these days.
+        """
+        if _LOGALL:
+            log.msg(message)
+        elif not (priority in ('dump',)):
+            log.msg(message)
+
+        if priority in ('info', 'NOTICE', 'ERROR'):
+            self.words.groupMessage(self.words.errorGroup, message)
+
+TendrilFactory.protocol = TendrilIRC
+
+class TendrilWords(wordsService.WordsClient):
+    nickname = 'tl'
+    networkSuffix = '@opn'
+    perspectiveName = nickname + networkSuffix
+    participants = None
+    irc = None
+    ircFactory = None
+
+    def __init__(self, service, ircFactory,
+                 nickname=None, networkSuffix=None, perspectiveName=None,
+                 groupList=None, errorGroup=None):
+        """
+        service -- a twisted.words.service.Service, or at least
+        something with a 'serviceName' attribute and 'createParticipant'
+        and 'getPerspectiveNamed' methods which work like a
+        words..Service.
+
+        groupList -- a list of strings naming groups on the Words
+        service to join and bridge to their counterparts on the IRC
+        server.
+
+        nickname -- a string to use as my nickname on the IRC network.
+
+        networkSuffix -- a string to append to the nickname of the
+        Participants I bring in through IRC, e.g. \"@opn\".
+
+        perspectiveName -- the name of my perspective with this
+        service.  Defaults to nickname + networkSuffix.
+        """
+        self.service = service
+        self.ircFactory = ircFactory
+        self.participants = {}
+
+        if nickname:
+            self.nickname = nickname
+        if networkSuffix:
+            self.networkSuffix = networkSuffix
+
+        if perspectiveName:
+            self.perspectiveName = perspectiveName
+        else:
+            self.perspectiveName = self.nickname + self.networkSuffix
+
+        if groupList:
+            self.groupList = groupList
+        else:
+            # Copy the class default's list so as to not modify the original.
+            self.groupList = self.groupList[:]
+
+        if errorGroup:
+            self.errorGroup = errorGroup
+
+        self.attachToWords()
+
+    def setIrc(self, ircProtocol):
+        self.irc = ircProtocol
+        self.irc.realname =  'Tendril to %s' % (self.service.serviceName,)
+        self.irc.words = self
+
+    def setupBot(self, perspective):
+        self.perspective = perspective
+        self.joinGroup(self.errorGroup)
+
+    def attachToWords(self):
+        """Get my perspective on the Words service; attach as a client.
+        """
+
+        self.service.addBot(self.perspectiveName, self)
+
+        # XXX: Decide how much of this code belongs in words..Service.addBot
+#         try:
+#             self.perspective = (
+#                 self.service.getPerspectiveNamed(self.perspectiveName))
+#         except wordsService.UserNonexistantError:
+#             self.perspective = (
+#                 self.service.createParticipant(self.perspectiveName))
+#             if not self.perspective:
+#                 raise RuntimeError, ("service %s won't give me my "
+#                                      "perspective named %s"
+#                                      % (self.service,
+#                                         self.perspectiveName))
+
+#         if self.perspective.client is self:
+#             log.msg("I seem to be already attached.")
+#             return
+
+#         try:
+#             self.attach()
+#         except error.Unauthorized:
+#             if self.perspective.client:
+#                 log.msg("%s is attached to my perspective: "
+#                         "kicking it off." % (self.perspective.client,))
+#                 self.perspective.detached(self.perspective.client, None)
+#                 self.attach()
+#             else:
+#                 raise
+
+
+
     ### WordsClient methods
     ##      Words.Group --> IRC
 
@@ -533,14 +678,14 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         """
         if (group == self.errorGroup) or self.isThisMine(member):
             return
-        self.say(groupToChannelName(group), "%s joined." % (member,))
+        self.irc.say(groupToChannelName(group), "%s joined." % (member,))
 
     def memberLeft(self, member, group):
         """Tell the IRC Channel when someone leaves the Words group.
         """
         if (group == self.errorGroup) or self.isThisMine(member):
             return
-        self.say(groupToChannelName(group), "%s left." % (member,))
+        self.irc.say(groupToChannelName(group), "%s left." % (member,))
 
     def receiveGroupMessage(self, sender, group, message, metadata=None):
         """Pass a message from the Words group on to IRC.
@@ -558,7 +703,7 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
                     c = irc.ctcpExtract(message)
                     for tag, data in c['extended']:
                         if tag == 'ACTION':
-                            self.say(channel, "* %s %s" % (sender, data))
+                            self.irc.say(channel, "* %s %s" % (sender, data))
                         else:
                             # Not an action.  Repackage the chunk,
                             msg = "%(X)s%(tag)s %(data)s%(X)s" % {
@@ -572,7 +717,7 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
                             c['normal'].append(msg)
 
                     for msg in c['normal']:
-                        self.say(channel, "<%s> %s" % (sender, msg))
+                        self.irc.say(channel, "<%s> %s" % (sender, msg))
                     return
 
                 elif irc.X_DELIM in message:
@@ -580,10 +725,10 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
 
                 if metadata and metadata.has_key('style'):
                     if metadata['style'] == "emote":
-                        self.say(channel, "* %s %s" % (sender, message))
+                        self.irc.say(channel, "* %s %s" % (sender, message))
                         return
 
-                self.say(channel, "<%s> %s" % (sender, message))
+                self.irc.say(channel, "<%s> %s" % (sender, message))
         else:
             # A message in our errorGroup.
             if message == "participants":
@@ -632,13 +777,16 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
             return
 
         for g in self.perspective.groups:
-            self.leaveGroup(g.name)
+            if g.name != self.errorGroup:
+                self.leaveGroup(g.name)
         for nick in self.participants.keys()[:]:
             self.logoutParticipant(nick)
         self.perspective.detached(self, None)
         if self.transport and getattr(self.transport, 'connected'):
+            self.ircFactory.doStop()
             self.transport.loseConnection()
 
+
     ### Participant event methods
     ##      Words.Participant --> IRC
 
@@ -650,7 +798,7 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
                 # Damn.  What am I supposed to do with this?
                 message = "[%s] %s" % (metadata['style'], message)
 
-            self.msg(toNick, '<%s> %s' % (sender, message))
+            self.irc.msg(toNick, '<%s> %s' % (sender, message))
         else:
             # If there is a CTCP delimeter at the beginning of the
             # message, let's leave it there to accomidate not-so-
@@ -665,13 +813,39 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
             else:
                 desc = "CTCP request"
 
-            self.msg(toNick, 'The following %s is from %s'
+            self.irc.msg(toNick, 'The following %s is from %s'
                      % (desc, sender))
-            self.msg(toNick, '%s' % (message,))
+            self.irc.msg(toNick, '%s' % (message,))
 
 
     # IRC Participant Management
 
+    def ircConnectionLost(self):
+        for nick in self.participants.keys()[:]:
+            self.logoutParticipant(nick)
+
+    def ircPartParticipant(self, nick, groupName):
+        participant = self._getParticipant(nick)
+        try:
+            participant.leaveGroup(groupName)
+        except wordsService.NotInGroupError:
+            pass
+
+        if not participant.groups:
+            self.logoutParticipant(nick)
+
+    def ircParticipantMsg(self, nick, groupName, message, emote=False):
+        participant = self._getParticipant(nick)
+        if emote:
+            metadata = {'style': 'emote'}
+        else:
+            metadata = None
+        try:
+            participant.groupMessage(groupName, message, metadata)
+        except wordsService.NotInGroupError:
+            participant.joinGroup(groupName)
+            participant.groupMessage(groupName, message, metadata)
+
     def evacuateGroup(self, groupName):
         """Pull all of my Participants out of this group.
         """
@@ -707,8 +881,7 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
 
     def _newParticipant(self, nick):
         try:
-            p = self.service.getPerspectiveNamed(nick +
-                                                 self.networkSuffix)
+            p = self.service.getPerspectiveNamed(nick + self.networkSuffix)
         except wordsService.UserNonexistantError:
             p = self.service.createParticipant(nick + self.networkSuffix)
             if not p:
@@ -718,7 +891,9 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
                                               (nick, self.networkSuffix))
 
         c = ProxiedParticipant(self, nick)
-        p.attached(c, None)
+        p.attached(LocalAsyncForwarder(c, wordsService.IWordsClient, 1),
+                   None)
+        # p.attached(c, None)
 
         self.participants[nick] = [p, c]
 
@@ -742,7 +917,7 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         # XXX: This must change if we ever start giving people 'real'
         #    perspectives!
         if not p.identityName:
-            del self.service.participants[p.name]
+            self.service.uncachePerspective(p)
         del self.participants[nick]
 
     def isThisMine(self, sender):
@@ -757,20 +932,6 @@ class TendrilClient(irc.IRCClient, wordsService.WordsClientInterface):
         return 0
 
 
-    ### Utility
-
-    def log(self, message, priority=None):
-        """I need to give Twisted a prioritized logging facility one of these days.
-        """
-        if _LOGALL:
-            log.msg(message)
-        elif not (priority in ('dump',)):
-            log.msg(message)
-
-        if priority in ('info', 'NOTICE', 'ERROR'):
-            self.groupMessage(self.errorGroup, message)
-
-
 def channelToGroupName(channelName):
     """Map an IRC channel name to a Words group name.
 
